/*****   Configuration data for all elliptic fibrations with a \mathcal{S} bad fibre - mch ****** 
1) Compute the configuration data for all genus 1 fibrations of S with a Kodaira bad fibre
   supported on our chosen set of 34 (-2)-curves, \mathcal{S}.
2) Find the equivalence classes of these up to w -> -w and a -> -a automorphisms.
3) Compute which have sections/sections in \mathcal{S}, so give elliptic fibrations.

- this is a continuation from the defns_and_basic_properties.m file, which should be run first,
  and also uses the intrinsics in intersection_matrix.m. The notes in defns_and_basic_properties.m
  about the Magma version and labelling of the curves in \mathcal{S} still apply.
***********************************************************************************************/

//Attach("intersection_matrix.m");

// 1)

/*
 A Kodaira fibre (supported on \mathcal{S}) is given in the form 
  <seq,label>
  where seq is a SeqEnum containing the index of the constituent curves in the ordered
  sequence of \mathcal{S}, which are given in a natural order for the fibre type (see below),
  and label is a string describing the fibre type.
[
 Recall: the order that we take the 34 curves in \mathcal{S} is
L1, L2, .. L10, L1', .. L10', E1, E_2,1, E_2,2, E_2,3, E_3,1, E_3,2, E_4,1, .. E_4,4, E_5,1, .. E_5,4
]
 Labels are 
  1) "In" (e.g. "I2", "I4") for an I_n configuration
  2) "Dn" (e.g. "D4","D7") for an extended D_n configuration
  3) "E6","E7","E8" for extended E_6,E_7,E_8 configurations.

The ordering of curves in a seq of a given type is as follows is as follows:
In: [a1,a2,..,an] - the cycle of curves have curve a1 linked to curve a2, linked to curve a3, .., linked to curve an,
linked to curve a1.
Dn: [a1,a2,..,a(n+1)] - the central axis of the extended Dn diagram is a3 - a4 - ..- a(n-1) and a1, a2  meet
a3 and an,a(n+1) meet a(n-1).
E6: [a1,a2,..,a7] - In the extended Dynkin diagram, a1 is the central node and a2-a3, a4-a5, a6-a7 are the arms
 with a2, a4 and a6 meeting a1.
E7: [a1,..,a8] - In the extended Dynkin diagram, a1 - a2 - .. a6 - a7 is the main axis with a8 meeting a4.
E8: [a1,..,a9] - In the extended Dynkin diagram, a1 - a2 - .. a7 - a8 is the main axis with a9 meeting a3.

For example,
<[ 11, 29, 30, 16, 32, 25, 26 ], "E6">
represents an extended E6 fibre which is
  3*L1'(curve 11)+2*E_4,3(curve 29)+2*L6'(curve 16)+2*E_3,2(curve 25)+E_4,4(curve 30)+E_5,2(curve 32)+E_3,3(curve 26)
as an effective divisor on S.

A genus 1 fibration configuration, as returned by intrinsic KodairaFibs(imat) is a sequence
[<seq1,label1>,<seq2,label2>,..,<seqn,labeln>]
which gives n DISJOINT (so giving the same genus 1 fibration) Kodaira fibres. It is maximal
if there are no other Kodaira fibres supported on \mathcal{S} disjoint from these.

KodairaFibs(imat); returns the full set of Kodaira fibres supported on \mathcal{S}, divided
into maximal genus 1 fibration configurations. Thus it returns a sequence of the full
collection of (inequivalent) genus 1 fibrations with at least one bad fibre supported on
 \mathcal{S}. 
*/

time fibs :=  KodairaFibs(imat);
/*
Getting simple paths...
Time: 21.870
Getting In loops...
Time: 4.020
Getting extended Dn configs...
Time: 11.110
Getting extended E6 configs...
Time: 0.560
Getting extended E7 configs...
Time: 0.490
Getting extended E8 configs...
Time: 1.290
Getting disjoint sets of configurations...
Dealing with 105856 configs.
Done 1000 configs
Done 2000 configs
Done 3000 configs
Done 4000 configs
...
Done 104001 configs
Done 105001 configs
Time: 636.310
*/

#fibs;
//104600 - total number of (inequivalent) genus 1 fibrations found

// Consider the number of distinct bad fibres supported entirely on
// \mathcal{S} for the fibrations in fibs
Seqset([#f : f in fibs]);
// { 1, 2, 3, 4, 5 }

// Which configurations have 5 bad fibres supported on \mathcal{S}?
// Only 1!
[i : i in [1..#fibs] | #(fibs[i]) eq 5];
// [ 104118 ]

fibs[104118];
/*
[
    <[ 21, 16, 22, 24, 23, 6 ], "I6">,
    <[ 17, 27, 29, 30, 28, 7 ], "I6">,
    <[ 15, 33, 34, 5, 26, 25 ], "I6">,
    <[ 4, 14 ], "I2">,
    <[ 3, 13 ], "I2">
]
- this fibration has 3 I6 and 2 I2 bad fibres supported on \mathcal{S}
*/

// Check that a "D7" configuration has the right intersection matrix:
f := fibs[88420];
f;
/*
[
    <[ 6, 19, 23, 8, 28, 10, 22, 26 ], "D7">
]
*/
config := f[1][1];
// [ 6, 19, 23, 8, 28, 10, 22, 26 ]
Submatrix(imat,config,config);
/*
[-2  0  1  0  0  0  0  0]
[ 0 -2  1  0  0  0  0  0]
[ 1  1 -2  1  0  0  0  0]
[ 0  0  1 -2  1  0  0  0]
[ 0  0  0  1 -2  1  0  0]
[ 0  0  0  0  1 -2  1  1]
[ 0  0  0  0  0  1 -2  0]
[ 0  0  0  0  0  1  0 -2]
*/

// 2)
/*
   Here, we divide the configurations up into equivalence classes under the V4 group G of 
   S automorphisms <w -> -w, a -> -a> [recall: a=sqrt(5)] which preserves \mathcal{S},
   so preserves the set of genus 1 fibrations given by fibs. We take one representative
   of each equivalence class.

   We also divide this set of classes up into 5 subsets fib1reps,..,fib5reps depending
   on how many \mathcal{S}-supported bad fibres they have and write them out to 3
   files DY_fib1reps, DY_fib2reps, DY_fibge3reps in a slightly different format:
   the labels are stripped and the <seq,label> pairs and fibration configurations
   [<seq1,label1>,<seq2,label2>,..,<seqn,labeln>]
   where {label1,..,labeln} are the same are bundled together into a sequence
   fn_label1_label2_.._labeln := [<seq1,seq2,..,seqn>,...];
   (where label1,..,labeln are taken in a particular order), using seq1
   rather than <seq1> when n=1.

   This breaking up into {label1,..,labeln}-types also helps in computing G-equiv.
   classes as the G action preserves the (set of labels)-type.
*/

// split (non-G-equivalence-class) fibs into size subsets
fibs1 := [*f : f in fibs|#f eq 1*];
fibs2 := [*f : f in fibs|#f eq 2*];
fibs3 := [*f : f in fibs|#f eq 3*];
fibs4 := [*f : f in fibs|#f eq 4*];
fibs5 := [*f : f in fibs|#f eq 5*];
#fibs1; #fibs2; #fibs3; #fibs4; #fibs5;
/*
103404
1143
47
5
1
*/

// permutations of w->-w, a->-a on the 34 curves
S34 := SymmetricGroup(34);
pw := S34!([11..20] cat [1..10] cat [21,23,22,24,26,25,28,27,30,29,32,31,34,33]);
pa := S34!([1,2,4,3] cat [5..8] cat [10,9] cat [11,12,14,13] cat [15..18] cat [20,19] cat [21..34]);
Gs := [pw,pa,pw*pa];

// 1-fibre cases
typs1 := Setseq({f[1][2] : f in fibs1});
f1typs := [[f[1][1] : f in fibs1 | f[1][2] eq t] : t in typs1];
f1typsets := [[Seqset(e) : e in f] : f in f1typs];
f1reps := [];
for i in [1..#typs1] do
  sts := f1typsets[i];
  inds := {@j : j in [1..#sts]@};
  reps := [];
  while #inds gt 0 do
    j := inds[1];
    js := [j] cat [Index(sts,{k^g : k in sts[j]}): g in Gs];
    js := Seqset(js);
    Append(~reps,Sort(Setseq(js)));
    inds diff:= IndexedSet(js);
  end while;
  Append(~f1reps,reps);
end for;
// this last for loop takes a minute or two to run
fib1reps := [[f[u[1]] : u in f1reps[i]] where f is f1typs[i] : i in [1..#typs1]];

// typs1 contains the different labels for the single fibre configurations
typs1;
/*
[ D14, E7, I10, I8, D5, I16, E8, I11, I9, D10, I4, D6, D16, I12, I5, D7, I13, 
D12, I6, D8, I14, E6, I7, D9, D4 ]
*/
// fib1reps is a sequence of #typs1 sequences where fib1reps[i] is the sequence
// of 1-fibre configurations of type typs1[i]
[#f : f in fib1reps];
/*
[ 280, 7215, 273, 161, 463, 43, 13108, 104, 132, 960, 7, 829, 18, 107, 12, 1037,
68, 479, 48, 885, 96, 1651, 132, 482, 90 ]
- the number of G-equivalence classes of 1-fibre configurations of each type
*/
&+($1);
// 28680 - the total number of G-equiv classes of single fibre configs
//Example:
typs1[11];
// I4
fib1reps[11];
/*
[
    [ 8, 21, 9, 28 ],
    [ 9, 21, 10, 15 ],
    [ 17, 10, 21, 18 ],
    [ 8, 23, 20, 21 ],
    [ 21, 10, 28, 9 ],
    [ 15, 8, 21, 10 ],
    [ 8, 21, 20, 25 ]
]
*/

// Print the results to file DY_fib1reps:
F := Open("DY_fib1reps","w");
fprintf F,"f1typs := [ " cat &cat["\"" cat typs1[i] cat "\",  " : i in [1..#typs1-1]]
  cat "\"" cat typs1[#typs1] cat "\" ];\n";
for i in [1..#typs1] do
  fprintf F,"//%o:\nf1_%o :=\n",typs1[i],typs1[i];
  fprintf F,"%o",fib1reps[i];
  fprintf F,";\n";
end for;
fprintf F,"f1s := [ " cat &cat["f1_" cat typs1[i] cat ", " : i in [1..#typs1-1]]
  cat "f1_" cat typs1[#typs1] cat " ];\n";
delete F;
/*
Start of DY_fib1reps: 

f1typs := [ "D14",  "E7",  "I10",  "I8",  "D5",  "I16",  "E8",  "I11",  "I9",  "D10",  "I4",  "D6",  "D16",  "I12",  "I5",  "D7",  "I13",  "D12",  "I6",  "D8",  "I14",  "E6",  "I7",  "D9",  "D4" ];
//D14:
f1_D14 :=
[
[ 3, 4, 18, 26, 25, 15, 33, 31, 6, 23, 24, 2, 28, 30, 7 ],
[ 14, 6, 31, 33, 34, 32, 3, 12, 27, 20, 25, 26, 9, 22, 28 ],
[ 11, 2, 13, 31, 6, 23, 19, 7, 32, 4, 12, 15, 10, 22, 26 ],
[ 17, 33, 31, 13, 11, 29, 30, 1, 4, 12, 24, 23, 19, 5, 21 ],
[ 29, 1, 30, 28, 7, 20, 21, 16, 22, 24, 12, 15, 33, 31, 34 ],
[ 4, 3, 12, 24, 23, 6, 31, 17, 7, 28, 30, 29, 11, 16, 25 ],
[ 6, 17, 31, 33, 15, 8, 3, 32, 16, 22, 24, 2, 5, 20, 26 ],
[ 19, 20, 23, 24, 12, 15, 33, 31, 17, 18, 26, 1, 30, 28, 29 ],
[ 17, 33, 31, 14, 2, 28, 30, 1, 3, 32, 16, 21, 20, 23, 25 ],
[ 26, 22, 18, 4, 12, 15, 33, 31, 6, 23, 19, 7, 28, 2, 30 ],
[ 1, 29, 30, 28, 2, 24, 22, 16, 21, 19, 25, 15, 33, 31, 34 ],
[ 3, 7, 8, 15, 33, 34, 5, 2, 24, 22, 16, 11, 29, 30, 27 ],
[ 14, 13, 18, 5, 34, 33, 15, 12, 24, 23, 6, 1, 30, 28, 29 ],
[ 14, 13, 11, 25, 26, 5, 34, 32, 7, 17, 27, 12, 24, 22, 23 ],
[ 27, 30, 29, 11, 25, 8, 21, 9, 22, 24, 2, 5, 34, 32, 33 ],
[ 2, 23, 24, 12, 3, 1, 30, 29, 11, 16, 21, 9, 17, 7, 31 ],
[ 22, 12, 24, 2, 5, 19, 25, 11, 29, 30, 1, 6, 31, 17, 33 ],
[ 2, 22, 24, 12, 27, 19, 7, 32, 34, 33, 31, 6, 1, 30, 26 ],
[ 12, 29, 27, 20, 25, 26, 9, 22, 16, 32, 34, 33, 31, 6, 14 ],
[ 3, 4, 1, 26, 5, 2, 28, 7, 17, 27, 29, 11, 16, 21, 22 ],
[ 17, 33, 31, 13, 11, 29, 30, 1, 26, 5, 19, 23, 24, 12, 22 ],
[ 16, 10, 22, 24, 2, 5, 34, 33, 31, 6, 1, 4, 8, 7, 25 ],
...

End of file DY_fib1reps:
...
[ 21, 17, 18, 4, 22 ],
[ 21, 4, 18, 3, 27 ],
[ 4, 21, 18, 3, 17 ],
[ 8, 10, 21, 9, 16 ],
[ 6, 20, 21, 19, 8 ],
[ 6, 19, 21, 10, 8 ],
[ 25, 21, 8, 4, 28 ],
[ 21, 13, 18, 4, 26 ],
[ 21, 13, 18, 4, 27 ],
[ 23, 21, 8, 4, 25 ],
[ 23, 21, 8, 14, 25 ],
[ 13, 25, 8, 4, 23 ],
[ 6, 18, 21, 10, 9 ],
[ 21, 13, 18, 4, 22 ],
[ 13, 25, 8, 14, 23 ]
];
f1s := [ f1_D14, f1_E7, f1_I10, f1_I8, f1_D5, f1_I16, f1_E8, f1_I11, f1_I9, f1_D10, f1_I4, f1_D6, f1_D16, f1_I12, f1_I5, f1_D7, f1_I13, f1_D12, f1_I6, f1_D8, f1_I14, f1_E6, f1_I7, f1_D9, f1_D4 ];

----
Loading the file into Magma gives f1typs, the sequence of type labels for the 1-fibre configs,
& and sequence f1s, s.t. f1s[i] gives the sequence of G-representatives of f1typs[i] configs.
*/

// 2-fibre cases
typs2 := Setseq({[f[1][2],f[2][2]] : f in fibs2});
f2typs := [[<f[1][1],f[2][1]> : f in fibs2 | [f[1][2],f[2][2]] eq t] : t in typs2];
f2typsets := [[<Seqset(e[1]),Seqset(e[2])> : e in f] : f in f2typs];
f2reps := [];
for i in [1..#typs2] do
  sts := f2typsets[i];
  sts1 := [e[1] : e in sts]; sts2 := [e[2] : e in sts];
  inds := {@j : j in [1..#sts]@};
  reps := [];
  while #inds gt 0 do
    j := inds[1];
    js := [j];
    for g in Gs do
      s1 := {k^g : k in sts1[j]}; s2 := {k^g : k in sts2[j]};
      if s1 eq sts1[j] then
	assert s2 eq sts2[j];
      elif s2 eq sts1[j] then
	assert s1 eq sts2[j];
      else
	i1 := Index(sts1,s1);
	if i1 eq 0 then
	  i1 := Index(sts2,s1);
	  assert (i1 ne 0) and (sts1[i1] eq s2);
	else
	  assert sts2[i1] eq s2;
	end if;
	Append(~js,i1);
      end if; 
    end for;
    js := Seqset(js);
    Append(~reps,Sort(Setseq(js)));
    inds diff:= IndexedSet(js);
  end while;
  Append(~f2reps,reps);
end for;
fib2reps := [[f[u[1]] : u in f2reps[i]] where f is f2typs[i] : i in [1..#typs2]];

typs2;
/*
[
    [ D7, E6 ],
    [ I6, D4 ],
    [ D7, I4 ],
    [ D6, D4 ],
    [ I7, I5 ],
    [ D4, I4 ],
    [ D5, I4 ],
    [ I6, I4 ],
    [ E7, E7 ],
    [ I8, I4 ],
    [ E7, D6 ],
    [ D8, D6 ],
    [ D5, D5 ],
    [ E6, D5 ],
    [ D6, D6 ],
    [ D8, E7 ],
    [ E6, E6 ],
    [ I8, D6 ],
    [ D5, I6 ],
    [ I6, I6 ],
    [ I10, D6 ],
    [ I8, E6 ],
    [ I12, I4 ],
    [ E7, D4 ],
    [ D8, D4 ],
    [ I8, D7 ],
    [ I10, D7 ],
    [ I8, D4 ],
    [ I9, I5 ],
    [ I10, D4 ],
    [ I8, I8 ],
    [ I12, D5 ],
    [ I8, D5 ],
    [ I10, D5 ],
    [ D6, I6 ],
    [ I7, I7 ],
    [ I10, I6 ],
    [ E8, E8 ],
    [ I9, I7 ],
    [ D10, D4 ]
]
*/
[#f : f in fib2reps];
/*
[ 26, 1, 10, 6, 10, 2, 2, 2, 27, 6, 13, 25, 12, 34, 13, 48, 12, 14, 12, 3, 4, 9,
4, 11, 2, 6, 4, 2, 6, 12, 2, 5, 12, 2, 9, 10, 4, 12, 8, 7 ]
- the number of G-equiv. classes of 2-fibre configs of each typs2 type
*/
&+($1);
// 409 - the total number of G-equiv classes of 2-fibre configs
//Example:
typs2[4];
// [ D6, D4 ] - maximal genus 1 fibration configurations with an extended D6 and
// an extended D4 fibre supported on \mathcal{S}
fib2reps[4];
/*
[
    <[ 5, 33, 34, 32, 3, 1, 12 ], [ 17, 28, 9, 21, 22 ]>,
    <[ 2, 8, 28, 30, 1, 6, 26 ], [ 17, 19, 27, 12, 20 ]>,
    <[ 27, 30, 29, 11, 14, 2, 31 ], [ 15, 22, 9, 21, 26 ]>,
    <[ 4, 34, 32, 16, 11, 29, 13 ], [ 26, 17, 10, 15, 28 ]>,
    <[ 7, 16, 32, 34, 5, 2, 18 ], [ 12, 10, 15, 9, 25 ]>,
    <[ 30, 7, 28, 10, 15, 25, 33 ], [ 5, 27, 18, 4, 13 ]>
]
*/
// Print the results to file DY_fib2reps:
F := Open("DY_fib2reps","w");
fprintf F,"f2typs := [ " cat &cat["[ \"" cat typs2[i][1] cat "\", \"" cat typs2[i][2] cat "\" ], " : i in [1..#typs2-1]]
  cat "[ \"" cat typs2[#typs2][1] cat "\", \"" cat typs2[#typs2][2] cat "\" ] ];\n";
for i in [1..#typs2] do
  fprintf F,"//%o:\nf2_%o_%o :=\n",typs2[i],typs2[i][1],typs2[i][2];
  fprintf F, "%o",fib2reps[i];
  fprintf F,";\n";
end for;
fprintf F,"f2s := [ " cat &cat["f2_" cat typs2[i][1] cat "_" cat typs2[i][2] cat ", " : i in [1..#typs2-1]]
  cat "f2_" cat typs2[#typs2][1] cat "_" cat typs2[#typs2][2] cat " ];\n";
delete F;
/*
Start of DY_fib2reps: 

f2typs := [ [ "D7", "E6" ], [ "I6", "D4" ], [ "D7", "I4" ], [ "D6", "D4" ], [ "I7", "I5" ], [ "D4", "I4" ], [ "D5", "I4" ], [ "I6", "I4" ], [ "E7", "E7" ], [ "I8", "I4" ], [ "E7", "D6" ], [ "D8", "D6" ], [ "D5", "D5" ], [ "E6", "D5" ], [ "D6", "D6" ], [ "D8", "E7" ], [ "E6", "E6" ], [ "I8", "D6" ], [ "D5", "I6" ], [ "I6", "I6" ], [ "I10", "D6" ], [ "I8", "E6" ], [ "I12", "I4" ], [ "E7", "D4" ], [ "D8", "D4" ], [ "I8", "D7" ], [ "I10", "D7" ], [ "I8", "D4" ], [ "I9", "I5" ], [ "I10", "D4" ], [ "I8", "I8" ], [ "I12", "D5" ], [ "I8", "D5" ], [ "I10", "D5" ], [ "D6", "I6" ], [ "I7", "I7" ], [ "I10", "I6" ], [ "E8", "E8" ], [ "I9", "I7" ], [ "D10", "D4" ] ];
//[ D7, E6 ]:
f2_D7_E6 :=
[
<[ 2, 12, 24, 23, 6, 31, 17, 33 ], [ 11, 29, 30, 16, 32, 25, 26 ]>,
<[ 23, 21, 6, 31, 33, 15, 12, 25 ], [ 28, 2, 5, 7, 32, 30, 29 ]>,
<[ 31, 1, 6, 21, 16, 32, 7, 34 ], [ 12, 24, 2, 15, 25, 27, 29 ]>,
<[ 23, 21, 6, 1, 4, 32, 7, 34 ], [ 11, 13, 2, 29, 27, 25, 15 ]>,
<[ 23, 22, 24, 2, 13, 11, 29, 25 ], [ 32, 4, 1, 34, 33, 7, 17 ]>,
<[ 21, 23, 6, 31, 13, 11, 29, 25 ], [ 32, 4, 12, 7, 28, 34, 5 ]>,
<[ 17, 33, 31, 14, 2, 24, 22, 23 ], [ 1, 30, 29, 3, 32, 26, 25 ]>,
<[ 23, 21, 6, 31, 17, 27, 12, 29 ], [ 5, 2, 28, 34, 32, 26, 25 ]>,
<[ 27, 15, 12, 24, 23, 6, 1, 31 ], [ 32, 34, 5, 16, 11, 7, 28 ]>,
<[ 22, 21, 16, 32, 4, 12, 15, 27 ], [ 2, 5, 26, 13, 31, 28, 30 ]>,
...

End of file DY_fib2reps:
...
//[ I9, I7 ]:
f2_I9_I7 :=
[
<[ 16, 11, 13, 31, 33, 15, 12, 24, 22 ], [ 26, 5, 19, 7, 28, 30, 1 ]>,
<[ 4, 32, 34, 33, 31, 6, 23, 24, 12 ], [ 11, 29, 30, 28, 10, 26, 25 ]>,
<[ 2, 24, 23, 6, 1, 30, 29, 11, 14 ], [ 15, 33, 34, 32, 7, 17, 9 ]>,
<[ 24, 22, 16, 32, 3, 1, 30, 28, 2 ], [ 15, 33, 31, 17, 27, 20, 25 ]>,
<[ 2, 24, 23, 6, 31, 33, 15, 9, 28 ], [ 11, 16, 32, 3, 18, 27, 29 ]>,
<[ 22, 24, 12, 27, 29, 30, 1, 26, 10 ], [ 7, 32, 34, 33, 31, 13, 8 ]>,
<[ 1, 6, 23, 24, 12, 15, 9, 28, 30 ], [ 18, 14, 11, 16, 32, 34, 5 ]>,
<[ 15, 12, 24, 23, 20, 7, 32, 34, 33 ], [ 1, 30, 29, 11, 14, 18, 26 ]>
];
//[ D10, D4 ]:
f2_D10_D4 :=
[
<[ 12, 17, 27, 29, 30, 1, 26, 5, 34, 32, 33 ], [ 13, 21, 8, 14, 23 ]>,
<[ 27, 30, 29, 11, 16, 22, 24, 2, 5, 26, 34 ], [ 7, 4, 8, 3, 15 ]>,
<[ 12, 25, 15, 33, 34, 32, 7, 28, 30, 1, 29 ], [ 21, 14, 18, 13, 22 ]>,
<[ 12, 2, 24, 22, 16, 32, 34, 33, 31, 6, 17 ], [ 19, 26, 25, 8, 20 ]>,
<[ 5, 33, 34, 32, 16, 22, 24, 12, 27, 17, 29 ], [ 13, 25, 8, 14, 28 ]>,
<[ 11, 30, 29, 27, 12, 24, 2, 5, 34, 32, 33 ], [ 6, 10, 21, 8, 9 ]>,
<[ 6, 26, 1, 30, 29, 11, 16, 22, 24, 2, 12 ], [ 8, 20, 7, 19, 17 ]>
];
f2s := [ f2_D7_E6, f2_I6_D4, f2_D7_I4, f2_D6_D4, f2_I7_I5, f2_D4_I4, f2_D5_I4, f2_I6_I4, f2_E7_E7, f2_I8_I4, f2_E7_D6, f2_D8_D6, f2_D5_D5, f2_E6_D5, f2_D6_D6, f2_D8_E7, f2_E6_E6, f2_I8_D6, f2_D5_I6, f2_I6_I6, f2_I10_D6, f2_I8_E6, f2_I12_I4, f2_E7_D4, f2_D8_D4, f2_I8_D7, f2_I10_D7, f2_I8_D4, f2_I9_I5, f2_I10_D4, f2_I8_I8, f2_I12_D5, f2_I8_D5, f2_I10_D5, f2_D6_I6, f2_I7_I7, f2_I10_I6, f2_E8_E8, f2_I9_I7, f2_D10_D4 ];

----
Loading the file into Magma gives f2typs and f2s, as for the 1-fibre case.
*/

// 3-,4- and 5-fibre cases

// These are relatively small in number and we show the full results rather than printing to a file.
// Having split into the various types, we just found the G-equivalence classes 'by hand'

 //3-case
typs3 := Setseq({[f[1][2],f[2][2],f[3][2]] : f in fibs3});
typs3;
/*
[
    [ E6, E6, I4 ],
    [ D6, D6, D4 ],
    [ I7, I7, I3 ],
    [ I6, D4, I4 ],
    [ D5, D5, I6 ],
    [ I6, I6, D4 ],
    [ I6, I6, I4 ],
    [ I7, I5, I5 ],
    [ I8, D5, I4 ],
    [ D5, D5, I4 ]
]
*/
f3typs := [[<f[1][1],f[2][1],f[3][1]> : f in fibs3 | 
   [f[1][2],f[2][2],f[3][2]] eq t] : t in typs3];
[#f :f in f3typs];
//[ 2, 6, 4, 8, 4, 3, 4, 8, 6, 2 ]
f3reps := [ [[1,2]],[[1,2],[3,4],[5,6]],[[1,2],[3,4]],[[1,3,4,5],[2,6,7,8]],
  [[1,2,3,4]],[[1],[2,3]],[[1,2,3,4]],[[1,2,3,5],[4,6,7,8]],[[1,4],[2,3],[5,6]],
  [[1,2]] ];
fib3reps := [[f[u[1]] : u in f3reps[i]] where f is f3typs[i] : i in [1..#typs3]];
fib3reps;
/*
[
    [
        <[ 33, 15, 12, 31, 17, 34, 32 ], [ 30, 28, 2, 29, 11, 1, 26 ], [ 21, 19,
        23, 20 ]>
    ],
    [
        <[ 26, 34, 5, 2, 28, 7, 30 ], [ 22, 11, 16, 21, 6, 31, 23 ], [ 3, 27, 
        12, 15, 4 ]>,
        <[ 26, 15, 25, 11, 29, 27, 30 ], [ 22, 2, 24, 23, 6, 21, 31 ], [ 4, 7, 
        32, 3, 34 ]>,
        <[ 28, 17, 7, 32, 34, 5, 33 ], [ 12, 22, 24, 23, 6, 1, 21 ], [ 14, 25, 
        11, 13, 29 ]>
    ],
    [
        <[ 17, 31, 6, 1, 30, 29, 27 ], [ 2, 5, 34, 32, 16, 22, 24 ], [ 15, 25, 8
        ]>,
        <[ 6, 23, 24, 2, 28, 30, 1 ], [ 16, 11, 25, 15, 33, 34, 32 ], [ 18, 27, 
        17 ]>
    ],
    [
        <[ 14, 2, 24, 22, 16, 11 ], [ 26, 6, 1, 3, 30 ], [ 17, 7, 20, 27 ]>,
        <[ 14, 31, 6, 23, 24, 2 ], [ 16, 7, 32, 3, 34 ], [ 9, 26, 25, 15 ]>
    ],
    [
        <[ 13, 17, 31, 33, 15, 34 ], [ 4, 26, 1, 30, 28, 29 ], [ 22, 16, 21, 19,
        23, 24 ]>
    ],
    [
        <[ 32, 7, 17, 31, 33, 34 ], [ 11, 25, 26, 1, 30, 29 ], [ 22, 12, 24, 2, 
        23 ]>,
        <[ 28, 2, 5, 34, 32, 7 ], [ 11, 25, 15, 12, 27, 29 ], [ 23, 21, 6, 1, 31
        ]>
    ],
    [
        <[ 2, 13, 31, 33, 34, 5 ], [ 12, 4, 1, 30, 29, 27 ], [ 21, 10, 22, 16 ]>
    ],
    [
        <[ 31, 6, 1, 4, 32, 34, 33 ], [ 25, 19, 27, 29, 11 ], [ 10, 28, 2, 24, 
        22 ]>,
        <[ 11, 16, 32, 3, 1, 30, 29 ], [ 15, 9, 17, 31, 33 ], [ 5, 20, 23, 24, 2
        ]>
    ],
    [
        <[ 2, 28, 30, 29, 11, 25, 26, 5 ], [ 17, 33, 31, 6, 23, 21 ], [ 3, 12, 
        4, 32 ]>,
        <[ 31, 17, 7, 28, 2, 5, 34, 33 ], [ 22, 21, 16, 11, 29, 25 ], [ 3, 12, 
        4, 1 ]>,
        <[ 26, 5, 34, 32, 7, 28, 30, 1 ], [ 15, 27, 12, 24, 22, 23 ], [ 11, 14, 
        31, 13 ]>
    ],
    [
        <[ 26, 2, 5, 34, 32, 33 ], [ 11, 30, 29, 27, 12, 17 ], [ 6, 21, 8, 23 ]>
    ]
]
*/
typs4 := [ [ "I10", "I2", "I2", "I2" ], [ "I8", "D5", "I2", "I2" ] ];
fib4reps := [
 [
    <[ 11, 16, 32, 34, 33, 31, 6, 1, 30, 29 ], [ 10, 20 ], [ 9, 19 ], [ 8, 18 ]>
 ],
 [
    <[ 28, 2, 24, 22, 16, 11, 29, 30 ], [ 17, 6, 31, 33, 15, 34 ], [ 4, 20 ], [3, 19]>,
    <[ 22, 24, 12, 15, 33, 34, 32, 16 ], [ 6, 26, 1, 30, 28, 29 ], [ 14, 19 ], [ 13, 20 ]>
 ]
];
typs5 := [ [ "I6", "I6", "I6", "I2", "I2" ] ]; // only 1
fib5reps := [
 [
  <[ 21, 16, 22, 24, 23, 6 ], [ 17, 27, 29, 30, 28, 7 ], [ 15, 33, 34, 5, 26, 25 ],
     [ 4, 14 ], [ 3, 13 ]>
 ]
]; // only 1 fib5 class

&+[#f : f in fib3reps];
// 18
&+[#f : f in fib4reps];
// 3
&+[#f : f in fib5reps];
// 1

/*** Overall, have 28680+409+18+3+1=29111 G-classes of genus 1 fibrations  ****/

// A genus 1 fibration configuration type of particular interest is [E8,E8]=typs2[38].
// These give genus 1 fibrations with 2 distinct bad fibres of extended E8 type.
// Such fibrations lead to Shioda-Inose structures. There are 12 occuring here,
// one of which is the third fibration in the paper. We show below that
// none of these fibrations have a section, although they have 2-sections.
fib2reps[38];
/*
 [
    <[ 22, 16, 11, 14, 2, 5, 34, 33, 25 ], [ 23, 6, 1, 3, 12, 27, 17, 7, 30 ]>,
    <[ 21, 16, 32, 4, 1, 26, 25, 15, 34 ], [ 23, 24, 2, 13, 31, 17, 27, 29, 28 
    ]>,
    <[ 21, 16, 32, 3, 1, 30, 29, 27, 7 ], [ 23, 24, 2, 14, 31, 33, 15, 25, 5 ]>,
    <[ 22, 16, 32, 34, 5, 26, 25, 15, 4 ], [ 23, 6, 31, 17, 27, 29, 30, 28, 13 
    ]>,
    <[ 23, 6, 31, 14, 2, 5, 26, 25, 33 ], [ 22, 16, 32, 3, 12, 27, 29, 30, 7 ]>,
    <[ 23, 24, 12, 4, 1, 30, 28, 7, 27 ], [ 21, 16, 11, 13, 31, 33, 34, 5, 25 
    ]>,
    <[ 22, 24, 12, 27, 29, 30, 28, 7, 4 ], [ 21, 6, 31, 33, 34, 5, 26, 25, 13 
    ]>,
    <[ 21, 16, 32, 7, 17, 27, 29, 30, 4 ], [ 23, 24, 2, 5, 26, 25, 15, 33, 13 
    ]>,
    <[ 21, 16, 11, 14, 31, 17, 7, 28, 29 ], [ 23, 24, 12, 3, 1, 26, 5, 34, 15 
    ]>,
    <[ 23, 6, 1, 30, 28, 7, 17, 27, 4 ], [ 22, 16, 11, 25, 15, 33, 34, 5, 13 ]>,
    <[ 21, 6, 1, 30, 29, 27, 17, 7, 4 ], [ 22, 24, 2, 5, 34, 33, 15, 25, 13 ]>,
    <[ 21, 16, 11, 25, 26, 5, 34, 33, 14 ], [ 23, 24, 12, 27, 17, 7, 28, 30, 3 
    ]>
]
*/

// 3) Look for sections in \mathcal{S} or otherwise

function kf_mult(str,n)
// For a Kodaira fibre of type str and with n components
// give the multiplicity of the components in the fibre.
  case str[1]:
    when "I": return [1 : i in [1..n]];
    when "D": return [1,1] cat [2: i in [1..n-4]] cat [1,1];
    when "E": 
      if n eq 7 then return [3,2,1,2,1,2,1];
      elif n eq 8 then return [1,2,3,4,3,2,1,2];
      else return [2,4,6,5,4,3,2,1,3]; end if;
  end case;
end function;

// First consider the G-equiv. classes with a \mathcal{S} section.
// This is equivalent to a curve in \mathcal{S} having intersection
// number 1 with one (equiv. all) of the fibres of the fibration.
// Tested by the num_with_S_sec function, using imat.

function num_with_S_sec(fibs,typs)
// for a sequence of configs, fibs, with type typs,
// return the number with a \mathcal{S} section
cnt := 0;
boo := Type(fibs[1]) cmpeq SeqEnum; // 1-fibre config?
for f in fibs do
  s := boo select f else f[1];
  m := kf_mult(typs[1],#s);
  for i in {1..34} diff Seqset(s) do
    if &+[imat[i,s[j]]*m[j] : j in [1..#s]] eq 1 then
      cnt +:= 1; break;
    end if;
  end for;
end for;
return cnt;
end function;

//For fibs1
[num_with_S_sec(fib1reps[i],typs1[i]) : i in [1..#typs1]];
/*
[ 162, 6775, 267, 161, 461, 31, 9166, 104, 132, 859, 7, 829, 0, 90, 12, 985, 68,
443, 48, 863, 96, 1603, 132, 482, 90 ]
*/
&+($1);
// 23866 - number of the 28680 fib1 G-classes with a \mathcal{S}-section

//For fibs2
[num_with_S_sec(fib2reps[i],typs2[i]) : i in [1..#typs2]];
/*
[ 26, 1, 10, 6, 10, 2, 2, 2, 27, 6, 13, 25, 12, 34, 13, 42, 8, 14, 12, 3, 4, 9, 
4, 11, 2, 6, 0, 2, 6, 12, 2, 4, 12, 2, 9, 10, 4, 0, 8, 7 ]
*/
&+($1);
// 382 - number of the 409 fib2 G-classes with a \mathcal{S}-section

//For fibs3
[num_with_S_sec(fib3reps[i],typs3[i]) : i in [1..#typs3]];
// [ 1, 3, 2, 2, 1, 2, 1, 2, 3, 1 ]
&+($1);
// 18 - number of the 18 fib3 G-classes with a \mathcal{S}-section. All of them!

//For fibs5
[num_with_S_sec(fib4reps[i],typs4[i]) : i in [1..#typs4]];
// [1, 2] - All 3 of them

//For fibs4
[num_with_S_sec(fib5reps[i],typs5[i]) : i in [1..#typs5]];
// [1] - the one class has a \mathcal{S}-section

// Now consider the G-equiv. classes with any section.
// This is equivalent to one (equiv. all) fibre F of the
// fibration having intersection number 1 with some class
// in Pic(Sbar). Since Pic is generated by \mathcal{S},
// this is equivalent to GCD([F.C : C in  \mathcal{S}])=1.
// Tested by the num_with_sec function, using imat.

function num_with_sec(fibs,typs)
// for a sequence of configs, fibs, with type typs,
// return the number with a section
cnt := 0;
boo := Type(fibs[1]) cmpeq SeqEnum; // 1-fibre config?
for f in fibs do
  s := boo select f else f[1];
  m := kf_mult(typs[1],#s);
  ints := [&+[imat[i,s[j]]*m[j] : j in [1..#s]] : i in {1..34} diff Seqset(s)]; 
  if GCD(ints) eq 1 then
    cnt +:= 1;
  end if; 
end for;
return cnt;
end function;

//For fibs1
[num_with_sec(fib1reps[i],typs1[i]) : i in [1..#typs1]];
/*
[ 240, 6955, 267, 161, 461, 31, 12370, 104, 132, 886, 7, 829, 0, 90, 12, 985, 
68, 479, 48, 871, 96, 1605, 132, 482, 90 ]
*/
&+($1);
// 27401 - number of the 28680 fib1 G-classes with a section

//For fibs2
[num_with_sec(fib2reps[i],typs2[i]) : i in [1..#typs2]];
/*
[ 26, 1, 10, 6, 10, 2, 2, 2, 27, 6, 13, 25, 12, 34, 13, 44, 8, 14, 12, 3, 4, 9, 
4, 11, 2, 6, 0, 2, 6, 12, 2, 4, 12, 2, 9, 10, 4, 0, 8, 7 ]
*/
&+($1);
// 384 - number of the 409 fib2 G-classes with a section

// Note that the 0 in the 38th entry of the last output says that none of
// the 12 [E8,E8] fibrations has a section.

// fibs3,fibs4 and fibs5 classes all have a section since they have a \mathcal{S}-section

/************************ Summary on sections **********************

Of the 29111 G-classes of fibration that we have found, 27807 have a section.
Of the 29111 G-classes of fibration that we have found, 24270 have a \mathcal{S}-section.

********************************************************************/
