/*****   First steps - mch ***********************************************************************
1) Construction of the Drell-Yan surface S as a singular surface scheme X in P(1,1,1,3).
2) Desingularise to get the exceptional divisors and define the Li, Cj divisors.
3) Compute the intersecion matrix of the set of 34 divisors in our special set \mathcal{S}.
4) Give a set of 19 divisors from S that form a free basis of Pic(Xbar) and express
   the other \mathcal{S} divisor classes and the hyperplane H divisor class in terms of these.

- Uses the intrinsics in file intersection_matrix.m. Note that this all requires Magma
  (patch) version V2.24-4 or higher since bugs in earlier versions caused errors in
  the desingularisation computations.

IMPORTANT NOTE:
The labelling of the divisors in \mathcal{S} used in these files differs from that of the paper.
This is slightly unfortunate, but to avoid making errors in changing many of the comments
in the code that refers to the divisors or trying to change the order of divisor multiplicities
in various sequences and vectors or in the intersection matrix, we have left the code as it is.

The following translates the labelling of divisors defined in Sections 3 and 4 of the paper to
the labelling here in the form (paper label) <-> (label here)

Li <-> Li, \tilde{Li} <-> Li'   1 <= i <= 7
C1 <-> L9, \tilde{C1} <-> L9'
C2 <-> L10, \tilde{C2} <-> L10'
C3 <-> L8, \tilde{C3} <-> L8'
E_1,1 <-> E_1,1
E_2,-1 <-> E_3,1 , E_2,1 <-> E_3,2
E_3,-1 <-> E_2,1 , E_3,0 <-> E_2,3 , E_3,1 <-> E_2,2
E_4,-2 <-> E_5,1 , E_4,-1 <-> E_5,3 , E_4,1 <-> E_5,4 , E_4,2 <-> E_5,2
E_5,-2 <-> E_4,1 , E_5,-1 <-> E_4,3 , E_5,1 <-> E_4,4 , E_5,2 <-> E_4,2
**********************************************************************************************/


//1)
K1<a>:=NumberField(Polynomial([-5,0,1])); // base field Q(a) - a^2=5
P1113<x,y,z,w>:=ProjectiveSpace(K1,[1,1,1,3]);
f6:=4*x*y^2*z*(x+z)^2+(x*(y+z)^2+y*(x+z)^2)*(x*(-y+z)^2+y*(z-x)^2);
X := Surface(P1113,w^2-f6); // S = (singular model of) Drell-Yan surface

// Set up singularity structures on S. Due to the weighting of
// the projective ambient, avoid internal functions and do directly
R2<a1,b,c> := PolynomialRing(K1,3,"grevlex");
sngs := ReducedSubscheme(SingularSubscheme(Scheme(Proj(R2),Evaluate(f6,[a1,b,c,0]))));
hm := hom<R2->CoordinateRing(P1113)|[x,y,z]>; // a1->x, b->y, c->z
X`Nonsingular := false;
X`SingularSubscheme := Scheme(X,[w] cat [hm(e) : e in DefiningPolynomials(sngs)]);

//2)
t1 := (-1+a)/2;
t2 := (-1-a)/2;

ls := [
    [x, w-y*z^2],
    [x-z,w-z*(y+z)^2],
    [y+z/2*(3-a),w-(a-3)/2*z*(x^2-x*z+z^2)],
    [y+z/2*(3+a),w-(-a-3)/2*z*(x^2-x*z+z^2)],
    [y, w-x*z^2],
    [y+z, w-z*(x-z)*(x+z)],
    [z,w-x*y*(x+y)],
    [x^2+y*z, x*y^2 - y^2*z - x*z^2 - y*z^2 - w],
    [x^2+t1*(x*y+x*z)+y*z, x*y^2 + 1/2*(a + 5)*x*y*z + 1/2*(3*a + 5)*y^2*z + 1/2*(a + 3)*x*z^2 + 1/2*(3*a + 5)*y*z^2 + 1/2*(a + 
        3)*w],
    [x^2+t2*(x*y+x*z)+y*z, x*y^2 + 1/2*(-a + 5)*x*y*z + 1/2*(-3*a + 5)*y^2*z + 1/2*(-a + 3)*x*z^2 + 1/2*(-3*a + 5)*y*z^2 + 1/2*(-a +
        3)*w]
];
Ls := [Curve(P1113,l) : l in ls ];
Lsd := [Curve(P1113,[Evaluate(e,[x,y,z,-w]) : e in l]) : l in ls];

// The strict transform of the Ls[i] divisor on S will be referred to as divisor Li
// The strict transform of the Lsd[i] divisor on S will be referred to as divisor Li'
// Li' is the image of Li under the w->-w automorphism of S.

// Desingularise and get the exceptional divisors in the dsds desingularisation data lists
dsds := ResolveSingularSurface(X);
//#dsds = 5 : one dsd desingularisation data list for each of the singularities
[*IntersectionMatrix(dsd) : dsd in dsds*];
/*
[*
    [-2],                // over singular point s1 = (-1:-1:1;0) - an A1 singularity

    [-2  0  1]
    [ 0 -2  1]
    [ 1  1 -2],          // over singular point s2 = (1:-1:1:0) - an A3 singularity

    [-2  1]
    [ 1 -2],             // over singular point s3 = (0:0:1:0) - an A2 singularity

    [-2  0  1  0]
    [ 0 -2  0  1]
    [ 1  0 -2  1]
    [ 0  1  1 -2],       // over singular point s4 = (0:1:0:0) - an A4 singularity

    [-2  0  1  0]
    [ 0 -2  0  1]
    [ 1  0 -2  1]
    [ 0  1  1 -2]        // over singular point s5 = (1:0:0:0) - an A4 singularity
*]
*/

// NB: The ith dsds[j] exceptional (-2)-curve over singularity si will be referred
// to as E_j,i, or sometimes just E1 for E_1,1.

//3)
Attach("intersection_matrix.m");
time imat := IntersectionMatrixOnDegree2K3(X,Ls cat Lsd);
// Time: 8.000
imat;
/*
[-2  0  1  1  0  1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  0  0  0  1  0  0  0  0]
[ 0 -2  0  0  1  0  0  0  0  0  0  0  1  1  0  0  0  0  0  0  0  0  0  1  0  0  0  1  0  0  0  0  0  0]
[ 1  0 -2  0  0  0  0  1  0  2  0  1  2  0  0  0  0  1  2  0  0  0  0  0  0  0  0  0  0  0  0  1  0  0]
[ 1  0  0 -2  0  0  0  1  2  0  0  1  0  2  0  0  0  1  0  2  0  0  0  0  0  0  0  0  0  0  0  1  0  0]
[ 0  1  0  0 -2  0  0  0  0  0  0  0  0  0  0  0  0  1  1  1  0  0  0  0  0  1  0  0  0  0  0  0  0  1]
[ 1  0  0  0  0 -2  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  0  1  0  0  0  0  0  0  0  1  0  0  0]
[ 0  0  0  0  0  0 -2  1  0  0  0  0  0  0  0  0  1  0  1  1  0  0  0  0  0  0  0  1  0  0  0  1  0  0]
[ 0  0  1  1  0  0  1 -2  0  0  0  0  1  1  1  0  0  2  0  0  1  0  1  0  1  0  0  1  0  0  0  0  0  0]
[ 0  0  0  2  0  0  0  0 -2  0  0  0  2  0  1  0  1  0  2  0  1  1  0  0  0  1  0  1  0  0  0  0  0  0]
[ 0  0  2  0  0  0  0  0  0 -2  0  0  0  2  1  0  1  0  0  2  1  1  0  0  0  1  0  1  0  0  0  0  0  0]
[ 0  0  0  0  0  0  0  0  0  0 -2  0  1  1  0  1  0  0  0  0  0  0  0  0  1  0  0  0  1  0  0  0  0  0]
[ 0  0  1  1  0  0  0  0  0  0  0 -2  0  0  1  0  0  0  0  0  0  0  0  1  0  0  1  0  0  0  0  0  0  0]
[ 0  1  2  0  0  0  0  1  2  0  1  0 -2  0  0  0  0  1  0  2  0  0  0  0  0  0  0  0  0  0  1  0  0  0]
[ 0  1  0  2  0  0  0  1  0  2  1  0  0 -2  0  0  0  1  2  0  0  0  0  0  0  0  0  0  0  0  1  0  0  0]
[ 0  0  0  0  0  0  0  1  1  1  0  1  0  0 -2  0  0  0  0  0  0  0  0  0  1  0  0  0  0  0  0  0  1  0]
[ 0  0  0  0  0  0  0  0  0  0  1  0  0  0  0 -2  0  0  0  0  1  1  0  0  0  0  0  0  0  0  0  1  0  0]
[ 0  0  0  0  0  0  1  0  1  1  0  0  0  0  0  0 -2  1  0  0  0  0  0  0  0  0  1  0  0  0  1  0  0  0]
[ 0  0  1  1  1  0  0  2  0  0  0  0  1  1  0  0  1 -2  0  0  1  1  0  0  0  1  1  0  0  0  0  0  0  0]
[ 0  0  2  0  1  0  1  0  2  0  0  0  0  2  0  0  0  0 -2  0  1  0  1  0  1  0  1  0  0  0  0  0  0  0]
[ 0  0  0  2  1  0  1  0  0  2  0  0  2  0  0  0  0  0  0 -2  1  0  1  0  1  0  1  0  0  0  0  0  0  0]
[ 0  0  0  0  0  1  0  1  1  1  0  0  0  0  0  1  0  1  1  1 -2  0  0  0  0  0  0  0  0  0  0  0  0  0]
[ 0  0  0  0  0  0  0  0  1  1  0  0  0  0  0  1  0  1  0  0  0 -2  0  1  0  0  0  0  0  0  0  0  0  0]
[ 0  0  0  0  0  1  0  1  0  0  0  0  0  0  0  0  0  0  1  1  0  0 -2  1  0  0  0  0  0  0  0  0  0  0]
[ 0  1  0  0  0  0  0  0  0  0  0  1  0  0  0  0  0  0  0  0  0  1  1 -2  0  0  0  0  0  0  0  0  0  0]
[ 0  0  0  0  0  0  0  1  0  0  1  0  0  0  1  0  0  0  1  1  0  0  0  0 -2  1  0  0  0  0  0  0  0  0]
[ 1  0  0  0  1  0  0  0  1  1  0  0  0  0  0  0  0  1  0  0  0  0  0  0  1 -2  0  0  0  0  0  0  0  0]
[ 0  0  0  0  0  0  0  0  0  0  0  1  0  0  0  0  1  1  1  1  0  0  0  0  0  0 -2  0  1  0  0  0  0  0]
[ 0  1  0  0  0  0  1  1  1  1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0 -2  0  1  0  0  0  0]
[ 0  0  0  0  0  0  0  0  0  0  1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  0 -2  1  0  0  0  0]
[ 1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  1 -2  0  0  0  0]
[ 0  0  0  0  0  1  0  0  0  0  0  0  1  1  0  0  1  0  0  0  0  0  0  0  0  0  0  0  0  0 -2  0  1  0]
[ 0  0  1  1  0  0  1  0  0  0  0  0  0  0  0  1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0 -2  0  1]
[ 0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  0 -2  1]
[ 0  0  0  0  1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  1 -2]

This is the 34x34 intersection matrix for the (-2)-curves in \mathcal{S}, with the order of divisors in
rows and columns being:
L1, L2, .. L10, L1', .. L10', E1, E_2,1, E_2,2, E_2,3, E_3,1, E_3,2, E_4,1, .. E_4,4, E_5,1, .. E_5,4
*/

/*
A further analysis shows that in each case of intersection number 2, the relevant divisors actually
intersect at 2 distinct points on S, so every intersection between 2 members of \mathcal{S} is a
normal crossing.

e.g. for L3 and L10:
> I3_10 := ReducedSubscheme(Ls[3] meet Ls[10]);
> I3_10;
Scheme over K1 defined by
z^3 + 1/5*(2*a + 5)*w,
x^2 - x*z + 1/2*(a - 3)*z^2,
y + 1/2*(-a + 3)*z
> Degree(I3_10);
2
> IsEmpty(I3_10 meet SingularSubscheme(X));
true

I3_10 is 2 disticnt points on X by inspection of the defining equations or by the
degree output (note that I3_10 is reduced). These points aren't singular points
of X by the last line so they represent genuine intersection points on S.

With a bit more work, we also show that each intersection is a nodal one: i.e.
no three curves meet at the same point. For example, in a few cases, the strict transforms
A1, B1 of A,B in {Li,Li'}, which meet at singular point si, intersect the same
exceptional divisor D over si. We can check that they don't intersect D at the same
point (which would be a triple-intersection of \mathcal{S} curves) because that would
mean A1.B1 is too large.

This is relevant to computing Kodaira fibres based on \mathcal{S} purely by
intersection numbers. It shows, for example, that if A,B,C are three curves
in \mathcal{S} with A.B=B.C=C.A=1 then they form an I3 configuration, rather
than an IV one.
*/

// 4)
/*
From Section 3 of the paper, we know that \mathcal{S} generates Pic(Sbar).
Using the above intersection matrix, imat, and some standard matrix computations,
we found a subset B of \mathcal{S} consisting of 19 curves that give free generators
of Pic(Sbar) and express the other elements of \mathcal{S} in terms of these.
We give the results of this computation
*/

/*
  B = [ L2,L3,L5,L6,L7,L10,L1',E1,E_2,3, E_3,1, E_3,2, E_4,1, E_4,2, E_4,3, E_4,4,
          E_5,1, E_5,2, E_5,3, E_5,4 ] 
*/

// inds = index of these generators in \mathcal{S}
// inds1 = indices of the other curves
inds := [ 2, 3, 5, 6, 7, 10, 11, 21, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34 ];
inds1 := [ 1, 4, 8, 9, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 23 ];
// the class of curve index inds1[i] in Pic(Sbar) is given by the linear combination of the 
// curves in B (i.e. the ones with indices in inds) with cofficients in rels[i]
rels := [
    [ 2, -1, 2, -1, 1, -1, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 2 ],
    [ 2, -2, 1, -1, 0, -1, 3, -1, 1, 2, 1, 2, 2, 4, 3, -1, -2, -1, -1 ],
    [ 2, -1, 3, 0, 0, -1, 0, -1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 2, 3 ],
    [ -2, 3, -1, 1, 0, 2, -3, 1, -1, -2, -1, -2, -2, -4, -3, 1, 2, 1, 1 ],
    [ -1, 1, 0, 1, 0, 1, -2, 1, -1, -1, 0, -2, -1, -3, -2, 1, 1, 1, 1 ],
    [ 2, -2, 2, -1, 1, -1, 1, -1, 1, 1, 1, 1, 2, 2, 2, -1, 0, 0, 1 ],
    [ 0, 1, 1, 0, 1, 0, -2, 0, 0, -1, 0, -1, 0, -2, -1, 0, 2, 1, 2 ],
    [ 2, -1, 1, -1, 1, -1, 1, -1, 1, 0, 0, 1, 2, 2, 2, -1, 0, -1, 0 ],
    [ 0, 1, 0, 0, 0, 1, -2, 0, 0, -1, 0, -1, 0, -2, -1, 0, 0, 0, 0 ],
    [ 2, -1, 2, -1, 0, -1, 1, -1, 1, 1, 1, 0, 1, 1, 1, -1, 0, 0, 1 ],
    [ 0, 1, -1, 0, 1, 1, -1, 0, 0, -1, -1, -1, 1, -1, 0, 0, 1, 0, 0 ],
    [ 4, -3, 3, -1, 1, -2, 2, -2, 2, 1, 1, 1, 3, 3, 3, 0, 0, 1, 2 ],
    [ 2, 0, 2, 0, 1, -1, -1, -1, 1, -1, 0, -1, 1, -1, 0, 1, 2, 2, 3 ],
    [ 3, -3, 3, -1, 1, -3, 3, -2, 1, 2, 1, 2, 2, 4, 3, 0, 0, 1, 2 ],
    [ -1, 1, -1, -1, 0, 1, 0, 0, -1, 0, 0, 0, 0, 0, 0, -1, 0, -1, -1 ]
];
// H (= hyperplave section) class is given by the following linear combination
hvec := [ 2, -1, 2, -1, 1, -1, 1, -1, 1, 1, 1, 1, 2, 2, 2, 0, 1, 1, 2 ];

// Check this: Pic(Sbar)=NS(Sbar)=Num(Sbar) since S is K3, which means that
// a Z-linear combination of curves in \mathcal{S} is 0 in Pic(Sbar) iff
// it's intersection number with every curve in \mathcal{S} is 0 [since
// we know that \mathcal{S} generates Pic(Sbar)].

V := VectorSpace(Rationals(),34);
imatQ := ChangeRing(imat,Rationals());
for i in [1..#inds1] do
  r := rels[i]; 
  vec := &+[r[j]*V.(inds[j]): j in [1..#inds]]-V.(inds1[i]);
  IsZero(vec*imatQ);
end for;

/* 
  Similarly for H once H is expressed in terms of \mathcal{S}in Pic(Sbar).
  For each i <= 7, H=Li+Li'+(lin. combination of ex. divisors over the sj
  which lie on Li/Li'). The linear combination can be computed in a number
  of ways. The quickest is to use Multiplicities on Cartier divisor H=div(l)
  where Li=(l=w-f1=0), Li`=(l=w+f1=0) for l= linear function in x,y,z.
  e.g. for L1,L1'
  > Multiplicities(X,dsds,Scheme(X,x));
  [
    [ 0 ],
    [ 0, 0, 0 ],
    [ 1, 1 ],
    [ 1, 1, 2, 2 ],
    [ 0, 0, 0, 0 ]
  ]
  shows that H=L1+L1'+E_3,1+E_3,2+E_4,1+E_4,2+2*(E_4,3+E_4,4) in Pic(Sbar)
*/

// We can get the intersection matrix for basis B and check that it is
// non-singular with determinant 24
imatB := Submatrix(imat,inds,inds);
imatB;
/*
[-2  0  1  0  0  0  0  0  1  0  0  0  1  0  0  0  0  0  0]
[ 0 -2  0  0  0  2  0  0  0  0  0  0  0  0  0  0  1  0  0]
[ 1  0 -2  0  0  0  0  0  0  0  1  0  0  0  0  0  0  0  1]
[ 0  0  0 -2  0  0  0  1  0  0  0  0  0  0  0  1  0  0  0]
[ 0  0  0  0 -2  0  0  0  0  0  0  0  1  0  0  0  1  0  0]
[ 0  2  0  0  0 -2  0  1  0  0  1  0  1  0  0  0  0  0  0]
[ 0  0  0  0  0  0 -2  0  0  1  0  0  0  1  0  0  0  0  0]
[ 0  0  0  1  0  1  0 -2  0  0  0  0  0  0  0  0  0  0  0]
[ 1  0  0  0  0  0  0  0 -2  0  0  0  0  0  0  0  0  0  0]
[ 0  0  0  0  0  0  1  0  0 -2  1  0  0  0  0  0  0  0  0]
[ 0  0  1  0  0  1  0  0  0  1 -2  0  0  0  0  0  0  0  0]
[ 0  0  0  0  0  0  0  0  0  0  0 -2  0  1  0  0  0  0  0]
[ 1  0  0  0  1  1  0  0  0  0  0  0 -2  0  1  0  0  0  0]
[ 0  0  0  0  0  0  1  0  0  0  0  1  0 -2  1  0  0  0  0]
[ 0  0  0  0  0  0  0  0  0  0  0  0  1  1 -2  0  0  0  0]
[ 0  0  0  1  0  0  0  0  0  0  0  0  0  0  0 -2  0  1  0]
[ 0  1  0  0  1  0  0  0  0  0  0  0  0  0  0  0 -2  0  1]
[ 0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  0 -2  1]
[ 0  0  1  0  0  0  0  0  0  0  0  0  0  0  0  0  1  1 -2]
*/
Determinant(imatB);
// 24

