////////////////////////////////////////////////////////////////////////////////////////
////										    ////
////       Intrinsic to compute intersection matrices for a set of (-2)-curves      ////
////         and the exceptional divisors over singular points on a degree 2        ////
////        K3 surface given by a standard weighted proj. model in P(1,1,1,3).      ////
////        Also contains intrinsics to compute Kodaira fibre configurations	    ////
////          from the intersection matrix of the curves.                           ////
////                           mch - 03/19                                          ////
////////////////////////////////////////////////////////////////////////////////////////

/*
   Current restrictions:
   1) All singular points must be defined over the base field.
   2) Have not done full local intersection multiplicity over 
       a singular point between the  strict transforms of 'bad' divisors yet.
*/

function sing_subscheme(X)
// returns the singular subscheme of X, a K3 surface of the form w^2-F(x,y,z),
// with a homogeneous sextic F, in P(1,1,1,3), char !=2

    P := Ambient(X);
    R := CoordinateRing(P);
    f := Equation(X);
    cs := Coefficients(f,4);
    assert (#cs eq 3) and IsZero(cs[2]) and (TotalDegree(cs[3]) eq 0);
    assert IsHomogeneous(cs[1]) and (TotalDegree(cs[1]) eq 6);
    R2<a,b,c> := PolynomialRing(BaseRing(X),3,"grevlex");
    F := Evaluate(cs[1],[a,b,c,0]);
    sngs := ReducedSubscheme(SingularSubscheme(Scheme(Proj(R2),F)));
    if IsEmpty(sngs) then
      return EmptySubscheme(P);
    else
      hm := hom<R2->R|[R.i : i in [1..3]]>;
      return Scheme(X,[R.4] cat [hm(e) : e in DefiningPolynomials(sngs)]);
    end if;

end function;

function same_tangents(C1,C2,s,pi)
// returns whether the tangent spaces of curves C1 and C2 (on surface X) in at point s 
// in X are the same (or one is contained in the other). pi is a patch index in the
// ambient of X in which s lies. s is given as a degree 1 cluster.

    sa := AffinePatch(s,pi);
    pt := Representative(Points(sa));
    C1a := AffinePatch(C1,pi); C2a := AffinePatch(C2,pi);
    T1 := TangentSpace(C1a!Eltseq(pt));
    T2 := TangentSpace(C2a!Eltseq(pt));
    return (Dimension(T1 meet T2) ge 1);

end function;

function int_fn(Cs,pt,ptc)
// Used in function below.

    // first use quick tangent test again for transverse intersection
    if Dimension(&meet[TangentSpace(C!Eltseq(pt)): C in Cs]) eq 0 then
      return 1;
    else
      Csmeet := &meet(Cs);
      d1 := Degree(Csmeet);
      d2 := Degree(Scheme(Ambient(Cs[1]),&meet[ColonIdeal(Ideal(Csmeet),f) :
		f in GroebnerBasis(ptc)]));
    end if;
    return d1-d2;

end function;

function easy_loc_int_over_sing(Cs,Ss,dsd,k,X)
// Computes some of the local intersection numbers for intersections on exceptional divisor (Ek)
// of index k over the singular point p represented by dsd between strict transforms of the
// curves in Cs which intersect Ek. The indices of such curves in the Cs sequence are divided
// into 4 sets which are the sequences S1,S2,S3,S4 in Ss.
//
// S1 gives curves that only intersect exceptional divisor Ek over p and maybe other Ej that
// no other member of Cs intersects. It intersects Ek  with intersection no. 1
// S2 gives curves that only intersect exceptional divisor Ek over p and maybe other Ej that
// no other member of Cs intersects. It intersects Ek  with intersection no. > 1
// S3 gives curves that intersect Ek and at least one other Ej over p that another Cs member
// intersects. It intersects Ek  with intersection no. 1
// S4 gives curves that intersect Ek and at least one other Ej over p that another Cs member
// intersects. It intersects Ek  with intersection no. >1
//
// This computes the intersection numbers between elements of S1 with other elements of S1
// and elements of S2,S3 and S4 and also returns the intersection numbers between
// elements of S2 and S3. Only the non-zero intersection numbers are returned in
// a sequence of triples [i,j,m] meaning the local intersection of Cs[i] and Cs[j] is m.
//
// Note that for the pairs considered, the m gives the total sum of local intersection
// multiplicities for intersections of the strict transforms over p.

    cls := dsd`classes; ld := dsd`div_leaf_dat;
    md := dsd`main_dat; piX := dsd`pi;
    ldks := [ld[i] : i in cls[k]];
    Sort(~ldks,func<x,y|#(md[x[1]][1])-#(md[y[1]][1])>);

    res := [];
    S1,S2,S3,S4 := Explode(Ss);

    for ldk in ldks do
      // first check termination condition
      if #S1 lt 2 then
	if (#S1 eq 0) or ((#S2+#S3+#S4) eq 0) then
	  if (#S3 eq 0) or (#S2 eq 0) then break; end if;
	end if;
      end if;
      pi := ldk[1]; ki := ldk[2];
      dat := md[pi]; A := Ambient(dat[2]);
      prj_map := dat[3];
      sat_pols := DefiningPolynomials(&join(dat[1]));
      Cs1 := [Scheme(A,&meet[ColonIdeal(I,f) : f in sat_pols]) where
	I is Ideal(AffinePatch(X meet Cs[i],piX)@@prj_map) : i in S1];
      ints := [C meet dat[1][ki] : C in Cs1];
      inds := [i : i in [1..#S1] | not IsEmpty(ints[i])];
      if dat[4][ki] then
	inds := [i : i in inds | not (ints[i] subset SingularSubscheme(dat[2]))];
      end if;
      if #inds gt 0 then
	pts := [A!Eltseq(pt) where pt is Representative(Points(ints[i])) : i in inds];
	S2to4 := &cat[S2,S3,S4];
	Cs2to4 := [Scheme(A,&meet[ColonIdeal(I,f) : f in sat_pols]) where
	  I is Ideal(AffinePatch(X meet Cs[i],piX)@@prj_map) : i in S2to4];
        // get intersection pairings between the S1 pairs
	for i in [1..#inds], j in [i+1..#inds] do
	  if pts[i] eq pts[j] then
	      Append(~res,[S1[i],S1[j],int_fn([Cs1[inds[e]]: e in [i,j]],pts[i],ints[inds[i]])]);
	  end if;
	end for;
	// Now do S1 and Si, i>=2, pairs
	for i in [1..#inds], j in [1..#S2to4] do
	  if pts[i] in Cs2to4[j] then
	      Append(~res,[S1[i],S2to4[j],int_fn([Cs1[inds[i]],Cs2to4[S2to4[j]]],
			pts[i],ints[inds[i]])]);	    
	  end if;	  	
	end for;
	S1 := [S1[i] : i in [1..#S1]| i notin inds];	
      end if;

      // Now do S2,S3 intersections
      if (#S2 eq 0) or (#S3 eq 0) then continue; end if;
      if #inds gt 0 then
	Cs3 := Cs2to4[(#S2)+1..(#S2)+(#S3)];
	Cs2 := Cs2to4[1..(#S2)];
      else
	Cs3 := [Scheme(A,&meet[ColonIdeal(I,f) : f in sat_pols]) where
	  I is Ideal(AffinePatch(X meet Cs[i],piX)@@prj_map) : i in S3];
	Cs2 := 0;
      end if;
      ints := [C meet dat[1][ki] : C in Cs3];
      inds := [i : i in [1..#S3] | not IsEmpty(ints[i])];
      if dat[4][ki] then
	inds := [i : i in inds | not (ints[i] subset SingularSubscheme(dat[2]))];
      end if;
      if #inds gt 0 then
	pts := [A!Eltseq(pt) where pt is Representative(Points(ints[inds[i]])) : i in inds];
	if Type(Cs2) cmpeq RngIntElt then
	  Cs2 := [Scheme(A,&meet[ColonIdeal(I,f) : f in sat_pols]) where
	    I is Ideal(AffinePatch(X meet Cs[i],piX)@@prj_map) : i in S2];
	end if;
	for i in [1..#inds], j in [1..#S2] do
	  if pts[i] in Cs2[j] then
	      Append(~res,[S3[i],S2[j],int_fn([Cs3[inds[i]],Cs2[S2[j]]],
			pts[i],ints[inds[i]])]);	    
	  end if;	  	
	end for;
	S3 := [S3[i] : i in [1..#S3]| i notin inds];	
      end if;

    end for; 

    assert (#S1 eq 0) or ((#S2+#S3+#S4) eq 0);
    assert (#S2 eq 0) or (#S3 eq 0);
    return res;
   
end function;

function paths_inc_1(G,paths)
// Gis a graph and paths are a sequence of length n>= 1 paths in G (all from a fixed vertex G.i)
// of n distinct vertices given as sets of sequences [i,i1,..,i(n-1)] representing
// the path G.i,G.i1,..,G.i(n-1). Return the set of all length (n+1) (distinct vertex) paths
// that extend the given paths by 1 vertex.

    v := VertexSet(G);
    new_paths := [];
    for p in paths do
      n := #p;
      good_nbrs := Neighbours(v.(p[n])) diff &join[Neighbours(v.(p[i])) : i in [1..n-1]];
      if n eq 2 then Exclude(~good_nbrs,v.(p[1])); end if;
      new_paths cat:= [p cat [Index(e)] : e in good_nbrs];
    end for;
    return new_paths;
    
end function;

intrinsic IntersectionMatrixOnDegree2K3(X::Srfc, crvs::SeqEnum) -> Mtrx
{ X is the standard model in P(1,1,1,3) of a double-cover-of-P2 K3 surface. crvs is a collection
  of (-2)-curves on X. Returns the intersection matrix of the strict transforms in the
  desingularisation of X of the curves and all of the exceptional divisors over singular points
  of X. The divisor order for the rows and columns of the matrix is the curves in the order given
  followed by the exceptional divisors in the order determined by the order of singular points
  in the dsds desingularisation object. }

    require IsProjective(X) and ((#grads eq 1) and (grads[1] eq [1,1,1,3]) where
	grads is Gradings(X)) and IsHypersurface(X) and (Degree(X) eq 6) : 
	"Argument 1 must be a degree 6 hypersurface in P(1,1,1,3)";

    if not assigned X`Nonsingular then
      try
        S := sing_subscheme(X);
      catch e
	error "Argument 1 does not have equation of the correct form";
      end try;
      if IsEmpty(S) then
	X`Nonsingular := true;
      else
	X`Nonsingular := false;
	X`SingularSubscheme := S;
      end if;
    end if;

    is_sing := not X`Nonsingular;
    if is_sing then
      dsds := ResolveSingularSurface(X);
      // set sngs to singular points (as clusters) of the dsds
      sngs := [pcm(dsd`Y) where pcm is ProjectiveClosureMap(
			AffinePatch(Ambient(X),dsd`pi)) : dsd in dsds];
      ss := X`SingularSubscheme;
    end if;

   // first get the intersection pairing between the crvs away from
   // singular points
    imat0 := DiagonalMatrix([-2 : i in [1..#crvs]]);
    if is_sing then bad_ints := [*[] : i in [1..#sngs]*]; end if;
    for i in [1..#crvs], j in [i+1..#crvs] do
      CD := crvs[i] meet crvs[j];
      d := Degree(CD);
      if (not is_sing) or (d eq 0) then
	imat0[i,j] := d; imat0[j,i] := d;
      else
	if IsEmpty(CD meet ss) then 
	  imat0[i,j] := d; imat0[j,i] := d; 
	  continue;
	end if;
	I := Ideal(CD);
	for k in [1..#sngs] do
	  if sngs[k] subset CD then
	    // remove sngs[j] component from ideal of CD
	    I := &meet[ColonIdeal(I,f) : f in DefiningPolynomials(sngs[k])];
	    // if sngs[k] is rational over the base field, do a quick tangent
	    // space check to quickly see whether crvs[i] and crvs[j] CANNOT
	    // intersect over the singular point. TODO: extend to non-rat. sngs[k]
	    if Degree(sngs[k]) eq 1 then
	      boo := same_tangents(crvs[i],crvs[j],sngs[k],dsds[k]`pi);
	    else boo := true; end if;
	    if boo then Append(~(bad_ints[k]),[i,j]); end if;
	  end if;
	end for;
	imat0[i,j] := Degree(Scheme(Ambient(X),I));
	imat0[j,i] := imat0[i,j];
      end if;	
    end for;
//printf "bad_ints: %o\n",bad_ints;

    if not is_sing then return imat0; end if;

    // Get the intersection matrix for the exceptional divisors
    imat3 := Matrix(Integers(),0,0,[]);
    for dsd in dsds do imat3 := DiagonalJoin(imat3,IntersectionMatrix(dsd)); end for;

    // Get the intersection numbers of crvs with exceptional divisors.
    // Also see what possible intersections between strict tansforms of
    // two crvs over singular points can be immediately ruled out.
    imat1 := Matrix(Integers(),#crvs,0,[]);
    for k in [1..#sngs] do
      badk := bad_ints[k];
      verybadk := [**];
      intsk := [];
      for C in crvs do
	_,intsC := MultiplicitiesAndIntersections(X,dsds[k],X meet C);
	Append(~intsk,intsC);
      end for;
      imatk := Matrix(intsk);
      imat1 := HorizontalJoin(imat1,imatk);
      // Now deal with adjusting imat0 for local interesections of curves
      // over sngs[k]. CURRENTLY ASSUMED THAT sngs[k] DEFINED OVER THE BASE FIELD.
      js := {e[1] : e in badk} join {e[2] : e in badk};
      if #js eq 0 then continue; end if;
      jsbad := {};
      r := Ncols(imatk); // number of exceptional divisors over sngs[k]
      for pair in badk do
	Ecommon := [i : i in [1..r] | &and[imatk[j,i] ne 0 : j in pair]];
	if (#Ecommon gt 1) or ((#Ecommon eq 1) and ((imatk[pair[1],i] gt 1)
	    and (imatk[pair[2],i] gt 1) where i is Ecommon[1])) then
	   // Cases where a more detailed general local intersection check needed.
	   // Should be very rare!
	   Append(~verybadk,[*pair,[[i,imatk[pair[1],i],imatk[pair[2],i]] : i in Ecommon]*]);
	   jsbad join:= Seqset(pair);
	end if;
      end for;
      jsnotsobad := {j : j in jsbad | &and[#e eq 1 : e in seq] and 
		(#{e[1][1] : e in seq} eq 1) where seq is [*u[2] : u in verybadk | j in u[1]*]};
      jsbad := jsbad diff jsnotsobad;
      js := js diff jsbad;
      ls := [l : l in [1..r] | #Support(T[l]) ge 2] where T is Transpose(imatk);
      for l in ls do
	S1 := {j : j in js | imatk[j,l] eq 1};
	S2 := {j : j in js | imatk[j,l] ge 2};
	S3 := {j : j in jsbad | imatk[j,l] eq 1};
	S4 := {j : j in jsbad | imatk[j,l] ge 2};
	Ss := [Sort([Integers()|j : j in s]) : s in [S1,S2,S3,S4]];
	adjs := easy_loc_int_over_sing(crvs,Ss,dsds[k],l,X);
	for e in adjs do
	    imat0[e[1],e[2]] +:= e[3];
	    imat0[e[2],e[1]] := imat0[e[1],e[2]];
	end for;
      end for;
      for e in verybadk do
	// TODO: general local intersection pairing over a singular point.
	error "Sorry! Not yet implemented general local intersection pairing",
		"over a singular point";
      end for;
    end for;
    imat2 := Transpose(imat1);

    mat1 := HorizontalJoin(imat0,imat1);
    mat2 := HorizontalJoin(imat2,imat3);
    imat := VerticalJoin(mat1,mat2);

    return imat;

end intrinsic;

intrinsic SimplePaths(imat::Mtrx) -> SeqEnum
{Returns all sets of simple paths of length n >= 2 in the graph with intersection
 matrix imat. Note that if imat[i,j] gt 1 then i and j will not be considered
 to be connected for a simple path. Paths and reverse paths are counted as the same. }

    N := Nrows(imat);
    require (Ncols(imat) eq N) and IsSymmetric(imat) and (BaseRing(imat) cmpeq IntegerRing())
      and &and([imat[i,j] ge 0 : j in [i+1..N], i in [1..N]]) :
	"First argument doesn't appear to be an intersection matrix";

    
    G := Graph<N|>;
    V := VertexSet(G);
    G := AddEdges(G,{{V.i,V.j} : j in [i+1..N], i in [1..N] | imat[i,j] eq 1});

    n_seqs := [];
    for i in [1..N] do
    // Get paths from vertex i.
      // start with length 2
      n := 2;
      pths := [[i,j] : j in [1..N] | (j ne i) and (imat[i,j] eq 1)];
      repeat
	pths1 := {(p[n] lt p[1]) select Reverse(p) else p : p in pths};
	if #n_seqs lt n-1 then
	    Append(~n_seqs,pths1);
	else
	    n_seqs[n-1] join:= pths1;
	end if;
	pths := paths_inc_1(G,pths); n +:= 1;
      until #pths eq 0;
    end for;

    // discard out paths where 2 nodes are joined by a multi-arrow
    bad_ij := [[i,j] : j in [i+1..N], i in [1..N] | imat[i,j] gt 1];
    for e in bad_ij do
      ij := {e[1],e[2]};
      for k in [2..#n_seqs] do
	n_seqs[k] := {s : s in n_seqs[k] | not (ij subset Seqset(s))};
      end for;
    end for;

    return n_seqs;

end intrinsic;

intrinsic InLoops(imat::Mtrx, paths::SeqEnum) -> SeqEnum
{}

    N := Nrows(imat);
    n := #paths;
    n_loops := [];

    // first get any 2-loops
    Append(~n_loops,[[i,j] : j in [i+1..N], i in [1..N] | imat[i,j] eq 2]);
    // now consider loops of length >=3
    for j in [1..n] do
      m := 0;
      loop_sets := {}; // for comparison
      loops_j := [];
      for p in paths[j] do
	v_new := [i : i in [1..N] | (i notin p) and (imat[i,p[1]] eq 1) and (imat[i,p[j+1]] eq 1)];
	v_new := [i : i in v_new | &and[imat[i,k] eq 0 : k in p[2..j]]];
	l_new := [p cat [i] : i in v_new];
	for l in l_new do
	  lset := Seqset(l);
	  Include(~loop_sets,lset);
	  if #loop_sets ne m then // new loop
	    Append(~loops_j,l); m +:= 1;
	  end if;
	end for;
      end for;
      Append(~n_loops,loops_j);
    end for;

    if &and[(#e) eq 0 : e in n_loops] then return []; end if;   
    m := Max([i : i in [1..n+1] | #(n_loops[i]) gt 0]);
    return n_loops[1..m];

end intrinsic;

intrinsic DnFigs(imat::Mtrx, paths::SeqEnum) -> SeqEnum
{}

    N := Nrows(imat);
    n := #paths;
    n_Dns := [];

    for j in [2..n] do
      m := 0;
      d_sets := {}; // for comparison
      ds_j := [];
      for p in paths[j] do
	v_new1 := [i : i in [1..N] | (i notin p) and (imat[i,p[2]] eq 1) and 
		(imat[i,p[1]] eq 0) and &and[imat[i,k] eq 0 : k in p[3..j+1]]];
	v_new2 := [i : i in [1..N]  | (i notin p) and (imat[i,p[j]] eq 1) and 
		(imat[i,p[j+1]] eq 0) and &and[imat[i,k] eq 0 : k in p[1..j-1]]];
	v_pair := [[l,k] : l in v_new1, k in v_new2 | (l ne k) and (imat[l,k] eq 0)];
	d_new := [[p[1],i[1]] cat p[2..j] cat [i[2],p[j+1]] : i in v_pair];
	for d in d_new do
	  dset := Seqset(d);
	  Include(~d_sets,dset);
	  if #d_sets ne m then // new Dn~
	    Append(~ds_j,d); m +:= 1;
	  end if;
	end for;
      end for;
      Append(~n_Dns,ds_j);
    end for;

    if &and[(#e) eq 0 : e in n_Dns] then return []; end if;   
    m := Max([i : i in [1..n-1] | #(n_Dns[i]) gt 0]);
    return n_Dns[1..m];

end intrinsic;

intrinsic E6Figs(imat::Mtrx, paths::SetEnum) -> SeqEnum
{paths contains length 5 paths}

    N := Nrows(imat);
    E6s := [];
    E6_sets :={}; // for comparison
    m := 0;

    for p in paths do
      v_new1 := [i : i in [1..N] | (i notin p) and (imat[i,p[3]] eq 1) and 
	&and[imat[i,k] eq 0 : k in p[1..2]] and &and[imat[i,k] eq 0 : k in p[4..5]]];
      for i in v_new1 do
	v_new2 := [j : j in [1..N] | (j notin (p cat [i])) and (imat[i,j] eq 1) and 
	  &and[imat[j,k] eq 0 : k in p[1..5]]];
	for j in v_new2 do
	  e := [p[3],p[2],p[1],p[4],p[5],i,j];
	  eset := Seqset(e);
	  Include(~E6_sets,eset);
	  if #E6_sets ne m then // new E6~
	    Append(~E6s,e); m +:= 1;
	  end if;
	end for;
      end for;
    end for;

    return E6s;

end intrinsic;

intrinsic E7Figs(imat::Mtrx, paths::SetEnum) -> SeqEnum
{paths contains length 7 paths}

    N := Nrows(imat);
    E7s := [];

    for p in paths do
      v_new := [i : i in [1..N] | (i notin p) and (imat[i,p[4]] eq 1) and 
	&and[imat[i,k] eq 0 : k in p[1..3]] and &and[imat[i,k] eq 0 : k in p[5..7]]];
      E7s cat:= [p cat [i] : i in v_new];
    end for;

    return E7s;

end intrinsic;

intrinsic E8Figs(imat::Mtrx, paths::SetEnum) -> SeqEnum
{paths contains length 8 paths}

    N := Nrows(imat);
    E8s := [];

    for p in paths do
      v_new1 := [i : i in [1..N] | (i notin p) and (imat[i,p[3]] eq 1) and 
	&and[imat[i,k] eq 0 : k in p[1..2]] and &and[imat[i,k] eq 0 : k in p[4..8]]];
      v_new2 := [i : i in [1..N] | (i notin p) and (imat[i,p[6]] eq 1) and 
	&and[imat[i,k] eq 0 : k in p[1..5]] and &and[imat[i,k] eq 0 : k in p[7..8]]];
      E8s cat:= [p cat [i] : i in v_new1];
      E8s cat:= [p1 cat [i] : i in v_new2] where p1 is Reverse(p);
    end for;

    return E8s;

end intrinsic;

intrinsic KodairaFibs(imat::Mtrx) -> SeqEnum
{}

    printf "Getting simple paths...\n";
    time paths := SimplePaths(imat);
    printf "Getting In loops...\n";
    time lps := InLoops(imat,paths);
    printf "Getting extended Dn configs...\n";
    time dns := DnFigs(imat,paths);
    if #(paths[4]) gt 0 then
      printf "Getting extended E6 configs...\n";
      time e6s := E6Figs(imat,paths[4]);
    end if;
    if #(paths[6]) gt 0 then
      printf "Getting extended E7 configs...\n";
      time e7s := E7Figs(imat,paths[6]);
    end if;
    if #(paths[7]) gt 0 then
      printf "Getting extended E8 configs...\n";
      time e8s := E8Figs(imat,paths[7]);
    end if;
    es := [e6s,e7s,e8s];
    delete paths;

    // Now the tricky job of seeing which sets of configs
    // are disjoint. If there are a lrge # of configs, howto do
    // this efficiently?
    printf "Getting disjoint sets of configurations...\n";  

    // Go through the largest configurations first, which
    // will have smaller disjoint complements
    N := Nrows(imat);
    seq := [<"I",i+1> : i in [1..#lps] | #(lps[i]) gt 0] cat
	[<"D",i+4> : i in [1..#dns] | #(dns[i]) gt 0] cat
	[<"E",i+6> : i in [1..3] | #(es[i]) gt 0];
    Sort(~seq,func<x,y|x[2]-y[2]>);
    seql := [s[2] : s in seq];
    bounds := [];
    biglst := []; sts := []; // sts is a set version for disjointedness testing

    for s in seq do
      case s[1]:
	when "I": arr := lps[s[2]-1]; dec := 0;
	when "D": arr := dns[s[2]-4]; dec := 1;
	when "E": arr := es[s[2]-6]; dec := 1;
      end case;
      lab := s[1] cat IntegerToString(s[2]-dec);
      biglst cat:= [<e,lab> : e in arr];
      sts cat:= [Seqset(e) : e in arr];
      Append(~bounds,((#bounds eq 0) select 0 else bounds[#bounds])+#arr);
    end for;
    n := #bounds;
    bounds := [bounds[i] : i in [1..n] | (i eq n) or (seql[i] lt seql[i+1])];
    seql := Sort(Setseq(Seqset(seql)));
    n := #bounds;
    assert n eq #seql;
    m := bounds[#bounds]; // total number of configs
    printf "Dealing with %o configs.\n",m;

    fibs := [**];
    cnt := 0; tot := 0;    
    while m gt 0 do
      cfg := biglst[m];
      fib := [cfg];
      lab := cfg[2]; cfg := cfg[1];
      m -:= 1; Prune(~biglst); Prune(~sts);
      cnt +:= 1;
      bounds[n] -:=1;
      if (n gt 1) and (bounds[n] eq bounds[n-1]) then
	Prune(~bounds); Prune(~seql); n -:= 1;
      end if;
      cfgc := {i : i in [1..N] | &and[imat[i,k] eq 0 : k in cfg]};
      k := m;
      while #cfgc ge seql[1] do
	r := #[0 : i in seql | i le #cfgc];
        k := Min((r eq n) select m else bounds[r],k);
	while k gt 0 do
	  if sts[k] subset cfgc then
	    e := biglst[k];
	    ind := Index(seql,#(e[1]));
	    Remove(~biglst,k); Remove(~sts,k);
	    m -:= 1; cnt +:= 1;
	    Append(~fib,e);
	    for i in [ind..n] do bounds[i] -:= 1; end for;
	    if ((ind gt 1) and (bounds[ind] eq bounds[ind-1])) or
		((ind eq 1) and (bounds[ind] eq 0)) then
	      Remove(~bounds,ind); Remove(~seql,ind); n -:= 1;
	    end if;
	    k -:= 1;
	    cfgc := {i : i in cfgc | &and[imat[i,k] eq 0 : k in e[1]]};
	    break;
	  end if;
	  k -:= 1;
	end while;
	if k eq 0 then break; end if;
      end while;
      Append(~fibs,fib);
      if cnt ge 1000 then
	tot +:= cnt; cnt := 0;
	printf "Done %o configs\n",tot;
      end if;   
    end while;
    return fibs;

end intrinsic;
