/********
   Functions to compute elliptic fibration maps and the generic fibres
   (as singular plane models) for a degree 2 K3-surface X, given in
   P(1,1,1,3) weighted proj. space as w^2=f6(x,y,z), along with
   a Kodaira fibre and a number of sections. mch.
**********/

/********
Main functions:
  ell_fib_rr_space : computes the Riemann-Roch space for the Kodaira
    fibre D. This space defines the fibration map.
  ell_gen_fib : Given the Riemann-Roch basis that defines the fibration
    map and a number of sections, computes the generic fibre as a
    (singular) curve (i.e., in P^2) curve and the points on it
    corresponding to the sections.
********/

function QuotientIdealBasis(I,J,n)
// finds a basis for degree n part of I/J, I >= J homog. ideals
    
    P := Generic(I);
    Mons_n := Setseq(MonomialsOfDegree(P,n));
    V := KSpace(BaseRing(P),#Mons_n);
    
    //subspace of V corr to I
    Ivecs := [V|];
    B := Basis(I);
    for b in B do
        d := LeadingTotalDegree(b);
        if d gt n then continue; end if;
        Monoms := MonomialsOfDegree(P,n-d);
        Ivecs cat:= [V![MonomialCoefficient(e,m):m in Mons_n] where
		e is b*m :m in Monoms];
   end for;
   WI := sub<V|Ivecs>;
   delete Ivecs;

    //subspace of V corr to J
    Jvecs := [V|];
    B := Basis(J);
    for b in B do
        d := LeadingTotalDegree(b);
        if d gt n then continue; end if;
        Monoms := MonomialsOfDegree(P,n-d);
        Jvecs cat:= [V![MonomialCoefficient(e,m):m in Mons_n] where
		e is b*m :m in Monoms];
   end for;
   WJ := sub<V|Jvecs>;
   delete Jvecs;

   WImodJ := Complement(WI,WJ);
   B := [Polynomial(Eltseq(f),Mons_n) : f in Basis(WImodJ)];
   return B;
    
end function;

function easy_rr_basis(X,I)
// Cut down version of DivisorToSheaf, just getting the RR space and
// avoiding the GetMax extra stuff, knowing that X <= P^n is linearly
// normal, i.e. (R/IX)_m maps onto H^0(P^n,OX(m)) for all m >= 0,
// where IX is the (saturated) defining ideal of X and R is the
// coordinate ring of P^n.

    R := CoordinateRing(Ambient(X));
    Saturate(~X);
    IX := Ideal(X);
    ID := Saturation(I);

    dX := Degree(HilbertPolynomial(IX)); dD := Degree(HilbertPolynomial(ID));
    assert dX eq dD+1;
    hD := HilbertSeries(ID);
    hX := HilbertSeries(IX);
    hXD := hX-hD;
    r := Valuation(Numerator(hXD));

    // find an Fr
    BX := GroebnerBasis(IX); 
    Reverse(~BX); 
    if IsEmpty(BX) then
      BX := [Generic(IX)| 0];
    end if;
    BD := GroebnerBasis(ID);
    Reverse(~BD);
      // assumed here that R has a degree ordering
    i := 1; while LeadingTotalDegree(BD[i]) lt r do i +:=1; end while;
    if LeadingTotalDegree(BX[#BX]) ge r then
      j := 1; while LeadingTotalDegree(BX[j]) lt r do j +:=1; end while;
      while (j le #BX) and (BD[i] eq BX[j]) do 
	j +:=1; i +:=1;
      end while;
    end if;
    Fr := BD[i];

    // Get IE
    IXF := Saturation(IX+ideal<R|Fr>);
    IE := ColonIdeal(IXF,ID); // IE is automatically saturated

    B := QuotientIdealBasis(IE,IX,r);   
    return B,Fr;

end function;

function ell_fib_rr_space(X,F,div_list,ex_div_list)
// X is a degree 2 K3 surface in P(1,1,1,3) with vars x,y,z,w and
// defining equation w^2=F(x,y,z), where F should be the 2nd argument.
// D is a Kodaira fibre on X specified by the 3rd and 4th arguments
// which give respectively the irreducible components of D which
// are strict transforms of curves on X with multiplicity and
// the irreducible components of D which are exceptional divisors
// in the blow-up desingularisation of X over singularities of F.
// (For format of (ex_)div_list, see the examples in the run code).
//
// Return is B,f. Here B=[f1,f2] with f,f1,f2 being weighted homogeneous
// polynomials of the same degree in the coordinate ring of the ambient
// of X and the 2-dimensional Riemann-Roch space H^0(X,L(D)) has
// basis f1/f,f2/f.

    Pw := Ambient(X); // P(1,1,1,3)
    K := BaseRing(Pw);
    a,b,c,t := Explode([Pw.i : i in [1..4]]);
    P10<[x]> := ProjectiveSpace(K,10);
    mons := [a^3, a^2*b, a^2*c, a*b^2, a*b*c, a*c^2, b^3, b^2*c, b*c^2, c^3, t];
    
    // convert F(a,b,c) to a quadric in the x[i]
    cs := Coefficients(F);
    ms := Monomials(F);
    ms1 := [];
    for m in ms do
      for i in [1..10] do
	boo,r := IsDivisibleBy(m,mons[i]);
	if boo then
	    j := Index(mons,r);
	    Append(~ms1,x[i]*x[j]);
	    break;
	end if;
      end for;
    end for;
    assert #ms1 eq #ms;
    F1 := Polynomial(cs,ms1);

    // X in P10 and the isomorphism from the weighted model to that.
    X1 := Surface(P10, [ -x[9]^2 + x[8]*x[10], -x[8]*x[9] + x[7]*x[10], -x[6]*x[9] + x[5]*x[10],
      -x[8]^2 + x[7]*x[9], -x[6]*x[8] + x[4]*x[10], -x[6]*x[8] + x[5]*x[9], -x[6]*x[7] + x[4]*x[9],
      -x[6]*x[7] + x[5]*x[8], -x[5]*x[7] + x[4]*x[8], -x[6]^2 + x[3]*x[10], -x[5]*x[6] + x[2]*x[10],
      -x[5]*x[6] + x[3]*x[9], -x[3]*x[6] + x[1]*x[10], -x[5]^2 + x[2]*x[9], -x[5]^2 + x[3]*x[8],
      -x[5]^2 + x[4]*x[6], -x[4]*x[5] + x[2]*x[8], -x[4]*x[5] + x[3]*x[7], -x[3]*x[5] + x[1]*x[9],
      -x[3]*x[5] + x[2]*x[6], -x[4]^2 + x[2]*x[7], -x[3]*x[4] + x[1]*x[8], -x[3]*x[4] + x[2]*x[5],
      -x[2]*x[4] + x[1]*x[7], -x[3]^2 + x[1]*x[6], -x[2]*x[3] + x[1]*x[5], -x[2]^2 + x[1]*x[4] ]
		cat [F1-x[11]^2] );
   imXtoX1 := iso<X->X1| mons, [[x[1],x[2],x[3],x[1]^2*x[11]],[x[4],x[7],x[8],x[7]^2*x[11]],
	[x[6],x[9],x[10],x[10]^2*x[11]]] : Check := false, CheckInverse := false>;

    // Get the div_list part of the R-R space on X1 and transfer back to X
    assert #div_list gt 0;
    R10 := CoordinateRing(P10);
    ID := R10;
    for e in div_list do
      p := imXtoX1(X meet e[1]);
      Saturate(~p);
      p := Ideal(p);
      n := e[2]; // multiplicity of divisor
      J := p;
      for i in [1..n-1] do // get symbolic power of p if multiplicity > 1
	J := &meet[c : c in PrimaryDecomposition(p*J+Ideal(X1)) | Dimension(c) eq 2];	
      end for;
      ID meet:= J;
    end for;
//printf "Getting RR space...\n";
    B,den := easy_rr_basis(X1,ID);
//printf "Done!\n";
      // Pull back to Pw
    B := [Evaluate(b,mons) : b in B];
    den := Evaluate(den,mons);

    // Now add pole conditions at exceptional divs over the singularities.
    if not X`Nonsingular then // should be already set
      dsds := [r[2] : r in X`res_list | r[1] eq 2][1]; // full blow-up desing should already be there!
      mult_seqs := [];
      for dsd in dsds do
	ms := Multiplicities(X,dsd,Scheme(X,den));
        conds := [[e[2],e[3]] : e in ex_div_list | IsIdentical(e[1],dsd)];
	for e in conds do ms[e[1]] := Max(ms[e[1]]-e[2],0); end for;
	Append(~mult_seqs,ms);
      end for;
      B := LinearSystemDivisorRestriction(X,B,mult_seqs);
    end if;

    // Remove any common factor of B[1],B[2],den
    cf := GCD(B cat [den]);
    if Degree(cf) gt 0 then
      B := [b div cf : b in B]; den := den div cf;
    end if;

    return B,den;
    
end function;

function simplify_map(X,B)
// B = [f1,f2] is an (elliptic fibration) map from X to P^1. f1 and f2
// are of weighted degree d in the x,y,z,t vars. Look for B1=[g1,g2]
// where g1,g2 are weighted of degree d1<d and B1 defines the same
// X->P^1 map as B.
    d := WeightedDegree(B[1]);
    assert d eq WeightedDegree(B[2]);
    if d eq 1 then return B; end if;
    R := CoordinateRing(Ambient(X));
    I := Ideal(X);
    for d1 in [1..d-1] do
      ms := Setseq(MonomialsOfWeightedDegree(R,d1));
      pows1 := [NormalForm(m*B[2],I) : m in ms];
      pows2 := [NormalForm(-m*B[1],I) : m in ms];
      mons := Setseq(&join[Seqset(Monomials(e)) : e in pows1 cat pows2]);
      rel_mat := Matrix([[MonomialCoefficient(e,m) : m in mons] :
	e in pows1 cat pows2]);
      K := Kernel(rel_mat);
      if Dimension(K) gt 0 then
	if d1 lt 6 then
	  cs := Eltseq(Basis(K)[1]);
	else
	  Bs := Basis(K);
	  cs := 0;
	  for b in Bs do
	    if Polynomial(Eltseq(b)[1..#ms],ms) notin I then
	      cs := Eltseq(b); break;
	    end if;
	  end for;
	  if Type(cs) cmpeq RngIntElt then continue; end if;
	end if;
	return [Polynomial(cs[1..#ms],ms),Polynomial(cs[(#ms)+1..2*(#ms)],ms)];
      end if;
    end for;
    return B;
end function;


function ell_gen_fib_old(X,B,sections : var_name := "t")
// Old version of the function below that produces a much nastier model of
// the generic fibre and is now only used in very bad cases.

   Xa := AffinePatch(X,4);
   k := BaseRing(X);
   K := RationalFunctionField(k);
   AssignNames(~K,[var_name]); t := K.1;
   XaK := BaseChange(Xa,K);
   R := CoordinateRing(Ambient(XaK));
   subst := [1,R.1,R.2,R.3];
   secs := [Scheme(Ambient(XaK),[Evaluate(b,subst) : b in Basis(Ideal(s))])
		: s in sections];
   B1 := [Evaluate(b,subst): b in B] ;
   I := ideal<R|[B1[1]-t*B1[2]] cat Basis(Ideal(XaK))>;
   I := ColonIdeal(I,B1[1]);
   Xfib :=  Scheme(Ambient(XaK),I);
   pts := [];
   for s in secs do
     I := ColonIdeal(ideal<R|[B1[1]-t*B1[2]] cat Basis(Ideal(s))>,B1[1]);
     assert Dimension(I) eq 0;
     p := Points(Scheme(Ambient(XaK),I));
     assert #p eq 1;
     Append(~pts,Xfib!Eltseq(p[1]));
   end for;
   return Xfib,pts,subst;
end function;


function ell_gen_fib(X,B,sections : var_name := "t")
// X is a degree 2 K3 surface in P(1,1,1,3)/k and B=[f1,f2] where f1, f2
// are two homogeneous polynomials of the same weighted degree in the
// polynomial ring of the ambient of X. mp:=map<X->P^1|B> is a
// genus 1 fibration map on X.
// Function computes a model the generic fibre of mp as a curve C in
// P(1,1,1)/K or P(1,2,1)/K where K=k(t).
// sections should be a (possibly empty) sequence of sections of mp
// given as subschemes of X.
// The return values are the curve C/K, a sequence of points in C(K)
// corresponding to the sections, and a sequence of 4 rational
// funcions in FOF(coord ring of ambient of C) that give the
// substitution in the variables of Ambient(X) to transform
// X_{gen fibre} to C.
 
   R := CoordinateRing(Ambient(X));
   f6 := -Evaluate(F,[R.1,R.2,R.3,0])/MonomialCoefficient(F,(R.4)^2)
	where F is DefiningEquation(X);
   cs := Coefficients(B[1],4); l := #cs;
   U := &+[cs[2*i+1]*f6^i :i in [0..(l-1) div 2]];
   V := &+[R|cs[2*i]*f6^(i-1) :i in [1..l div 2]];
   cs := Coefficients(B[2],4); l := #cs;
   W := &+[cs[2*i+1]*f6^i :i in [0..(l-1) div 2]];
   Y := &+[R|cs[2*i]*f6^(i-1) :i in [1..l div 2]];
   // B[1]/B[2]=(U+V*w)/(W+Y*w)

   k := BaseRing(X);
   K := RationalFunctionField(k);
   AssignNames(~K,[var_name]); t := K.1;

if not &and[IsZero(e) : e in [Y,V]] then
   // Now use the substitution w=(-W*t+U)/(Y*t-V)
   P2<a,b,c> := ProjectiveSpace(K,2);
   R1 := CoordinateRing(P2);
   subst := [R1.1,R1.2,R1.3,0];
   U,V,W,Y := Explode([Evaluate(e,subst) : e in [U,V,W,Y]]);
   P2eqn := (W*t-U)^2-(Y*t-V)^2*Evaluate(f6,subst);
   fact := Factorisation(P2eqn);
   fact1 := [f : f in fact | not &and[c in k : c in Coefficients(f[1])]];
   assert (#fact1 eq 1) and (fact1[1][2] eq 1);
   P2eqn := fact1[1][1];
   Xfib := Curve(P2,P2eqn);
   
   subst1 := [R1.1,R1.2,R1.3,(U-t*W)/(Y*t-V)];
else
   // Here t := U/W. Find an x,y or z variable that occurs to at most degree
   // 1 in U & W and eliminate that. If this fails, fall back on old function.
   P2<a,b,c> := ProjectiveSpace(K,[1,2,1]);
   R1 := CoordinateRing(P2);
   i := 0;
   for j in [1..3] do
      d1 := Degree(U,R.j); d2 := Degree(W,R.j);
      if Max(d1,d2) eq 1 then i := j; break; end if;
   end for;
   if i eq 0 then
     return ell_gen_fib_old(X,B,sections : var_name := var_name);
   end if;
   cs := Coefficients(U,i);
   U1 := cs[1];
   V1 := (#cs eq 1) select R!0 else cs[2];
   cs := Coefficients(W,i);
   W1 := cs[1];
   Y1 := (#cs eq 1) select R!0 else cs[2];
   // t=(U1+V1*v)/(W1+Y1*v), where v=R.i and U1,V1,W1,Y1 are polys in the
   // other 2 R.1,R.2,R.3 variables.
   // Use the elimination substitution v=(t*W1-U1)/(V1-t*Y1).
   subst := [a,c];
   Insert(~subst,i,R1!0);
   U1,V1,W1,Y1 := Explode([Evaluate(e,subst cat [0]) : e in [U1,V1,W1,Y1]]);
   ChangeUniverse(~subst,FieldOfFractions(R1));
   subst[i] := (t*W1-U1)/(V1-t*Y1);
   P2pol := Evaluate(f6,subst cat [0]);
   num := Numerator(P2pol); den := Denominator(P2pol);
   if TotalDegree(den) eq 0 then
     num := (1/K!den)*num; den := R1!1;
   end if;
   factn := Factorisation(num);
   factd := Factorisation(den);
   fnd := Seqset([e[1] : e in factn]) meet Seqset([e[1] : e in factd]);
   // remove common factors from num and den
   for fact in fnd do
     i1 := Index([e[1] : e in factn],fact);
     i2 := Index([e[1] : e in factd],fact);
     m1 := factn[i1][2]; m2 := factd[i2][2];
     if m1 gt m2 then
       Remove(~factd,i2); factn[i1] := <fact,m1-m2>;
     elif m2 gt m1 then
       Remove(~factn,i1); factd[i2] := <fact,m2-m1>;
     else //m1=m2
       Remove(~factn,i1); Remove(~factd,i2);
     end if;
   end for;
   for i in [1..#factd] do
     if IsOdd(factd[i][2]) then
	Append(~factn,<factd[i][1],1>);
	factd[i] := <factd[i][1],factd[i][2]+1>;
     end if;
   end for;
   lc1 := LeadingCoefficient(num*(&*[R1|e[1]^e[2] : e in factd]));
   lc2 := LeadingCoefficient(den*(&*[R1|e[1]^e[2] : e in factn]));
   odd_ms := [i : i in [1..#factn] | IsOdd(factn[i][2])];
   assert #odd_ms gt 0;
   P2pol := (lc1/lc2) * &*[factn[i][1]  : i in odd_ms];
   assert LeadingTotalDegree(P2pol) eq 4;
   wden := &*[R1|e[1]^(e[2] div 2) : e in factd];
   wnum := &*[R1|e[1]^(e[2] div 2) : e in factn];
   // num/den = (wnum/wden)^2*P2pol
   Xfib := Curve(P2,P2pol-b^2);

   subst1 := subst cat [(wnum/wden)*b];
end if;
   pts := [];
   for s in sections do
     B1 := Basis(Ideal(s));
     B1 := [Numerator(Evaluate(e,subst1)) : e in B1];
     pcs := RadicalDecomposition(ideal<R1|B1>);
     pcs := [p : p in pcs | Dimension(p) eq 1];
     if #pcs gt 1 then //remove spurious 'constant' points
       pcs := [p : p in pcs | not &and[&and[c in k : c in Coefficients(b)] : b in Basis(p)]];
     end if;
     assert #pcs eq 1;
     Xs := Scheme(P2,pcs[1]);
     assert Degree(Xs) eq 1;  
     p := Points(Xs);
     assert #p eq 1;
     Append(~pts,Xfib!Eltseq(p[1]));
   end for;
   return Xfib,pts,subst1;

end function;
