/************* Computation of genus 1 fibrations - mch ****************************************
1) Explicitly compute the 3 genus 1 fibrations from the paper
2) The Shioda-Inose computations from the end of Section 4 of the paper.

- this is a continuation from the defns_and_basic_properties.m file, which should be run first,
  and also uses the functions in k3_ell_fib.m . The notes in defns_and_basic_properties.m
  about the Magma version and labelling of the curves in \mathcal{S} still apply.
***********************************************************************************************/

// load the functions to compute Riemann-Roch spaces and generic fibres of fibrations.
load "k3_ell_fib.m";

// Example 1 elliptic fibration
// ___________________________

// This has 3 bad fibres supported entirely on \mathcal{S} curves of types I6,I6 and extended D4(I0*).
// It corresponds to the first of the 2 G-classes of fibration in fib3reps[6] (see all_genus_1_fibrations.m).


// We compute the fibration map from the Riemann-Roch space of the 2nd I6 fibre of fib3reps[6]:
// I6 fibre F = L1+E_3,2+E_3,1+L1'+E_4,3+E_4,4
B,den := ell_fib_rr_space(X,f6,[[*Ls[1],1*],[*Lsd[1],1*]],
     [[*dsds[3],1,1*],[*dsds[3],2,1*],[*dsds[4],3,1*],[*dsds[4],4,1*]]);
B; den;
/*
[
    x,
    z
]
x
 - The Riemann-Roch space H^0(S,L(F)) is a 2-dimensional K1 vector-space generated by 1=x/x and z/x
*/

// The fibration map (up to equivalence) is given by map<X->P1|B>, where P1=ProjectiveSpace(K1,1).
// In fact, it is defined over Q.

// By computing the intersection F.C for C in \mathcal{S}, using imat (as in the last computations in
// all_genus_1_fibrations.m) find that L3,L4,L5,L6,L8,L9,L10,L3',L4',L5',L6',L8',L9',L10',E_4,1,E_4,2
// all give sections (but no other \mathcal{S} curves do).

// Get a singular model Xfib of the generic fibre of the the fibration over K1(t) and the
// transformation of X(S) to it.
// Also get the points in Xfib(K1(t)) corresponding to the sections L3,L4,L5,L6,L8,L9,L10
// (the points corr. to L3',L4' etc. follow easily from these)
// We take t=z/x rather than x/z for F to be the fibre over 0 rather than infty
B := Reverse(B);
Xfib,pts,subst := ell_gen_fib(X,B,[Ls[i] : i in [3,4,5,6,8,9,10]]);
Ambient(Xfib);Xfib; pts; subst;
/*
Projective Space of dimension 2 over Univariate rational function field over K1
Variables: t
Variables: a, b, c
The grading is:
    1, 2, 1
Curve over Univariate rational function field over K1 defined by
1/t^2*a^4 + (2*t^2 + 2)/t^3*a^3*c + (t^4 + 4*t^3 - 4*t^2 + 4*t + 1)/t^4*a^2*c^2 + (2*t^2 + 2)/t^3*a*c^3 +
    1/t^2*c^4 - b^2
[ (1/2*(a - 3) : (1/2*(a - 3)*t^2 + 1/2*(-a + 3)*t + 1/2*(a - 3))/t^2 : 1), (1/2*(-a - 3) : 
    (1/2*(-a - 3)*t^2 + 1/2*(a + 3)*t + 1/2*(-a - 3))/t^2 : 1), (0 : 1/t : 1), (-1 : (-t^2 + 
    1)/t^2 : 1), (-1/t^2 : (-t^4 + t^3 - t + 1)/t^5 : 1), ((1/2*(-a + 1)*t - 1)/(t^2 + 1/2*(a - 
    1)*t) : (1/2*(-a + 3)*t^4 + 1/2*(3*a - 3)*t^3 + 1/2*(-3*a + 3)*t + 1/2*(a - 3))/(t^5 + (a - 
    1)*t^4 + 1/2*(-a + 3)*t^3) : 1), ((1/2*(a + 1)*t - 1)/(t^2 + 1/2*(-a - 1)*t) : (1/2*(a + 
    3)*t^4 + 1/2*(-3*a - 3)*t^3 + 1/2*(3*a + 3)*t + 1/2*(-a - 3))/(t^5 + (-a - 1)*t^4 + 1/2*(a + 
    3)*t^3) : 1) ]
[
    1/t*$.3,
    $.1,
    $.3,
    $.2*$.3
]
- Xfib is given by b^2=f4(a,c) in weighted P(1,2,1).
-  subst shows that the transformation to go from X to Xfib is
  x=(1/t)*c, y=a, z=c, w=b*c.
*/

// We make the slight transformation on Xfib by replacing t*a by a and t^3*b by b
// so that in the b^2=f4(a,c) equation, f4 is monic (i.e. a^4+..)
K<t> := RationalFunctionField(K1); // t=z/x
P121<u,w,v> := ProjectiveSpace(K,[1,2,1]);
Xfib := Curve(P121,u^4+2*(t^2+1)*u^3*v+(t^4 + 4*t^3 - 4*t^2 + 4*t + 1)*v^2*u^2+
  2*t^2*(t^2+1)*u*v^3+t^4*v^4-w^2);
// sections Ls[3,4,5,6,8,9,10] -> pts1
pts1 := [Xfib!s : s in
  [[(1/2)*(a - 3)*t,(1/2)*(a - 3)*(t^3 - t^2 + t),1],
   [-(1/2)*(a + 3)*t,-(1/2)*(a + 3)*(t^3 - t^2 + t),1],
   [0,t^2,1],
   [-t,t-t^3,1],
   [-1/t,-(t^4 - t^3 + t - 1)/t^2],
   [-((1/2)*(a - 1)*t + 1)/(t + (1/2)*(a - 1)),
    (t^4 + (3/2)*(a + 1)*t^3 - (3/2)*(a+1)*t - 1)/ ((1/2)*(a + 3)*t^2 + (a + 1)*t + 1),1],
   [((1/2)*(a + 1)*t - 1)/(t - (1/2)*(a + 1)),
    (t^4 + (3/2)*(-a + 1)*t^3 + (3/2)*(a-1)*t - 1)/ ((1/2)*(-a + 3)*t^2 + (-a + 1)*t + 1),1]
  ]];
// sections Lsd[3,4,5,6,8,9,10] -> pts2
pts2 := [Xfib![e[1],-e[2],e[3]] where e is Eltseq(p) : p in pts1];

// ell_gen_fib doesn't allow the computation of the points on Xfib(K) corr. to
// exceptional divisor sections, but we did this by considering the fibration map
// pulled back to the blow-up patches of the dsds desingularisation data objects.
// The result is that E_4,1 <-> Xfib![1,-1,0], E4,2 <-> Xfib![1,1,0]
p41 := Xfib![1,-1,0]; p42 := Xfib![1,1,0];

// Now get an isomorphism of Xfib onto an elliptic curve E, using pts1[3] <-> L5
// as the O-point, using Magma library functions and then simplifying the model.
E1,mp1 := EllipticCurve(Xfib,pts1[3]);
E2,mp2 := SimplifiedModel(E1);
E2;
/*
Elliptic Curve defined by y^2 = x^3 + (-1/48*t^12 - 1/6*t^11 + 1/12*t^10 + 1/2*t^9 - 19/24*t^8 + 
    1/2*t^7 + 1/12*t^6 - 1/6*t^5 - 1/48*t^4)/(t^8 - 6*t^7 + 35/2*t^6 - 63/2*t^5 + 609/16*t^4 - 
    63/2*t^3 + 35/2*t^2 - 6*t + 1)*x + (1/864*t^18 + 1/72*t^17 + 1/48*t^16 - 23/216*t^15 + 
    7/96*t^14 - 1/18*t^13 + 23/216*t^12 - 1/18*t^11 + 7/96*t^10 - 23/216*t^9 + 1/48*t^8 + 1/72*t^7
    + 1/864*t^6)/(t^12 - 9*t^11 + 159/4*t^10 - 225/2*t^9 + 3615/16*t^8 - 5409/16*t^7 + 
    24689/64*t^6 - 5409/16*t^5 + 3615/16*t^4 - 225/2*t^3 + 159/4*t^2 - 9*t + 1) over Univariate 
rational function field over K1
*/
// Find a simpler (integral & more minimal) model for E2:
a4,a6 := Explode(aInvariants(E2)[4..5]);
Factorisation(Numerator(a4));
/*
[
    <t, 4>,
    <t - 1, 2>,
    <t^6 + 10*t^5 + 15*t^4 - 4*t^3 + 15*t^2 + 10*t + 1, 1>
]
*/
Factorisation(Denominator(a4));
/*
[
    <t^2 - 3/2*t + 1, 4>
]
*/
Factorisation(Numerator(a6));
/*
[
    <t, 6>,
    <t - 1, 4>,
    <t^2 + 1, 1>,
    <t^2 + 6*t + 1, 1>,
    <t^4 + 10*t^3 + 14*t^2 + 10*t + 1, 1>
]
*/
Factorisation(Denominator(a6));
/*
[
    <t^2 - 3/2*t + 1, 6>
]
*/
P<T> := PolynomialRing(K);
pol := T^3+a4*T+a6;
pol1 := Evaluate(pol,(t/(t^2-(3/2)*t+1))^2*T)*((t^2-(3/2)*t+1)/t)^6;
pol1;
/*
T^3 + (-1/48*t^8 - 1/6*t^7 + 1/12*t^6 + 1/2*t^5 - 19/24*t^4 + 1/2*t^3 + 1/12*t^2 - 1/6*t - 1/48)*T
    + 1/864*t^12 + 1/72*t^11 + 1/48*t^10 - 23/216*t^9 + 7/96*t^8 - 1/18*t^7 + 23/216*t^6 - 
    1/18*t^5 + 7/96*t^4 - 23/216*t^3 + 1/48*t^2 + 1/72*t + 1/864
*/
Factorisation(pol1);
/*
[
    <T - 1/12*t^4 - 1/3*t^3 + 5/6*t^2 - 1/3*t - 1/12, 1>,
    <T^2 + (1/12*t^4 + 1/3*t^3 - 5/6*t^2 + 1/3*t + 1/12)*T - 1/72*t^8 - 1/9*t^7 + 1/18*t^6 + 
        5/36*t^4 + 1/18*t^2 - 1/9*t - 1/72, 1>
]
*/
// Translate the 2-torsion point to (0,0)
pol2 := Evaluate(pol1,T + 1/12*t^4 + 1/3*t^3 - 5/6*t^2 + 1/3*t + 1/12);
pol2;
/*
T^3 + (1/4*t^4 + t^3 - 5/2*t^2 + t + 1/4)*T^2 + (-t^5 + 2*t^4 - t^3)*T
*/
// and make t coeffs more integral
pol3 := Evaluate(pol2,(1/4)*T)*2^6;
pol3;
// T^3 + (t^4 + 4*t^3 - 10*t^2 + 4*t + 1)*T^2 + (-16*t^5 + 32*t^4 - 16*t^3)*T

// this gives our desired nice Weierstrass model: y^2=x^3+(t-1)^2*(t^2+6*t+1)*x^2-16*t^3*(t-1)^2*x
E := EllipticCurve([0,(t-1)^2*(t^2+6*t+1),0,-16*t^3*(t-1)^2,0]);
_,mp3 := IsIsomorphic(E2,E);
// get combined map from Xfib to E
mptoE := Expand(mp1*mp2*mp3);
mptoE;
/*
Mapping from: CrvPln: Xfib1 to CrvEll: E
with equations : 
(2*t^2 + 2)/t^2*u^3*v^3 + (4*t^4 + 6*t^2 + 4)/t^2*u^2*v^4 + (2*t^6 + 8*t^5 - 10*t^4 + 16*t^3 - 
    10*t^2 + 8*t + 2)/t^2*u*v^5 + (2*t^4 - 8*t^3 + 18*t^2 - 8*t + 2)*v^6 + (-2*t^2 - 
    2)/t^4*u^3*w*v + (-4*t^4 - 4*t^2 - 4)/t^4*u^2*w*v^2 + (-2*t^6 - 8*t^5 + 14*t^4 - 16*t^3 + 
    14*t^2 - 8*t - 2)/t^4*u*w*v^3 + (16*t^2 - 30*t + 16)/t*w*v^4 - 2/t^4*u^2*w^2 + (-4*t^2 - 
    4)/t^4*u*w^2*v + (-2*t^4 - 8*t^3 + 14*t^2 - 8*t - 2)/t^4*w^2*v^2 - 2/t^4*w^3
(-2*t^4 + 8*t^3 - 16*t^2 + 8*t - 2)/t^2*u^3*v^3 + (-4*t^6 + 16*t^5 - 34*t^4 + 32*t^3 - 34*t^2 + 
    16*t - 4)/t^2*u^2*v^4 + (-2*t^8 + 28*t^6 - 96*t^5 + 128*t^4 - 96*t^3 + 28*t^2 - 2)/t^2*u*v^5 +
    (-2*t^6 + 24*t^5 - 48*t^4 + 48*t^3 - 48*t^2 + 24*t - 2)*v^6 + (-2*t^4 - 8*t^3 + 8*t^2 - 8*t - 
    2)/t^4*u^3*w*v + (-4*t^6 - 16*t^5 + 12*t^4 - 32*t^3 + 12*t^2 - 16*t - 4)/t^4*u^2*w*v^2 + 
    (-2*t^8 - 16*t^7 - 16*t^6 + 48*t^5 - 92*t^4 + 48*t^3 - 16*t^2 - 16*t - 2)/t^4*u*w*v^3 + 
    (-4*t^6 - 16*t^5 + 22*t^4 - 32*t^3 + 22*t^2 - 16*t - 4)/t^2*w*v^4 + (-2*t^2 - 2)/t^4*u^2*w^2 +
    (-4*t^4 - 12*t^2 - 4)/t^4*u*w^2*v + (-2*t^6 - 8*t^5 + 4*t^4 - 16*t^3 + 4*t^2 - 8*t - 
    2)/t^4*w^2*v^2 + (-2*t^2 - 2)/t^4*w^3
v^6 - 3/t^2*w*v^4 + 3/t^4*w^2*v^2 - 1/t^6*w^3
*/
// Can get the birational inverse of mptoE using Magma's IsInvertible intrinsic;
_,invmptoE := IsInvertible(mptoE);
invmptoE;
/*
Mapping from: CrvEll: E to CrvPln: Xfib
with equations : 
(1/4*t^10 + t^9 + 1/4*t^8 + 6*t^7 - 7/2*t^6 + 10*t^5 - 7/2*t^4 + 6*t^3 + 1/4*t^2 + t + 1/4)/(t^7 + 2*t^6 
    - 9*t^5 + 12*t^4 - 9*t^3 + 2*t^2 + t)*$.1^3*$.2 + (-6*t^10 - 8*t^9 - 20*t^8 - 24*t^7 - 28*t^6 - 
    24*t^5 - 20*t^4 - 8*t^3 - 6*t^2)/(t^4 + 4*t^3 - 2*t^2 + 4*t + 1)*$.1^3*$.3 + (1/4*t^12 + 5/2*t^11 + 
    13/2*t^10 + 21/2*t^9 + 163/4*t^8 - 7*t^7 + 69*t^6 - 7*t^5 + 163/4*t^4 + 21/2*t^3 + 13/2*t^2 + 5/2*t +
    1/4)/(t^5 + 4*t^4 - 2*t^3 + 4*t^2 + t)*$.1^2*$.2*$.3 + 2*t^2*$.1*$.2^2*$.3 + (-1/4*t^10 - t^9 - 
....
*/
// Get the full set of bad fibres of the fibration via Tate's algorithm
LocalInformation(E);
/*
[ <(t + 1), 2, 1, 2, I2, false>, <(t^2 + (-2*a + 4)*t + 1), 1, 1, 1, I1, false>, <(t - 1), 6, 2, 4, I0*, 
true>, <(t^2 + (2*a + 4)*t + 1), 1, 1, 1, I1, false>, <(t), 6, 1, 6, I6, true>, <(1/t), 6, 1, 6, I6, 
true> ]
*/

// Lastly get the images of our sections and find the subgroup of points of E(K) that they generate
pts1E := [mp(p) : p in pts1];
pts2E := [mp(p) : p in pts2];
p41E := mp(p41); p42E := mp(p42);
pts1E; pts2E; p41E; p42E;
/*
[ ((-2*a - 6)*t^3 + (4*a + 12)*t^2 + (-2*a - 6)*t : (-2*a - 6)*t^5 + (12*a + 28)*t^4 + (-20*a - 44)*t^3 +
    (12*a + 28)*t^2 + (-2*a - 6)*t : 1), ((2*a - 6)*t^3 + (-4*a + 12)*t^2 + (2*a - 6)*t : (2*a - 6)*t^5 +
    (-12*a + 28)*t^4 + (20*a - 44)*t^3 + (-12*a + 28)*t^2 + (2*a - 6)*t : 1), (0 : 1 : 0), (-4*t^3 + 
    4*t^2 : -4*t^5 + 4*t^4 + 4*t^3 - 4*t^2 : 1), (-4*t^3 + 4*t^2 : 4*t^5 - 4*t^4 - 4*t^3 + 4*t^2 : 1), 
(4*t^4 - 4*t^3 : -4*a*t^6 + 4*a*t^5 + 4*a*t^4 - 4*a*t^3 : 1), (4*t^4 - 4*t^3 : 4*a*t^6 - 4*a*t^5 - 
    4*a*t^4 + 4*a*t^3 : 1) ]
[ ((2*a + 6)*t^2 : (-2*a - 6)*t^4 + (-4*a - 4)*t^3 + (-2*a - 6)*t^2 : 1), ((-2*a + 6)*t^2 : (2*a - 6)*t^4
    + (4*a - 4)*t^3 + (2*a - 6)*t^2 : 1), (-4*t^3 + 8*t^2 - 4*t : 4*t^5 - 8*t^4 + 8*t^3 - 8*t^2 + 4*t : 
    1), (4*t^2 - 4*t : -4*t^4 + 4*t^3 + 4*t^2 - 4*t : 1), (4*t^6 - 4*t^5 : -8*t^9 + 12*t^8 - 4*t^7 - 
    4*t^6 + 12*t^5 - 8*t^4 : 1), (((-2*a - 6)*t^5 + (-2*a + 2)*t^4 + 4*a*t^3 + 4*t^2)/(t^2 + (a + 1)*t + 
    1/2*(a + 3)) : ((2*a + 6)*t^8 + (-2*a - 10)*t^7 + (-2*a + 2)*t^6 + (4*a + 8)*t^5 + (-2*a - 10)*t^4 + 
    (-2*a + 2)*t^3 + (2*a + 2)*t^2)/(t^3 + 1/2*(3*a + 3)*t^2 + 1/2*(3*a + 9)*t + (a + 2)) : 1), (((2*a - 
    6)*t^5 + (2*a + 2)*t^4 - 4*a*t^3 + 4*t^2)/(t^2 + (-a + 1)*t + 1/2*(-a + 3)) : ((-2*a + 6)*t^8 + (2*a 
    - 10)*t^7 + (2*a + 2)*t^6 + (-4*a + 8)*t^5 + (2*a - 10)*t^4 + (2*a + 2)*t^3 + (-2*a + 2)*t^2)/(t^3 + 
    1/2*(-3*a + 3)*t^2 + 1/2*(-3*a + 9)*t - a + 2) : 1) ]
(0 : 0 : 1)
(4*t^2 : 4*t^4 + 4*t^2 : 1)
 - p41 -> T=(0,0) the 2-Torsion point of E
*/
hpm := HeightPairingMatrix(pts1E cat pts2E cat [p42E]);
hpm;
/*
[ 4/3 -2/3    0  1/3 -1/3    1   -1 -2/3  4/3  2/3  1/3    1 -1/3  5/3  2/3]
[-2/3  4/3    0  1/3 -1/3   -1    1  4/3 -2/3  2/3  1/3    1  5/3 -1/3  2/3]
[   0    0    0    0    0    0    0    0    0    0    0    0    0    0    0]
[ 1/3  1/3    0  1/3 -1/3    0    0  1/3  1/3  2/3  1/3    1  2/3  2/3  2/3]
[-1/3 -1/3    0 -1/3  1/3    0    0 -1/3 -1/3 -2/3 -1/3   -1 -2/3 -2/3 -2/3]
[   1   -1    0    0    0    1   -1   -1    1    0    0    0   -1    1    0]
[  -1    1    0    0    0   -1    1    1   -1    0    0    0    1   -1    0]
[-2/3  4/3    0  1/3 -1/3   -1    1  4/3 -2/3  2/3  1/3    1  5/3 -1/3  2/3]
[ 4/3 -2/3    0  1/3 -1/3    1   -1 -2/3  4/3  2/3  1/3    1 -1/3  5/3  2/3]
[ 2/3  2/3    0  2/3 -2/3    0    0  2/3  2/3  4/3  2/3    2  4/3  4/3  4/3]
[ 1/3  1/3    0  1/3 -1/3    0    0  1/3  1/3  2/3  1/3    1  2/3  2/3  2/3]
[   1    1    0    1   -1    0    0    1    1    2    1    3    2    2    2]
[-1/3  5/3    0  2/3 -2/3   -1    1  5/3 -1/3  4/3  2/3    2  7/3  1/3  4/3]
[ 5/3 -1/3    0  2/3 -2/3    1   -1 -1/3  5/3  4/3  2/3    2  1/3  7/3  4/3]
[ 2/3  2/3    0  2/3 -2/3    0    0  2/3  2/3  4/3  2/3    2  4/3  4/3  4/3]
*/
Rank(hpm);
// 2

/* 
  We used hpm to find relations between pts1E, pts2E and p42E which must
  generate a rank 2 subgroup of E(K) [the canonical height,h , is a quadratic
  form on E(K) with h(P)=0 iff P is a torsion point.
  
  Find that, if we define
   T1 = (0,0),  P1 = (4*t*(t-1),-4*t*(t+1)*(t-1)^2) and P2 = (4*t^3*(t-1), -4*a*t^3*(t+1)*(t-1)^2)
  as in section 4 of the paper, then
  
  L3 <-> P1+P2+T1, L4 <-> P1-P2+T1, L5 <-> O, L6 <-> P1+T1, L8 <-> -P1+T1, L9 <-> P2, L10 <-> -P2,
  L3' <-> P1-P2, L4' <-> P1+P2, L5' <-> 2*P1+T1, L6' <-> P1, L8' <-> 3*P1, L9' <-> 2*P1-P2+T1,
  L10' <-> 2*P1+P2+T1, E_4,1 <-> T1, E_4,2 <-> 2*P1

  and that <P1,P2> <= E(K) is ZxZ, so the group generated by the sections is <T1,P1,P2>=(Z/2)xZxZ

  Check this:
*/

T1 := E![0,0];
P1 := E![4*t*(t-1),-4*t*(t+1)*(t-1)^2];
P2 := E![4*t^3*(t-1), -4*a*t^3*(t+1)*(t-1)^2];
HeightPairingMatrix([P1,P2]);
/*
[1/3   0]
[  0   1]
 - it's non-singular so <P1,P2>=ZxZ
*/
pts1cs := [[1,1,1],[1,-1,1],[0,0,0],[1,0,1],[-1,0,1],[0,1,0],[0,-1,0]];
[pts1E[i]-&+[m[1]*P1+m[2]*P2+m[3]*T1] eq E!0 where m is pts1cs[i] : i in [1..7]];
// [ true, true, true, true, true, true, true ]
pts2cs := [[1,-1,0],[1,1,0],[2,0,1],[1,0,0],[3,0,0],[2,-1,1],[2,1,1]];
[pts2E[i]-&+[m[1]*P1+m[2]*P2+m[3]*T1] eq E!0 where m is pts2cs[i] : i in [1..7]];
// [ true, true, true, true, true, true, true ]
pts42E-2*P1;
// (0 : 1 : 0)


// Example 2 elliptic fibration
// ____________________________

// This has 4 bad fibres supported entirely on \mathcal{S} curves of types I10,I2,I2,I2.
// It corresponds to the unique G-class of fibration in fib4reps[1] (see all_genus_1_fibrations.m).

// We compute the fibration map from the Riemann-Roch space of the 3rd I2 fibre of fib4reps[1]:
// I2 fibre F = L8+L8'
B,den := ell_fib_rr_space(X,f6,[[*Ls[8],1*],[*Lsd[8],1*]],[]);
B; den;
/*
[
    x^2 + y*z,
    x*y + x*z
]
x^2 + y*z
 - The Riemann-Roch space H^0(S,L(F)) is a 2-dimensional K1 vector-space generated by 1 and x(y+z)/(x^2+y*z)
*/

// The fibration map (up to equivalence) is given by map<X->P1|B>, where P1=ProjectiveSpace(K1,1).
// Again, it is actually defined over Q.

// By computing the intersection F.C for C in \mathcal{S}, using imat (as in the last computations in
// all_genus_1_fibrations.m) find that L5,L7,L5',L7',E_2,1,E_2,2,E_3,1,E_3,2,E_4,1,E_4,2
// all give sections (but no other \mathcal{S} curves do).

// As above, we reverse B for the fibration map.
// We then get Xfib for this fibration and the points <-> sections L5,L5',L7,L7'
B := Reverse(B);
Xfib,pts,subst := ell_gen_fib(X,B,[Ls[5],Lsd[5],Ls[7],Lsd[7]]);
Ambient(Xfib);Xfib;pts;subst;
/*
Projective Space of dimension 2 over Univariate rational function field over K1
Variables: t
Variables: a, b, c
The grading is:
    1, 2, 1
Curve over Univariate rational function field over K1 defined by
(t^4 + 2*t^3 + t^2)*a^4 + (-2*t^4 - 6*t^3 - 2*t^2 - 2*t)*a^3*c + (3*t^4 + 4*t^3 + 8*t^2 + 1)*a^2*c^2 + 
    (-2*t^4 - 6*t^3 - 2*t^2 - 2*t)*a*c^3 + (t^4 + 2*t^3 + t^2)*c^4 - b^2
[ (1/t : -t^2 + 1 : 1), (1/t : t^2 - 1 : 1), (1 : t^2 + t : 0), (1 : -t^2 - t : 0) ]
[
    $.1,
    (t*$.1^2 - $.1*$.3)/($.1 - t*$.3),
    $.3,
    ($.1^3*$.2 - $.1*$.2*$.3^2)/($.1^2 - 2*t*$.1*$.3 + t^2*$.3^2)
]
- Aagin, Xfib is given by b^2=f4(a,c) in weighted P(1,2,1).
-  subst shows that the transformation to go from X to Xfib is
  x=a, y=a*(t*a-c)/(a-t*c), z=c, w=((a^2-c^2)*b*a)/(a-t*c)^2
*/

// We transform (somewhat unnecessarily) to use the old P(1,2,1) ambient for Xfib
Xfib := Curve(P121,(t*(t+1))^2*u^4-2*t*(t^3+3*t^2+t+1)*u^3*v+(3*t^4+4*t^3+8*t^2+1)*u^2*v^2-
  2*t*(t^3+3*t^2+t+1)*u*v^3+(t*(t+1))^2*v^4-w^2);
pts := [Xfib!Eltseq(p) : p in pts];

// As in the first example, we worked out the Xfib(K) points corresponding to the
// E_2,1, E_2,2, E_3,1, E_3,2, E_4,1, E_4,2 sections by examining the fibration map
// pulled back to various blow-up resolution patches. We just add the results
// here at the front of the sequence of Li,Li' points.
pts := [Xfib!(s cat [1]) : s in 
  [[1,-(t-1)^2],[1,(t-1)^2],[0,t*(t+1)],[0,-t*(t+1)],
  [t,-t^2*(t^2-1)],[t,t^2*(t^2-1)]]] cat pts;

// Now transform to a Weierstrass model using the E_3,1-section point as O-point
// and simplify as before
E1,mp1 := EllipticCurve(Xfib,pts[3]);
E2,mp2 := SimplifiedModel(E1);
E2;
/*
Elliptic Curve defined by y^2 = x^3 + (-3*t^12 - 12*t^11 - 58/3*t^10 - 68/3*t^9 - 95/3*t^8 - 88/3*t^7 + 
    4/3*t^6 + 24*t^5 + 41/3*t^4 - 4/3*t^3 - 10/3*t^2 - 4/3*t - 1/3)/(t^28 + 8*t^27 + 40*t^26 + 152*t^25 +
    452*t^24 + 1112*t^23 + 2296*t^22 + 3976*t^21 + 5830*t^20 + 7128*t^19 + 7096*t^18 + 5640*t^17 + 
    3268*t^16 + 1160*t^15 + 232*t^14 + 24*t^13 + t^12)*x + (-2*t^18 - 12*t^17 - 10*t^16 + 2144/27*t^15 + 
    2168/9*t^14 + 704/3*t^13 - 1000/27*t^12 - 2048/9*t^11 - 292/3*t^10 + 2392/27*t^9 + 788/9*t^8 + 
    32/9*t^7 - 88/3*t^6 - 160/9*t^5 - 8/3*t^4 + 64/27*t^3 + 14/9*t^2 + 4/9*t + 2/27)/(t^42 + 12*t^41 + 
    84*t^40 + 436*t^39 + 1806*t^38 + 6252*t^37 + 18564*t^36 + 47988*t^35 + 109167*t^34 + 219832*t^33 + 
    392904*t^32 + 623688*t^31 + 876996*t^30 + 1086744*t^29 + 1177224*t^28 + 1099816*t^27 + 869487*t^26 + 
    565884*t^25 + 289316*t^24 + 108996*t^23 + 28686*t^22 + 5052*t^21 + 564*t^20 + 36*t^19 + t^18) over 
Univariate rational function field over K1
*/
a4,a6 := Explode(aInvariants(E2)[4..5]);
Factorisation(Numerator(a4));
/*
[
    <t + 1, 4>,
    <t^8 + 4/9*t^6 + 16/9*t^5 - 2/9*t^4 - 16/9*t^3 + 4/9*t^2 + 1/9, 1>
]
*/
Factorisation(Denominator(a4));
/*
[
    <t, 12>,
    <t^4 + 2*t^3 + 4*t^2 + 6*t + 1, 4>
]
*/
Factorisation(Numerator(a6));
/*
[
    <t + 1, 6>,
    <t^4 + 8/3*t^3 - 2/3*t^2 - 1/3, 1>,
    <t^8 - 8/3*t^7 - 20/9*t^6 + 40/9*t^5 - 2/9*t^4 - 16/9*t^3 + 4/9*t^2 + 1/9, 1>
]
*/
Factorisation(Denominator(a6));
/*
[
    <t, 18>,
    <t^4 + 2*t^3 + 4*t^2 + 6*t + 1, 6>
]
*/
pol := T^3+a4*T+a6;
pol1 := Evaluate(pol,((t+1)/(t^3*(t^4+2*t^3+4*t^2+6*t+1)))^2*T)*((t^3*(t^4+2*t^3+4*t^2+6*t+1))/(t+1))^6;
pol1;
/*
T^3 + (-3*t^8 - 4/3*t^6 - 16/3*t^5 + 2/3*t^4 + 16/3*t^3 - 4/3*t^2 - 1/3)*T - 2*t^12 + 20*t^10 - 16/27*t^9
    - 230/9*t^8 + 80/9*t^7 + 184/27*t^6 - 16/9*t^5 + 2/9*t^4 - 16/9*t^3 + 4/9*t^2 + 2/27
*/
Factorisation(pol1);
/*
[
    <T + t^4 + 8/3*t^3 - 2/3*t^2 - 1/3, 1>,
    <T^2 + (-t^4 - 8/3*t^3 + 2/3*t^2 + 1/3)*T - 2*t^8 + 16/3*t^7 + 40/9*t^6 - 80/9*t^5 + 4/9*t^4 + 
        32/9*t^3 - 8/9*t^2 - 2/9, 1>
]
*/
// Again translate the 2-torsion point to (0,0)
pol2 := Evaluate(pol1,T - t^4 - 8/3*t^3 + 2/3*t^2 + 1/3);
pol2;
// T^3 + (-3*t^4 - 8*t^3 + 2*t^2 + 1)*T^2 + (16*t^7 + 16*t^6 - 16*t^5)*T

//-> nice model for E y^2=x^3-(3t^4+8t^3-2t^2-1)x^2+16*t^5*(t^2+t-1)x
E := EllipticCurve([0,-(3*t^4+8*t^3-2*t^2-1),0,16*t^5*(t^2+t-1),0]);
_,mp3 := IsIsomorphic(E2,E);
// get combined map from Xfib to E
mptoE := Expand(mp1*mp2*mp3);
mptoE;
/*
Mapping from: CrvPln: Xfib to CrvEll: E
with equations : 
(-2*t^4 - 6*t^3 - 2*t^2 - 2*t)*u^3*v^3 + (2*t^6 + 16*t^5 + 32*t^4 + 24*t^3 + 26*t^2 + 8*t + 4)/(t^2 + 2*t
    + 1)*u^2*v^4 + (-2*t^7 - 6*t^6 - 14*t^5 - 38*t^4 - 14*t^3 - 18*t^2 - 2*t - 2)/(t^3 + 2*t^2 + t)*u*v^5
    + (12*t^5 + 18*t^4 + 12*t^2 + 4*t + 2)/(t^2 + 2*t + 1)*v^6 + (2*t^3 + 6*t^2 + 2*t + 2)/(t + 
    1)*u^3*w*v + (-8*t^5 - 20*t^4 - 16*t^3 - 24*t^2 - 8*t - 4)/(t^4 + 3*t^3 + 3*t^2 + t)*u^2*w*v^2 + 
    (-2*t^7 - 14*t^6 - 18*t^5 + 14*t^4 + 2*t^3 + 14*t^2 + 2*t + 2)/(t^5 + 3*t^4 + 3*t^3 + t^2)*u*w*v^3 + 
    (2*t^5 - 8*t^4 - 4*t^3 + 24*t^2 + 2*t)/(t^3 + 3*t^2 + 3*t + 1)*w*v^4 - 2*u^2*w^2 + (4*t^3 + 12*t^2 + 
    4*t + 4)/(t^3 + 2*t^2 + t)*u*w^2*v + (4*t^3 - 10*t^2 - 2)/(t^4 + 2*t^3 + t^2)*w^2*v^2 - 2/(t^2 + 
    t)*w^3
(2*t^7 - 4*t^6 - 6*t^5 + 8*t^4 - 10*t^3 - 4*t^2 - 2*t)/(t + 1)*u^3*v^3 + (-6*t^9 - 18*t^8 - 6*t^7 - 
    30*t^6 - 54*t^5 + 38*t^4 + 22*t^3 + 38*t^2 + 12*t + 4)/(t^3 + 3*t^2 + 3*t + 1)*u^2*v^4 + (6*t^10 + 
    4*t^9 + 18*t^8 + 32*t^7 + 32*t^6 + 80*t^5 - 64*t^4 - 16*t^3 - 22*t^2 - 4*t - 2)/(t^4 + 3*t^3 + 3*t^2 
    + t)*u*v^5 + (-4*t^9 + 42*t^7 + 6*t^6 - 66*t^5 + 6*t^4 - 10*t^3 + 18*t^2 + 6*t + 2)/(t^3 + 3*t^2 + 
    3*t + 1)*v^6 + (-6*t^4 - 8*t^3 - 16*t^2 - 2)*u^3*w*v + (12*t^7 + 52*t^6 + 92*t^5 + 124*t^4 + 52*t^3 +
    44*t^2 + 4*t + 4)/(t^3 + 2*t^2 + t)*u^2*w*v^2 + (-10*t^8 - 16*t^7 - 80*t^6 - 96*t^5 - 132*t^4 - 
    16*t^3 - 32*t^2 - 2)/(t^4 + 2*t^3 + t^2)*u*w*v^3 + (2*t^7 + 2*t^6 + 12*t^5 + 64*t^4 + 22*t^3 + 34*t^2
    + 4*t + 4)/(t^3 + 2*t^2 + t)*w*v^4 + (2*t^3 + 6*t^2 + 2*t + 2)/(t + 1)*u^2*w^2 + (-8*t^6 - 40*t^5 - 
    68*t^4 - 48*t^3 - 32*t^2 - 8*t - 4)/(t^4 + 3*t^3 + 3*t^2 + t)*u*w^2*v + (8*t^7 + 36*t^6 + 62*t^5 + 
    74*t^4 + 32*t^3 + 24*t^2 + 2*t + 2)/(t^5 + 3*t^4 + 3*t^3 + t^2)*w^2*v^2 + (2*t^3 + 6*t^2 + 2*t + 
    2)/(t^3 + 2*t^2 + t)*w^3
v^6 - 3/(t^2 + t)*w*v^4 + 3/(t^4 + 2*t^3 + t^2)*w^2*v^2 - 1/(t^6 + 3*t^5 + 3*t^4 + t^3)*w^3
*/
// Can get the birational inverse of mptoE using Magma's IsInvertible intrinsic;
_,invmptoE := IsInvertible(mptoE);
invmptoE;
/*
Mapping from: CrvEll: E to CrvPln: Xfib
with equations : 
(9/20*t^13 + 81/20*t^12 + 16*t^11 + 381/10*t^10 + 1237/20*t^9 + 1419/20*t^8 + 297/5*t^7 + 204/5*t^6 + 
    83/4*t^5 + 39/4*t^4 + 3*t^3 + 11/10*t^2 + 3/20*t + 1/20)/(t^12 + 23/5*t^11 + 44/5*t^10 + 41/5*t^9 + 
    8/5*t^8 - 3*t^7 - 8/5*t^6 - 1/5*t^5 - 1/5*t^4)*$.1^3*$.2 + (-18/5*t^13 - 184/5*t^12 - 784/5*t^11 - 
    1864/5*t^10 - 2842/5*t^9 - 3088/5*t^8 - 2584/5*t^7 - 1664/5*t^6 - 878/5*t^5 - 344/5*t^4 - 24*t^3 - 
    24/5*t^2 - 6/5*t)/(t^7 + 23/5*t^6 + 49/5*t^5 + 59/5*t^4 + 39/5*t^3 + 13/5*t^2 + 3/5*t + 
....
*/

// Get the full set of bad fibres of the fibration via Tate's algorithm
LocalInformation(E);
/*
[ <(t + 1/2*(-a + 1)), 2, 1, 2, I2, false>, <(t - 1), 4, 1, 4, I4, true>, <(t^2 + 2/9*t + 1/9), 1, 1, 1, I1, false>, <(t +
    1/2*(a + 1)), 2, 1, 2, I2, false>, <(1/t), 2, 1, 2, I2, false>, <(t), 10, 1, 10, I10, true>, <(t + 1), 2, 1, 2, I2, 
false> ]
*/

// Lastly get the images of our sections and find the subgroup of points of E(K) that they generate
ptsE := [mptoE(p) : p in pts];
ptsE;
/*
[ (4*t^3 + 4*t^2 - 4*t : -4*t^5 + 4*t^4 + 8*t^3 - 12*t^2 + 4*t : 1), (4*t^4 : 4*t^6 - 8*t^5 + 4*t^4 : 1), 
(0 : 1 : 0), ((4*t^5 + 4*t^4 - 4*t^3)/(t^2 + 2*t + 1) : (4*t^8 + 16*t^7 + 12*t^6 - 4*t^5 - 4*t^3)/(t^3 + 
    3*t^2 + 3*t + 1) : 1), (4*t : -8*t^4 - 4*t^3 + 8*t^2 + 4*t : 1), (4*t^4 + 4*t^3 - 4*t^2 : -4*t^6 - 
    4*t^5 + 8*t^4 + 4*t^3 - 4*t^2 : 1), (4*t^3 : 4*t^5 - 4*t^3 : 1), (4*t^6 + 4*t^5 - 4*t^4 : 8*t^9 + 
    12*t^8 - 12*t^7 - 16*t^6 + 4*t^5 + 4*t^4 : 1), (4*t^4 + 8*t^3 + 4*t^2 : -4*t^6 - 16*t^5 - 16*t^4 - 
    8*t^3 - 4*t^2 : 1), (0 : 0 : 1) ]
*/
hpm := HeightPairingMatrix(ptsE);
hpm;
/*
[  3/5   3/5     0   6/5  9/10  3/10  3/10  9/10   6/5     0]
[  3/5   3/5     0   6/5  9/10  3/10  3/10  9/10   6/5     0]
[    0     0     0     0     0     0     0     0     0     0]
[  6/5   6/5     0  12/5   9/5   3/5   3/5   9/5  12/5     0]
[ 9/10  9/10     0   9/5 27/20  9/20  9/20 27/20   9/5     0]
[ 3/10  3/10     0   3/5  9/20  3/20  3/20  9/20   3/5     0]
[ 3/10  3/10     0   3/5  9/20  3/20  3/20  9/20   3/5     0]
[ 9/10  9/10     0   9/5 27/20  9/20  9/20 27/20   9/5     0]
[  6/5   6/5     0  12/5   9/5   3/5   3/5   9/5  12/5     0]
[    0     0     0     0     0     0     0     0     0     0]
*/
Rank(hpm);
// 1

// It is easy to see by inspection of hpm that ptsE should generate
// a (Z/2)+Z subgroup of E(K) with generators T1=(0,0) and
// P1=ptsE[7]. This is easily checked:
P1 := ptsE[7]; // (4*t^3,4*t^3*(t^2-1))
T1 := E![0,0];
[[m : m in [-4..4] | m*P1-p in {E!0,T1}][1] : p in ptsE];
// [ 2, 2, 0, 4, 3, 1, 1, 3, 4, 0 ]

/* 
  Explicitly,
  L5 <-> P1, L5' <-> 3*P1+T1, L7 <-> 4*P1, L7' <-> T1, E_2,1 <-> 2*P1+T1, E_2,2 <-> 2*P1, 
  E_3,1 <-> O, E_3,2 <-> 4*P1+T1, E_4,1 <-> 3*P1, E_4,2 <-> P1+T1.
*/ 

// Example 3 genus 1 fibration
// ____________________________

// This has 2 bad fibres supported entirely on \mathcal{S} curves both of type extended E8.
// It corresponds to the 7th G-classe of fibration in fib2reps[38] (see all_genus_1_fibrations.m).
//
// The extE8 fibres are F = 6*E_5,1+5*E_5,3+4*E_5,4+4*L6+3*L3'+3*L5+2*E1+2*E_3,2+E_3,1
// and F1 = 6*L2'+5*E_4,1+4*E_2,3+4*E_4,3+3*L4+3*E_4,4+2*E_2,1+2*E_4,2+L7
// We compute the fibration map from F.
time B,den := ell_fib_rr_space(X,f6,[[*Ls[6],4*],[*Lsd[3],3*],[*Ls[5],3*]], 
      [[*dsds[1],1,2*],[*dsds[3],1,1*],[*dsds[3],2,2*],[*dsds[5],1,6*],
      [*dsds[5],3,5*],[*dsds[5],4,4*]]);
// Time: 112.730
B; den;
/*
[
    x^8*y + 6*x^3*y^6 + 5*x^2*y^7 + 1/19*(-a - 28)*x^7*y*z + 1/19*(72*a + 2567)*x^3*y^5*z + 1/19*(71*a + 
        1399)*x^2*y^6*z + x^7*z^2 + 1/19*(-16*a - 220)*x^6*y*z^2 + 1/38*(2601*a + 33099)*x^3*y^4*z^2 + 
        1/19*(706*a + 5043)*x^2*y^5*z^2 + 1/38*(-9*a + 413)*x*y^6*z^2 + 1/19*(-a - 104)*x^6*z^3 + 
        1/38*(393*a + 1713)*x^5*y*z^3 + 1/19*(5797*a + 44459)*x^3*y^3*z^3 + 1/38*(2257*a - 55)*x^2*y^4*z^3
        + 1/19*(89*a + 3366)*x*y^5*z^3 + 1/19*(26*a + 120)*x^5*z^4 + 1/38*(-1737*a - 5107)*x^4*y*z^4 + 
        1/19*(8190*a + 47927)*x^3*y^2*z^4 + 1/19*(-3391*a - 28030)*x^2*y^3*z^4 + 1/19*(1524*a + 
        17497)*x*y^4*z^4 + 1/38*(-9*a + 223)*y^5*z^4 + 1/2*(-7*a + 37)*x^4*z^5 + 1/19*(3305*a + 
        14051)*x^3*y*z^5 + 1/38*(-13477*a - 69423)*x^2*y^2*z^5 + 1/19*(5701*a + 39795)*x*y^3*z^5 + 
        1/19*(9*a + 1620)*y^4*z^5 + 1/38*(-205*a - 3707)*x^3*z^6 + 1/19*(-131*a + 4312)*x^2*y*z^6 + 
        1/2*(721*a + 4185)*x*y^2*z^6 + 1/19*(446*a + 6522)*y^3*z^6 + 1/19*(1316*a + 8196)*x^2*z^7 + 
        1/38*(5733*a + 29519)*x*y*z^7 + 1/38*(2427*a + 17017)*y^2*z^7 + 1/19*(7*a - 203)*x*z^8 + 
        1/19*(-47*a + 299)*y*z^8 + 1/19*(-2*a + 58)*z^9 - x^6*w - x^5*y*w + 1/19*(a + 66)*x^5*z*w + 
        1/38*(9*a + 233)*x^4*y*z*w + 1/2*(3*a - 9)*x^3*y^2*z*w + 1/19*(-35*a + 103)*x^2*y^3*z*w - 
        x*y^4*z*w + 1/19*(14*a - 64)*x^4*z^2*w + 1/19*(30*a - 414)*x^3*y*z^2*w + 1/38*(-309*a + 
        943)*x^2*y^2*z^2*w + 1/19*(70*a - 282)*x*y^3*z^2*w + 1/38*(-9*a + 33)*y^4*z^2*w + 1/19*(-42*a - 
        93)*x^3*z^3*w + 1/19*(-141*a + 859)*x^2*y*z^3*w + 1/38*(447*a - 1601)*x*y^2*z^3*w + 1/19*(-53*a + 
        169)*y^3*z^3*w + 1/38*(75*a + 599)*x^2*z^4*w + 1/19*(174*a - 923)*x*y*z^4*w + 1/38*(-249*a + 
        799)*y^2*z^4*w + 1/19*(-10*a - 299)*x*z^5*w + 1/2*(-9*a + 39)*y*z^5*w + 1/19*(-5*a + 107)*z^6*w + 
        2*x^3*w^2 - 3*x^2*y*w^2 + 4*x*y^2*w^2 - 5*y^3*w^2 + 1/38*(-11*a - 669)*x^2*z*w^2 + 1/19*(35*a + 
        733)*x*y*z*w^2 + 1/19*(-71*a - 1418)*y^2*z*w^2 + 1/19*(180*a + 1563)*x*z^2*w^2 + 1/38*(-1281*a - 
        11681)*y*z^2*w^2 + 1/19*(-1356*a - 7853)*z^3*w^2,
    1/19*(71*a + 183)*x^3*y^5*z + 1/38*(-213*a - 549)*x^2*y^6*z + 1/38*(69*a + 241)*x*y^7*z + 1/19*(-18*a 
        - 48)*y^8*z + 1/19*(118*a + 435)*x^3*y^4*z^2 + 1/38*(-533*a - 1757)*x^2*y^5*z^2 + 1/38*(191*a + 
        1149)*x*y^6*z^2 + 1/19*(-130*a - 410)*y^7*z^2 + 1/38*(51*a + 953)*x^3*y^3*z^3 + 1/19*(-206*a - 
        1265)*x^2*y^4*z^3 + 1/19*(-18*a + 1092)*x*y^5*z^3 + 1/38*(-389*a - 2133)*y^6*z^3 + 1/38*(-107*a + 
        595)*x^3*y^2*z^4 + 1/19*(18*a - 978)*x^2*y^3*z^4 + (-12*a + 60)*x*y^4*z^4 + 1/38*(-15*a - 
        3137)*y^5*z^4 + 1/2*(-3*a + 9)*x^3*y*z^5 + 1/38*(153*a - 713)*x^2*y^2*z^5 + 1/38*(-435*a + 
        1139)*x*y^3*z^5 + 1/38*(427*a - 2921)*y^4*z^5 + 1/38*(13*a + 3)*x^2*y*z^6 + 1/19*(-43*a - 
        26)*x*y^2*z^6 + 1/38*(397*a - 1671)*y^3*z^6 + 1/2*(-a + 3)*x^2*z^7 + (2*a - 8)*x*y*z^7 + 
        1/38*(159*a - 545)*y^2*z^7 + 1/19*(16*a - 46)*x*z^8 + 1/19*(16*a - 46)*y*z^8 + x^3*y^3*w + 
        1/38*(-7*a - 25)*x^2*y^4*w + 1/38*(-7*a - 63)*x*y^5*w + (3*a + 7)*y^6*w + 1/2*(-a + 3)*x^3*y^2*z*w
        + (-a - 2)*x^2*y^3*z*w + 1/38*(109*a - 45)*x*y^4*z*w + 1/38*(229*a + 693)*y^5*z*w + 1/19*(-16*a - 
        11)*x^2*y^2*z^2*w + 1/38*(235*a - 89)*x*y^3*z^2*w + 1/19*(61*a + 530)*y^4*z^2*w + 1/2*(a - 
        3)*x^2*y*z^3*w + 1/38*(177*a - 79)*x*y^2*z^3*w + 1/19*(-34*a + 625)*y^3*z^3*w + 1/38*(13*a + 
        79)*x*y*z^4*w + 1/38*(-149*a + 939)*y^2*z^4*w + 1/2*(-a + 3)*x*z^5*w + 1/2*(-7*a + 25)*y*z^5*w + 
        1/19*(-16*a + 46)*z^6*w + x*y^2*w^2 + 1/38*(-7*a - 63)*y^3*w^2 + 1/2*(-a + 3)*x*y*z*w^2 + 1/2*(-a 
        - 11)*y^2*z*w^2 + 1/19*(3*a - 87)*y*z^2*w^2 + (a - 3)*z^3*w^2
]
x^3*y^5*z - 3/2*x^2*y^6*z + 1/218*(-59*a + 258)*x*y^7*z + 1/218*(3*a - 63)*y^8*z + 1/436*(-489*a + 
    1985)*x^3*y^4*z^2 + 1/218*(358*a - 1741)*x^2*y^5*z^2 + 1/436*(-1227*a + 3749)*x*y^6*z^2 + 1/109*(70*a 
    - 380)*y^7*z^2 + 1/436*(-1535*a + 4113)*x^3*y^3*z^3 + 1/436*(2743*a - 8335)*x^2*y^4*z^3 + 
    1/218*(-2127*a + 5427)*x*y^5*z^3 + 1/218*(1056*a - 3319)*y^6*z^3 + 1/436*(-1627*a + 3865)*x^3*y^2*z^4 
    + 1/109*(957*a - 2439)*x^2*y^3*z^4 + 1/109*(-1614*a + 3810)*x*y^4*z^4 + 1/436*(5789*a - 14967)*y^5*z^4
    + 1/218*(-297*a + 678)*x^3*y*z^5 + 1/436*(2069*a - 4863)*x^2*y^2*z^5 + 1/436*(-4223*a + 
    9549)*x*y^3*z^5 + 1/218*(3757*a - 9028)*y^4*z^5 + 1/436*(57*a - 107)*x^2*y*z^6 + 1/436*(-317*a + 
    553)*x*y^2*z^6 + 1/218*(2517*a - 5878)*y^3*z^6 + 1/218*(-99*a + 226)*x^2*z^7 + 1/218*(467*a - 
    1087)*x*y*z^7 + 1/218*(892*a - 2055)*y^2*z^7 + 1/218*(163*a - 371)*x*z^8 + 1/218*(163*a - 371)*y*z^8 +
    1/436*(-71*a + 183)*x^3*y^3*w + 1/436*(13*a - 55)*x^2*y^4*w + 1/218*(42*a - 119)*x*y^5*w + 1/109*(13*a
    + 54)*y^6*w + 1/218*(-99*a + 226)*x^3*y^2*z*w + 1/436*(-41*a - 11)*x^2*y^3*z*w + 1/436*(609*a - 
    1235)*x*y^4*z*w + 1/218*(-96*a + 599)*y^5*z*w + 1/436*(-113*a + 193)*x^2*y^2*z^2*w + 1/218*(649*a - 
    1312)*x*y^3*z^2*w + 1/436*(-1393*a + 3965)*y^4*z^2*w + 1/218*(99*a - 226)*x^2*y*z^3*w + 1/218*(500*a -
    1017)*x*y^2*z^3*w + 1/436*(-2663*a + 6655)*y^3*z^3*w + 1/436*(-85*a + 259)*x*y*z^4*w + 1/218*(-1236*a 
    + 2957)*y^2*z^4*w + 1/218*(-99*a + 226)*x*z^5*w + 1/218*(-764*a + 1765)*y*z^5*w + 1/218*(-163*a + 
    371)*z^6*w + 1/436*(-71*a + 183)*x*y^2*w^2 + 1/218*(42*a - 119)*y^3*w^2 + 1/218*(-99*a + 
    226)*x*y*z*w^2 + 1/436*(299*a - 829)*y^2*z*w^2 + 1/218*(177*a - 447)*y*z^2*w^2 + 1/109*(99*a - 
    226)*z^3*w^2
- as usual H^0(S,L(F))=K1(B[1]/den)+K2(B[2]/den)
*/
Xfib,_,subst := ell_gen_fib(X,B,[]);
Ambient(Xfib);Xfib;
/*
Projective Space of dimension 2 over Univariate rational function field over K1
Variables: t
Variables: a, b, c
Curve over Univariate rational function field over K1 defined by
a^6*b^4 + a^5*b^5 + 1/2*(3*a + 7)*t*b^10 + 1/2*(-3*a + 7)/t*a^6*b^3*c - 7*a^4*b^5*c + a^3*b^6*c - a^2*b^7*c + 
    a*b^8*c + 1/2*(15*a + 37)*t*b^9*c + 1/2*(-3*a + 9)/t*a^6*b^2*c^2 + (t + (9*a - 21))/t*a^5*b^3*c^2 - 
    15*a^4*b^4*c^2 + 16*a^3*b^5*c^2 - 4*a^2*b^6*c^2 + 8*a*b^7*c^2 + (15*a + 42)*t*b^8*c^2 + 3/t*a^6*b*c^3 + (9*a - 
    27)/t*a^5*b^2*c^3 + (-7*t + 1/2*(-45*a + 105))/t*a^4*b^3*c^3 + 34*a^3*b^4*c^3 - 11*a^2*b^5*c^3 + 21*a*b^6*c^3 + 
    (1/2*(29*a + 109)*t + 1)*b^7*c^3 + 1/2*(a + 3)/t*a^6*c^4 - 18/t*a^5*b*c^4 + 1/2*(-45*a + 135)/t*a^4*b^2*c^4 + 
    (16*t + (30*a - 70))/t*a^3*b^3*c^4 - 17*a^2*b^4*c^4 + 23*a*b^5*c^4 + (1/2*(11*a + 91)*t + 10)*b^6*c^4 + (-3*a - 
    9)/t*a^5*c^5 + 45/t*a^4*b*c^5 + (t + (30*a - 90))/t*a^3*b^2*c^5 + (-11*t + 1/2*(-45*a + 105))/t*a^2*b^3*c^5 + 
    16*a*b^4*c^5 + (1/2*(-3*a + 51)*t + 34)*b^5*c^5 + 1/2*(15*a + 45)/t*a^4*c^6 - 60/t*a^3*b*c^6 + (-4*t + 
    1/2*(-45*a + 135))/t*a^2*b^2*c^6 + (23*t + (9*a - 21))/t*a*b^3*c^6 + ((-2*a + 9)*t + 51)*b^4*c^6 + (-10*a - 
    30)/t*a^3*c^7 + (-t + 45)/t*a^2*b*c^7 + (21*t + (9*a - 27))/t*a*b^2*c^7 + (1/2*(-a + 3)*t^2 + 34*t + 1/2*(-3*a +
    7))/t*b^3*c^7 + 1/2*(15*a + 45)/t*a^2*c^8 + (8*t - 18)/t*a*b*c^8 + (10*t + 1/2*(-3*a + 9))/t*b^2*c^8 + (t - 3*a 
    - 9)/t*a*c^9 + (t + 3)/t*b*c^9 + 1/2*(a + 3)/t*c^10
*/
// Revert Xfib variable names back to u,v,w to prevent a clash with a=K1.1
P2<u,v,w> := ProjectiveSpace(K,2);
Xfib := Curve(P2,
   u^6*v^4 + u^5*v^5 + 1/2*(3*a + 7)*t*v^10 + 1/2*(-3*a + 7)/t*u^6*v^3*w - 7*u^4*v^5*w + u^3*v^6*w - u^2*v^7*w
    + u*v^8*w + 1/2*(15*a + 37)*t*v^9*w + 1/2*(-3*a + 9)/t*u^6*v^2*w^2 + (t + (9*a - 21))/t*u^5*v^3*w^2 - 
    15*u^4*v^4*w^2 + 16*u^3*v^5*w^2 - 4*u^2*v^6*w^2 + 8*u*v^7*w^2 + (15*a + 42)*t*v^8*w^2 + 3/t*u^6*v*w^3 +
    (9*a - 27)/t*u^5*v^2*w^3 + (-7*t + 1/2*(-45*a + 105))/t*u^4*v^3*w^3 + 34*u^3*v^4*w^3 - 11*u^2*v^5*w^3 +
    21*u*v^6*w^3 + (1/2*(29*a + 109)*t + 1)*v^7*w^3 + 1/2*(a + 3)/t*u^6*w^4 - 18/t*u^5*v*w^4 + 1/2*(-45*a +
    135)/t*u^4*v^2*w^4 + (16*t + (30*a - 70))/t*u^3*v^3*w^4 - 17*u^2*v^4*w^4 + 23*u*v^5*w^4 + (1/2*(11*a + 
    91)*t + 10)*v^6*w^4 + (-3*a - 9)/t*u^5*w^5 + 45/t*u^4*v*w^5 + (t + (30*a - 90))/t*u^3*v^2*w^5 + (-11*t 
    + 1/2*(-45*a + 105))/t*u^2*v^3*w^5 + 16*u*v^4*w^5 + (1/2*(-3*a + 51)*t + 34)*v^5*w^5 + 1/2*(15*a + 
    45)/t*u^4*w^6 - 60/t*u^3*v*w^6 + (-4*t + 1/2*(-45*a + 135))/t*u^2*v^2*w^6 + (23*t + (9*a - 
    21))/t*u*v^3*w^6 + ((-2*a + 9)*t + 51)*v^4*w^6 + (-10*a - 30)/t*u^3*w^7 + (-t + 45)/t*u^2*v*w^7 + (21*t
    + (9*a - 27))/t*u*v^2*w^7 + (1/2*(-a + 3)*t^2 + 34*t + 1/2*(-3*a + 7))/t*v^3*w^7 + 1/2*(15*a + 
    45)/t*u^2*w^8 + (8*t - 18)/t*u*v*w^8 + (10*t + 1/2*(-3*a + 9))/t*v^2*w^8 + (t - 3*a - 9)/t*u*w^9 + (t +
    3)/t*v*w^9 + 1/2*(a + 3)/t*w^10 );

//  2-section of Xfib coming from L6. Computed as a degree 2 (over K) scheme-theoretic point on Xfib <= P2
// using subst:
sec2 := Scheme(P2,[ u^2 + (2*t - a + 1)/(t + 1/2*(a - 1))*u*w + w^2, v + w]);

/*****************
  
Now Xfib has no non-singular K-rational points (the fibration has no section), but we want to
convert it to a y^2=quartic, degree 2 homogeneous space model using sec2.

IDEA: Compute the degree 2 map of the desingularisation of Xfib to P1 (ramified at 4 pts)
corresponding to sec2 (as a degree 2 divisor). Then the y^2=quartic model follows from some
computations with differentials in the usual way.

Hardest bit is to find the explicit degree 2 cover of P^1.

We do this by first birationally mapping Xfib to a non-singular model E9 in P^9

Xfib turns out to only have only ordinary singularities : 2 of multiplicity 4 at (1 : -1 : 1) and(1 : 0 : 0) and  
23  conjugate nodes/simple cusps. The divisor map into P^9 comes from an adjoint ideal computation
with vanishing conditions on the singularities. As Degree(Xfib)=10, the adjoint ideal Iadj is generated
by polynomials that vanish at the 23 conjugate sings and also to order >= 3 at the two others.

The degree 8=(10-3)+1 homogeneous polynomials in Iadj give a full Riemann-Roch basis for the
adjoint divisor map K+H, where H is the degree 10 hyperplane Cartier divisor class on the
desing. of Xfib. As K=0 (genus 1), K+H=H and the divisor map into P^9 has image a degree 
10 (projectively normal) model of the desing. of Xfib.

The degree 2 sec2 map then comes from a Riemann-Roch computation on E9. We do this more
efficiently than using the built-in Magma functionality.

Note that the (unique up to scalar) homogeneous degree 7=10-3 polynomial in Iadj
corresponds to the everywhere holomorphic differential on desing.(Xfib) in the same
way and will be used in the next stage.

*********************/

//sng := SingularSubscheme(Xfib);
//Degree(sng); - this is VERY slow. Result is 41.
//Points(sng);
// -> {@ (1 : -1 : 1), (1 : 0 : 0) @}

/* Now must get the reduced subscheme of sng using ReducedSubscheme(sng).
   That was a REALLY long computation, taking about 7 hours [24827.180s].
   Once we have the result, the rest is pretty quick though!
*/

//defining equations for the reduced singular subscheme. it is degree 25
sng_eqns := [u*v^6 + 1/2*(3*a - 7)/t*u^3*v^3*w + ((-a + 4)*t + (a - 2))/t*u*v^5*w + (1/4*(-3*a - 1)*t + 1/4*(3*a - 
    7))/t*v^6*w + (1/4*(a - 1)*t + 1/4*(3*a - 7))/t^2*u^5*w^2 + (1/2*(-a + 3)*t - 4*a + 9)/t^2*u^4*v*w^2 + 
    (1/4*(5*a - 15)*t + 1/4*(37*a - 83))/t^2*u^3*v^2*w^2 + (1/4*(a - 1)*t^2 + 1/2*(-11*a + 28)*t + 1/4*(5*a
    - 11))/t^2*u^2*v^3*w^2 + (1/4*(-11*a + 23)*t^2 + 1/2*(12*a - 27)*t + 1/4*(21*a - 47))/t^2*u*v^4*w^2 + 
    (1/4*(-9*a + 13)*t + 1/4*(5*a - 13))/t*v^5*w^2 + (1/4*(-5*a + 5)*t + 1/4*(-15*a + 35))/t^2*u^4*w^3 + 
    ((3*a - 9)*t + (24*a - 54))/t^2*u^3*v*w^3 + (1/2*(a - 2)*t^2 + 1/4*(-29*a + 87)*t + 1/4*(-83*a + 
    187))/t^2*u^2*v^2*w^3 + (1/2*(-4*a + 5)*t^2 + 1/2*(31*a - 78)*t + 1/2*(37*a - 83))/t^2*u*v^3*w^3 + 
    (1/4*(-9*a + 37)*t^2 + 1/2*(-17*a + 38)*t + 1/4*(-21*a + 47))/t^2*v^4*w^3 + ((3*a - 4)*t + 1/2*(23*a - 
    53))/t^2*u^3*w^4 + (1/4*(-29*a + 91)*t + 1/4*(-151*a + 341))/t^2*u^2*v*w^4 + (1/4*(-3*a + 3)*t^2 + 
    1/2*(16*a - 57)*t + 1/4*(181*a - 407))/t^2*u*v^2*w^4 + (1/4*(a + 25)*t^2 + 1/2*(-53*a + 125)*t + 
    1/4*(-79*a + 177))/t^2*v^3*w^4 + (1/2*(-8*a + 15)*t + 1/2*(-30*a + 69))/t^2*u^2*w^5 + (1/2*(-a + 2)*t^2
    + 1/2*(5*a - 30)*t + 1/2*(81*a - 183))/t^2*u*v*w^5 + (1/2*(3*a + 1)*t^2 + 1/4*(-75*a + 195)*t + 
    1/4*(-135*a + 303))/t^2*v^2*w^5 + (1/2*(4*a - 11)*t + 1/2*(24*a - 55))/t^2*u*w^6 + (1/4*(5*a - 7)*t^2 +
    (-3*a + 12)*t + 1/4*(-91*a + 205))/t^2*v*w^6 + (1/2*(a - 2)*t^2 + 1/2*a*t + 1/2*(-11*a + 25))/t^2*w^7,
v^7 + 1/2*(-3*a + 7)/t*u^3*v^3*w + 1/2*(-5*a + 11)/t*u*v^5*w + ((-a + 5)*t + 1/4*(-3*a + 7))/t*v^6*w + 
    1/4*(-3*a + 7)/t^2*u^5*w^2 + (4*a - 9)/t^2*u^4*v*w^2 + (1/2*(-a + 3)*t + 1/4*(-37*a + 
    83))/t^2*u^3*v^2*w^2 + (1/4*(13*a - 33)*t + 1/4*(-5*a + 11))/t^2*u^2*v^3*w^2 + (1/4*(-53*a + 121)*t + 
    1/4*(-21*a + 47))/t^2*u*v^4*w^2 + (1/2*(-7*a + 21)*t + 1/4*(-5*a + 13))/t*v^5*w^2 + 1/4*(15*a - 
    35)/t^2*u^4*w^3 + (-24*a + 54)/t^2*u^3*v*w^3 + (1/2*(4*a - 13)*t + 1/4*(83*a - 187))/t^2*u^2*v^2*w^3 + 
    (1/2*(-50*a + 119)*t + 1/2*(-37*a + 83))/t^2*u*v^3*w^3 + ((-4*a + 10)*t^2 + 1/4*(33*a - 73)*t + 
    1/4*(21*a - 47))/t^2*v^4*w^3 + 1/2*(-23*a + 53)/t^2*u^3*w^4 + (1/2*(a - 3)*t + 1/4*(151*a - 
    341))/t^2*u^2*v*w^4 + (1/4*(-53*a + 141)*t + 1/4*(-181*a + 407))/t^2*u*v^2*w^4 + (1/2*(-3*a + 7)*t^2 + 
    1/4*(93*a - 215)*t + 1/4*(79*a - 177))/t^2*v^3*w^4 + 1/2*(30*a - 69)/t^2*u^2*w^5 + (1/2*(-5*a + 16)*t +
    1/2*(-81*a + 183))/t^2*u*v*w^5 + (1/2*(27*a - 67)*t + 1/4*(135*a - 303))/t^2*v^2*w^5 + (1/2*(-a + 3)*t 
    + 1/2*(-24*a + 55))/t^2*u*w^6 + (1/4*(-a - 5)*t + 1/4*(91*a - 205))/t^2*v*w^6 + (1/2*(-3*a + 6)*t + 
    1/2*(11*a - 25))/t^2*w^7,
u^6*w + t*u^3*v^3*w + 1/2*(a + 1)*t*u*v^5*w + 1/4*(a + 3)*t*v^6*w + 1/4*(a - 13)*u^5*w^2 + 1/2*(-a - 
    3)*u^4*v*w^2 + (1/2*(-a + 3)*t + 1/4*(-5*a + 19))*u^3*v^2*w^2 + (1/4*(-3*a - 5)*t + 1/4*(5*a - 
    15))*u^2*v^3*w^2 + (1/4*(9*a + 3)*t + 1/4*(-a + 3))*u*v^4*w^2 + 1/4*(5*a + 23)*t*v^5*w^2 + 1/4*(-9*a + 
    9)*u^4*w^3 + 14*u^3*v*w^3 + (1/2*(-a - 2)*t + 1/4*(23*a - 87))*u^2*v^2*w^3 + (1/2*(8*a - 3)*t + 
    1/2*(-7*a + 21))*u*v^3*w^3 + (1/4*(9*a + 71)*t + 1/4*(a - 3))*v^4*w^3 + 1/2*(11*a + 15)*u^3*w^4 + 
    (1/2*(a - 3)*t + 1/4*(13*a - 147))*u^2*v*w^4 + (1/4*(13*a - 5)*t + 1/4*(-37*a + 135))*u*v^2*w^4 + 
    (1/4*(9*a + 113)*t + 1/4*(9*a - 27))*v^3*w^4 + 1/2*(-12*a - 37)*u^2*w^5 + (5/2*t + 1/2*(-11*a + 
    81))*u*v*w^5 + (23*t + 1/4*(19*a - 67))*v^2*w^5 + (1/2*(-a + 3)*t + 1/2*(6*a + 33))*u*w^6 + (1/4*(-7*a 
    + 45)*t + 1/4*(11*a - 65))*v*w^6 + (1/2*(-a + 4)*t + 1/2*(-a - 11))*w^7,
u^5*v + 1/2*(-a - 3)*t*u*v^5 + 1/2*(2*a + 5)*t*v^6 + 1/2*u^5*w + 1/2*(a - 9)*u^4*v*w + 1/4*(-3*a - 
    1)*u^3*v^2*w + (1/2*t + 1/4*(3*a - 3))*u^2*v^3*w + (1/2*(-2*a - 9)*t + 1/4*(-a + 1))*u*v^4*w + 1/2*(8*a
    + 23)*t*v^5*w - 5/2*u^4*w^2 + (-3*a + 7)*u^3*v*w^2 + (1/4*(-a + 3)*t + 1/2*(8*a - 1))*u^2*v^2*w^2 + 
    (1/4*(-a - 21)*t + 1/2*(-5*a + 5))*u*v^3*w^2 + (1/2*(11*a + 42)*t + 1/4*(a - 1))*v^4*w^2 + 1/2*(-a + 
    9)*u^3*w^3 + 1/4*(29*a - 17)*u^2*v*w^3 + (1/2*(a - 6)*t + 1/4*(-29*a + 13))*u*v^2*w^3 + (1/4*(11*a + 
    77)*t + 1/4*(7*a - 7))*v^3*w^3 + 1/4*(7*a - 13)*u^2*w^4 + (1/4*(a - 3)*t + 1/2*(-17*a + 3))*u*v*w^4 + 
    (10*t + 1/2*(8*a - 5))*v^2*w^4 + 1/4*(-9*a + 3)*u*w^5 + (1/4*(-3*a + 15)*t + 1/4*(15*a - 3))*v*w^5 + 
    (1/4*(-a + 3)*t + a)*w^6,
u^4*v^2 + 1/4*(7*a + 15)*t*v^6 + 1/4*(-a - 1)*u^5*w - u^4*v*w + 1/2*(a - 4)*u^3*v^2*w + (1/4*(-a - 1)*t + 
    1/2*(-a - 6))*u^2*v^3*w + (1/4*(3*a + 7)*t + 1/2*(a + 4))*u*v^4*w + (1/4*(9*a + 17)*t - 1)*v^5*w + 
    1/4*(5*a + 5)*u^4*w^2 + 5*u^3*v*w^2 + (1/4*(-a + 1)*t + 1/4*(-11*a - 19))*u^2*v^2*w^2 + (1/4*(5*a + 
    11)*t + (3*a + 14))*u*v^3*w^2 + (1/4*(-7*a - 19)*t + 1/2*(-a - 14))*v^4*w^2 + 1/2*(-5*a - 3)*u^3*w^3 + 
    1/2*(-2*a - 21)*u^2*v*w^3 + (1/4*(3*a + 3)*t + 1/2*(14*a + 47))*u*v^2*w^3 + (1/2*(-8*a - 17)*t + 
    1/2*(-5*a - 42))*v^3*w^3 + 1/4*(9*a - 1)*u^2*w^4 + (1/4*(a - 1)*t + (4*a + 18))*u*v*w^4 + (1/2*(-5*a - 
    9)*t + 1/4*(-19*a - 111))*v^2*w^4 + 1/4*(-a + 13)*u*w^5 + (1/2*(-2*a - 1)*t + 1/2*(-6*a - 33))*v*w^5 + 
    (1/4*(-a + 1)*t + 1/2*(-a - 7))*w^6,
u^2*v^4 + 1/2*(-a - 1)*u*v^5 + 1/2*v^6 + 1/2/t*u^5*w + 1/2*(a - 3)/t*u^4*v*w + 1/4*(-5*a + 13)/t*u^3*v^2*w 
    + (1/2*(-a + 4)*t + 1/4*(-a + 1))/t*u^2*v^3*w + (1/2*(-3*a - 2)*t + 1/4*(-3*a + 7))/t*u*v^4*w + 1/2*(a 
    + 6)*v^5*w - 5/2/t*u^4*w^2 + (-3*a + 9)/t*u^3*v*w^2 + (1/4*(-a + 3)*t + 1/2*(5*a - 16))/t*u^2*v^2*w^2 +
    (1/4*(-5*a - 5)*t + 1/2*(-5*a + 13))/t*u*v^3*w^2 + (1/2*(4*a + 9)*t + 1/4*(3*a - 7))/t*v^4*w^2 + 
    1/2*(-a + 13)/t*u^3*w^3 + 1/4*(17*a - 61)/t*u^2*v*w^3 + (-3/2*t + 1/4*(-23*a + 67))/t*u*v^2*w^3 + 
    (1/4*(13*a + 11)*t + 1/4*(11*a - 27))/t*v^3*w^3 + 1/4*(3*a - 33)/t*u^2*w^4 + (1/4*(a - 3)*t + 1/2*(-9*a
    + 33))/t*u*v*w^4 + (1/2*(3*a + 5)*t + 1/2*(9*a - 24))/t*v^2*w^4 + 1/4*(-3*a + 25)/t*u*w^5 + (1/4*(-a + 
    9)*t + 1/4*(11*a - 35))/t*v*w^5 + (1/4*(-a + 3)*t + 1/2*(a - 5))/t*w^6
];

sng := Scheme(P2,sng_eqns);
sng1 := Scheme(P2,ColonIdeal(Ideal(sng),v));
sng2 := Scheme(P2,ColonIdeal(Ideal(sng1),v+w));//irreducible deg 23 node/simple cusp

R2 :=  CoordinateRing(P2);
Iadj := Ideal(sng2) meet ideal<R2|[v,w]>^3 meet ideal<R2|[u+v,w+v]>^3;
B := MinimalBasis(Iadj);
B1 := [B[1]*u,B[1]*v,B[1]*w] cat B[2..8]; // B[1] has degree 7 and B[2],..B[8] have degree 8
P := B[1];

// B1 defines the correct divisor map into P^9.

// Now must compute the image, E9. We use a speed up of the general Magma image machinery,
// knowing that the ideal of the image (a projective normal genus 1 curve) is generated
// by quadrics.

P9<[x]> := ProjectiveSpace(K,9);
R := CoordinateRing(P9);
mons := Setseq(MonomialsOfDegree(R,2));
time B2s := [NormalForm(Evaluate(m,B1),Ideal(Xfib)) : m in mons];
mons1 := Setseq(&join([Seqset(Monomials(e)) : e in B2s]));
mat := Matrix([[MonomialCoefficient(b,m) : m in mons1] : b in B2s]);
time K := Kernel(mat);
pols := [Polynomial(Eltseq(e),mons) : e in Basis(K)];

E9 := Scheme(P9,pols); // This is the required image
// image of section sec2:
s9 := Scheme(P9,[x[9]^2 + (1/11*(16*a + 2)*t^2 + 1/121*(-535*a + 1117)*t + 1/121*(-119*a + 267))/(t^2 + 1/242*(271*a - 
    545)*t + 1/242*(-119*a + 267))*x[9]*x[10] + (1/121*(16*a + 321)*t^2 + 1/242*(2071*a - 4665)*t + 
    1/242*(-119*a + 267))/(t^2 + 1/242*(271*a - 545)*t + 1/242*(-119*a + 267))*x[10]^2,
x[8] + (1/179*(-54*a + 170)*t^2 + 1/716*(903*a - 2087)*t + 1/716*(-261*a + 583))/(t^2 + 1/358*(205*a - 
    473)*t)*x[9] + (1/179*(-44*a + 291)*t^2 + 1/716*(317*a - 693)*t + 1/716*(-261*a + 583))/(t^2 + 
    1/358*(205*a - 473)*t)*x[10],
x[7] + (1/179*(-83*a + 195)*t + 1/179*(211*a - 472))/(t^2 + 1/358*(205*a - 473)*t)*x[9] + (1/179*(-61*a + 
    139)*t + 1/179*(211*a - 472))/(t^2 + 1/358*(205*a - 473)*t)*x[10],
x[6] + (1/358*(-59*a - 159)*t^2 + 1/358*(-5*a + 29)*t)/(t + 1/358*(205*a - 473))*x[9] + (1/358*(-121*a - 
    229)*t^2 + 1/358*(-5*a + 29)*t)/(t + 1/358*(205*a - 473))*x[10],
x[5] + (1/358*(8*a + 61)*t + 1/716*(39*a - 83))/(t + 1/358*(205*a - 473))*x[9] + (1/358*(-20*a - 63)*t + 
    1/716*(39*a - 83))/(t + 1/358*(205*a - 473))*x[10],
x[4] + (1/358*(4*a - 59)*t + 1/716*(-83*a + 195))/(t + 1/358*(205*a - 473))*x[9] + (1/358*(-10*a - 121)*t +
    1/716*(-83*a + 195))/(t + 1/358*(205*a - 473))*x[10],
x[3] + (1/716*(-59*a - 159)*t^2 + 1/358*(8*a + 61)*t)/(t + 1/358*(205*a - 473))*x[9] + (1/716*(-121*a - 
    229)*t^2 + 1/358*(16*a - 57)*t)/(t + 1/358*(205*a - 473))*x[10],
x[2] + (1/716*(59*a + 159)*t^2 + 1/358*(-8*a - 61)*t)/(t + 1/358*(205*a - 473))*x[9] + (1/716*(121*a + 
    229)*t^2 + 1/358*(-16*a + 57)*t)/(t + 1/358*(205*a - 473))*x[10],
x[1] + (1/716*(59*a + 159)*t^2 + 1/358*(13*a + 32)*t)/(t + 1/358*(205*a - 473))*x[9] + (1/716*(121*a + 
    229)*t^2 + 1/358*(21*a - 86)*t)/(t + 1/358*(205*a - 473))*x[10] ]);

/****
To compute the Riemann-Roch space of s9 as a degree 2 divisor on E9, we take a hyperplane section H9
of E9 passing through s9. For the linear equation of H9, we take the 2nd defining polynomial of
s9.

Then H9cs9=H9\s9 is a degree 8 divisor on E9 and the hyperplanes passing through H9s9 define the
Rimann-Roch space of s9. This is computed directly through colon ideal operations.
*******/

Hpol := DefiningPolynomials(s9)[2];
I1 := ideal<R|Hpol>+Ideal(E9);
/****
  The colon ideal computation. This does take a while to run (about 3 hours on the machine I'm using),
  so I've commented this bit out and I will just give the result below for the final basis B.
  It can be checked that
  Hyperplane(B[1]) meet Hyperplane(B[2]) meet E9 DOES have degree 8 and that Hyperplane(B[2])
  doesn't meet s9, so B[2]/B[1] is a non-constant rational function with polar divisor -s9.
time I2 := &meet[ColonIdeal(I1,f) : f in DefiningPolynomials(s9) | f ne dp];
//Time: 10014.080
mb := MinimalBasis(I2);
B := [b : b in mb | TotalDegree(b) eq 1];
********/
  
// For RR space on E9 for divisor s9 get
B := [ x[8] + (1/179*(-54*a + 170)*t^2 + 1/716*(903*a - 2087)*t + 1/716*(-261*a + 583))/(t^2 + 1/358*(205*a - 
        473)*t)*x[9] + (1/179*(-44*a + 291)*t^2 + 1/716*(317*a - 693)*t + 1/716*(-261*a + 583))/(t^2 + 
        1/358*(205*a - 473)*t)*x[10],
    x[1] + (1/2*(-71*a + 163)*t^2 + 1/2*(295*a - 660)*t + 1/4*(-233*a + 521))/(t^2 + 1/2*(109*a - 244)*t + 
        1/4*(-233*a + 521))*x[2] + ((-123*a + 276)*t^2 + 1/4*(965*a - 2157)*t + 1/4*(2639*a - 5901))/(t^2 +
        1/2*(109*a - 244)*t + 1/4*(-233*a + 521))*x[3] + ((-136*a + 290)*t^3 + (119*a - 269)*t^2 + 
        1/4*(-481*a + 1077)*t + 1/4*(89*a - 199))/(t^2 + 1/2*(109*a - 244)*t + 1/4*(-233*a + 521))*x[4] + 
        ((89*a - 197)*t^2 + 1/4*(-2599*a + 5811)*t + 1/4*(-1597*a + 3571))/(t^2 + 1/2*(109*a - 244)*t + 
        1/4*(-233*a + 521))*x[5] + (1/2*(235*a - 527)*t^3 + 1/2*(-314*a + 703)*t^2 + (288*a - 644)*t + 
        1/4*(-1597*a + 3571))/(t^3 + 1/2*(109*a - 244)*t^2 + 1/4*(-233*a + 521)*t)*x[6] + (1/2*(-9*a + 
        1)*t^3 + 1/4*(153*a - 341)*t^2 + 1/4*(-21*a + 47)*t)/(t^2 + 1/2*(109*a - 244)*t + 1/4*(-233*a + 
        521))*x[7] + (1/179*(5394*a - 11810)*t^4 + 1/358*(-47337*a + 105686)*t^3 + 1/1432*(210511*a - 
        470739)*t^2 + 1/179*(-18935*a + 42340)*t + 1/716*(16042*a - 35871))/(t^3 + 1/358*(19716*a - 
        44149)*t^2 + 1/179*(-35821*a + 80099)*t + 1/716*(108507*a - 242629))*x[9] + (1/358*(14737*a - 
        30987)*t^4 + 1/358*(-38427*a + 86407)*t^3 + 1/1432*(398347*a - 890871)*t^2 + 1/716*(-211811*a + 
        473625)*t + 1/716*(16042*a - 35871))/(t^3 + 1/358*(19716*a - 44149)*t^2 + 1/179*(-35821*a + 
        80099)*t + 1/716*(108507*a - 242629))*x[10]
];
den := B[1];
Bs := [Evaluate(b,B1) : b in B];// pull back to Xfib

// Now, given the degree 2 rational function X1=Bs[1]/Bs[2] on Xfib, use a differentials computation
// on Xfib to compute a Y function s.t. Y^2=quartic in X1 is the required equation. Uses the fact
// that a holomorphic non-vanishing differential on Xfib is dX1/Y, on the one hand, but can
// be computed explicitly on the Xfib model on the other.

/******************* Function to get Y^2=Q(X1) where Q is a quartic poly for a 
   (singular model of) genus 1 curve in P2 C, given X as a rational function B[1]/B[2]
   on C [B= sequence of 2 homog polys on P2 of same degree] and a homog poly P of degree
   Deg(C)-3 s.t. P*(canonical form on C) is the everywhere defined differential (up to scalar)
***********************************************/


function getYnEqn(C,B,P)
  F := Equation(C);
  Yn := (B[2]*Derivative(B[1],1)-B[1]*Derivative(B[2],1))*Derivative(F,2)-
	(B[2]*Derivative(B[1],2)-B[1]*Derivative(B[2],2))*Derivative(F,1);
  E := P*(C.3);
  Yd :=E*B[2]^2;
  // Y = Yn/Yd
  // get a0,..,a4 coeffs of Y^2=a4*X^4+a3*X^3+..a0 eqn.
  I := Ideal(C);
  lhs := NormalForm(Yn^2,I);
  rhs := [NormalForm(E^2*B[2]^(4-i)*B[1]^i,I) : i in [0..4]];
  mons := Setseq(&join[Seqset(Monomials(e)) : e in [lhs] cat rhs]);
  mat := Matrix([[MonomialCoefficient(b,m) : m in mons] : b in rhs]);
  vec := Vector([MonomialCoefficient(lhs,m) : m in mons]);
  soln := Solution(mat,vec);
  return Eltseq(soln),Yn,Yd;
end function;

// Get the Y function and coefficients of the quartic poly for Y^2=Q(X1)
// where X1=Bs[2]/Bs[1]
time cs,Yn,Yd := getYnEqn(Xfib,Reverse(Bs),P);
// Time: 1154.370

// Y=Yn/Yd as a rational function on Xfib and cs are the coefficients of Q.

//transform to simpler
al := ((64+26*a)/179)*(t^2 + 1/2*(109*a - 244)*t + 1/4*(-233*a + 521))/(t^3 + 1/358*(205*a - 473)*t^2);
// Y -> Y/al gives new cs
cs := [c/al^2 : c in cs];
cs;
/*
[
    (1/2*(-12559933*a + 28082669)*t^13 + (27904810*a - 62399044)*t^12 + 1/4*(-310915091*a + 695227261)*t^11
        + 1/8*(897429833*a - 2006713987)*t^10 + 1/32*(-2350993161*a + 5256980503)*t^9 + 1/4*(19147095*a - 
        42814206)*t^8 + 1/16*(-811180843*a + 1813855507)*t^7 + 1/8*(1597519007*a - 3572161095)*t^6 + 
        1/16*(-31622993*a + 70711162)*t^5)/(t^8 + (218*a - 488)*t^7 + 1/2*(-160042*a + 357865)*t^6 + 
        1/2*(26284340*a - 58773571)*t^5 + 1/16*(-13319991480*a + 29784406409)*t^4 + 1/8*(27388383183*a - 
        61242286591)*t^3 + 1/16*(-86884586551*a + 194279841725)*t^2 + 1/4*(15423526950*a - 34488054713)*t +
        1/32*(-32951280099*a + 73681302247)),
    ((-305750*a + 684538)*t^10 + 1/2*(2075397*a - 4639101)*t^9 + (-2714603*a + 6070000)*t^8 + 
        1/4*(17652361*a - 39471887)*t^7 + 1/2*(-10526810*a + 23538663)*t^6 + 1/4*(17204125*a - 
        38469593)*t^5 + 1/4*(-514229*a + 1149851)*t^4)/(t^6 + 1/2*(327*a - 732)*t^5 + 1/4*(-160275*a + 
        358386)*t^4 + 1/8*(26625263*a - 59535898)*t^3 + 1/16*(-167007213*a + 373439481)*t^2 + 
        1/16*(177621537*a - 397173831)*t + 1/8*(-31622993*a + 70711162)),
    ((-5646*a + 12372)*t^7 + 1/2*(23663*a - 53085)*t^6 + 1/4*(-88843*a + 198685)*t^5 + 1/2*(39297*a - 
        87871)*t^4 + 1/4*(-12543*a + 28047)*t^3)/(t^4 + (109*a - 244)*t^3 + 1/4*(-53658*a + 119983)*t^2 + 
        1/4*(113641*a - 254109)*t + 1/8*(-121393*a + 271443)),
    ((-38*a + 118)*t^4 + (41*a - 91)*t^3 + (-34*a + 76)*t^2)/(t^2 + 1/2*(109*a - 244)*t + 1/4*(-233*a + 
        521)),
    1/2*(-a - 1)*t
];
*/

// transform to parameter s=-(1/2)(a+1)t [t =-(1/2)(a-1)s] and call that t again then get
// find that the generic fibre is isomorphic to its transform under t -> 1/t
cs1 := [
    ((-69643152*a + 155726921)*t^13 + (191265401*a - 427682729)*t^12 + 1/4*(-1317057443*a + 
        2945029977)*t^11 + 1/8*(2349501743*a - 5253645563)*t^10 + 1/16*(-1901993416*a + 4252986577)*t^9 + 
        1/4*(19147095*a - 42814206)*t^8 + 1/8*(-250668666*a + 560512177)*t^7 + 1/8*(610197963*a - 
        1364444125)*t^6 + 1/16*(-7465176*a + 16692641)*t^5)/(t^8 + (135*a - 301)*t^7 + 1/4*(-122261*a + 
        273385)*t^6 + 1/2*(6204891*a - 13874558)*t^5 + 1/32*(-3886721133*a + 8690972663)*t^4 + 
        1/8*(2469608971*a - 5522213537)*t^3 + 1/16*(-4841912059*a + 10826844505)*t^2 + 1/8*(1062429719*a - 
        2375665073)*t + 1/32*(-701408733*a + 1568397607)),
    ((-2096932*a + 4689008)*t^10 + 1/2*(8789895*a - 19655187)*t^9 + 1/2*(-14213809*a + 31783015)*t^8 + 
        1/2*(14281062*a - 31933423)*t^7 + 1/2*(-10526810*a + 23538663)*t^6 + 1/2*(5316367*a - 11887758)*t^5
        + 1/2*(-98209*a + 219602)*t^4)/(t^6 + 1/4*(405*a - 903)*t^5 + 1/8*(-122439*a + 273783)*t^4 + 
        1/8*(6285372*a - 14054519)*t^3 + 1/8*(-12183012*a + 27242043)*t^2 + 1/8*(8008062*a - 17906571)*t + 
        1/8*(-1762289*a + 3940598)),
    ((-23664*a + 52974)*t^7 + 1/2*(62037*a - 138785)*t^6 + 1/2*(-71882*a + 160725)*t^5 + 1/2*(39297*a - 
        87871)*t^4 + 1/2*(-3876*a + 8667)*t^3)/(t^4 + 1/2*(135*a - 301)*t^3 + 1/8*(-40991*a + 91659)*t^2 + 
        1/4*(26827*a - 59987)*t + 1/8*(-17711*a + 39603)),
    ((-116*a + 272)*t^4 + (66*a - 148)*t^3 + (-34*a + 76)*t^2)/(t^2 + 1/4*(135*a - 301)*t + 1/4*(-89*a + 
        199)),
    t
];

// Find bad fibres (not over 0 and infinity, where we already know the curve has II* bad fibres)
// of y^2=Q(x) where Q=PolynomialRing(K!cs1 [the quartic poly with coefficients cs1] by base changing
// to Q(s) where t=s^2, where a point exists, and then trasforming to
// a Weierstarss equation using a (now rational) point at infinity as a O-point.
// Rather than introducing s, will just replace t by t^2
C_s := HyperellipticCurve(PolynomialRing(K)![Evaluate(c,t^2) : c in cs1]);
pt := C_s![1,t,0];
LocalInformation(E_s);
/*
[ <(t^8 + 1/27*(-1118*a - 2598)*t^6 + 1/27*(-89700*a - 200362)*t^4 + 1/27*(-1118*a - 2598)*t^2 + 1), 1, 1, 1, I1, 
false>, <(t), 8, 2, 3, IV*, true>, <(1/t), 8, 2, 3, IV*, true> ]
*/

/*
  So, our original fibration has bad fibres as follow:
    over t=infty,0     :  II*
    over t^4+1/27*(-1118*a - 2598)*t^3+1/27*(-89700*a - 200362)*t^2+1/27*(-1118*a - 2598)*t+1=0 : I1
*/

// Transform further to get a nice equation:

// under x1=x*(t^2 + 1/4*(135*a - 301)*t + 1/4*(-89*a + 199))/t
//      y1=y*(t^2 + 1/4*(135*a - 301)*t + 1/4*(-89*a + 199))^2/t^3
// get simpler equation: t*y1^2=H1(x1) where H1 has coeffs cs1:
cs1 := [
    (-69643152*a + 155726921)*t^8 + (191265401*a - 427682729)*t^7 + 1/4*(-1317057443*a + 2945029977)*t^6 + 
        1/8*(2349501743*a - 5253645563)*t^5 + 1/16*(-1901993416*a + 4252986577)*t^4 + 1/4*(19147095*a - 
        42814206)*t^3 + 1/8*(-250668666*a + 560512177)*t^2 + 1/8*(610197963*a - 1364444125)*t + 1/16*(-7465176*a +
        16692641),
    (-2096932*a + 4689008)*t^6 + 1/2*(8789895*a - 19655187)*t^5 + 1/2*(-14213809*a + 31783015)*t^4 + 
        1/2*(14281062*a - 31933423)*t^3 + 1/2*(-10526810*a + 23538663)*t^2 + 1/2*(5316367*a - 11887758)*t + 
        1/2*(-98209*a + 219602),
    (-23664*a + 52974)*t^4 + 1/2*(62037*a - 138785)*t^3 + 1/2*(-71882*a + 160725)*t^2 + 1/2*(39297*a - 87871)*t + 
        1/2*(-3876*a + 8667),
    (-116*a + 272)*t^2 + (66*a - 148)*t - 34*a + 76,
    1
];
/**********

t*y^2 = Q(x) for a monic quartic Q, is the form of the equations for the fibration given in
Section 4 of the paper. Q is the H1 here.

**********/

////////////////////////////////////// Shioda-Inose stuff ///////////////////////////////

/*
  Now we compute the Nikulin involution w (on the generic fibre of the fibration at least) that swaos the
  two II* fibres and s.t. the quotient S/w is a (non-split) Kummer surface, as described in the last
  part of Section 4 of the paper.
*/

// w will induce t -> 1/t on the base P1 of the phi:S -> P1 fibration map.
// To see that such an involution exists, we must find one on the generic fibre
// of phi (which will then extend to all of S, as it is the Minimal model of
// its generic fibre).

// We go back to work with the y^2=Q1(x) model.
R := PolynomialRing(K);
H1 := HyperellipticCurve(R![t*c: c in cs1]);
H2 := HyperellipticCurve(R![Evaluate(t*e,1/t) : e in cs1]);
time boo,im := IsIsomorphic(H1,H2);
//Time: 3.450
boo;
//true;
im;
/*
Mapping from: CrvHyp: H1 to CrvHyp: H2
with equations : 
(1/179*(67*a - 138)*t + 1)/(t^3 + 1/179*(67*a - 138)*t^2)*$.1 + (1/358*(20159*a - 45000)*t^3 + 1/358*(-30693*a + 
    69796)*t^2 + 1/358*(30693*a - 69796)*t + 1/358*(-20159*a + 45000))/(t^3 + 1/179*(67*a - 138)*t^2)*$.3
(1/32041*(-18492*a + 41489)*t^2 + 1/179*(134*a - 276)*t + 1)/(t^7 + 1/179*(134*a - 276)*t^6 + 1/32041*(-18492*a + 
    41489)*t^5)*$.2
$.3
and inverse
$.1 + (1/358*(-20159*a + 45000)*t^3 + 1/358*(30693*a - 69796)*t^2 + 1/358*(-30693*a + 69796)*t + 1/358*(20159*a - 
    45000))/(t^3 + 1/179*(67*a - 138)*t^2)*$.3
t*$.2
(1/179*(67*a - 138)*t + 1)/(t^3 + 1/179*(67*a - 138)*t^2)*$.3
*/

e := 1/19*(67*a + 138); f := (1/355)*(1324*a + 2770);
al := (t + e)/(t^2*(e*t + 1));
be := 1/38*(-1302*a + 3035)*(t-1)*(t^2 + f*t + 1)/(t^2*(e*t + 1));
E := al^2/t;

// Generic fibre is y^2=t*H1(x) over K1(t) [with renormalised t=s, y=t*y1, x=x1] 
// Nikulin involution, i, is t -> 1/t, x -> al*x+be, y -> -E*y.

/*
  To prove that i genuinely is a Nikulin involution, we must show that it preserves
  the (non-vanishing) holomorphic differential 2-form on S.
  But this is equivalent to saying that i only has finitely many fixed points on S.
  As i is an involution i^*(omega)=+/-omega, for the holomorphic differential 2-form omega,
  and if i^*(omega)=-omega, then it isn't hard to see that, at any fixed point p of i,
  i would have signature (+,-) rather than (-,-) on the tangent space, and then, analytically,
  it would fix a whole curve through p on S(complex numbers) [ apply the same argument along
  the analytic locus].  But the fixed point set is algebraic (since i is), so would have to
  have dimension >=1, contradicting only finitely many fixed points.

  Now, because of the t -> 1/t, i can only fix points on the nonsingular fibres over t=1 and -1,
  and, specialising to those values of t, we get the transforms y -> -y, x-> x at t=1 and
  y -> y, x-> -x+(108*a - 248). There are precisely 4 fixed points of each of these involutions
  on the 2 fibres giving only 8 fixed points (the correct number for a Nikulin inv.!) on S.
  
  Notice that composing i with the y->-y automorphism gives an involution j that will then have
  j^*(omega)=-omega and we notice that j does indeed specialise to the identity on the 
  t=1 fibre, so fixes the whole fibre! [and it actually has no fixed points on the t=-1 fibre].
*/

// Try to find K1(t,x,y)^<i*>. Let ' denote the image under i*.
// s=t+t'=t+(1/t).
// X=x+x', Y := x*x'


// Use: 
/*******************************************
**  Functions to convert f(t) rat fns invt under t -> 1/t to
**   g(s) where s=t+(1/t)********************/

function sn(P,r)
  ss := [P!1,P.1];
  for i in [2..r] do
    si := (P.1)^i-&+[Binomial(i,j)*ss[i+1-2*j] : j in [1..(i div 2)]];
    Append(~ss,si);
  end for;
  return ss;
end function;

function StoP(pol)
   if IsZero(pol) then return pol; end if;
   P := Parent(pol);
   cs := Coefficients(pol);
   r := Min([i : i in [1..#cs] | not IsZero(cs[i])]);
   cs := cs[r..#cs];
   d := (#cs)-1;
   if d eq 0 then return P!(cs[1]); end if;
   assert IsEven(d) and (cs eq Reverse(cs));
   cs := cs[((#cs+1) div 2)..#cs];
   ss := sn(P,d div 2);
   return &+[ss[i]*cs[i] : i in [1..#cs]];
end function;

function sexp(e)
   t := Parent(e).1;
   ed := Evaluate(e,1/t);
   e1 := (e+ed)/2;
   e2 := (e-ed)/2*(t-(1/t));
   return [e1-e2*(t+(1/t)),2*e2];
end function;

// Results of some strightforward computations, using the above gives
// (which can be easily verified by substitution!

L<s> := RationalFunctionField(K1,1);
// Y = c1[1]*v1[1]*X^2+c1[2]*v1[2]*X+c1[3]*v1[3] where
c1 := [ 1/179*(67*a - 138), 1/358*(20159*a - 45000), 1/128164*(1814310000*a - 4056926405) ];
v1 := [
    (s + 1/3401*(13266*a + 22080))/(s^3 + 1/179*(134*a - 276)*s^2 + 1/32041*(5494*a - 104038)*s + 
        1/32041*(-84956*a + 245868)),
    (s^3 + 1/63545*(260781*a + 383295)*s^2 + 1/63545*(-281688*a - 463660)*s + 1/63545*(-479748*a - 1114220))/(s^3 
        + 1/179*(134*a - 276)*s^2 + 1/32041*(5494*a - 104038)*s + 1/32041*(-84956*a + 245868)),
    (s^3 + 1/355*(2648*a + 4830)*s^2 + 1/25205*(1090976*a + 2500876)*s + 1/25205*(-2933984*a - 6575112))/(s^3 + 
        1/179*(134*a - 276)*s^2 + 1/32041*(5494*a - 104038)*s + 1/32041*(-84956*a + 245868))
];

// g = (x-x')(t-(1/t)) : g = u*X+v where
uv := [
    (s + 1/179*(67*a + 41))/(s^2 + 1/179*(67*a + 41)*s + 1/179*(134*a - 634)),
    (1/179*(-20159*a + 45000)*s + 1/179*(10534*a - 24796))/(s^2 + 1/179*(67*a + 41)*s + 1/179*(134*a - 634))
];

// Setting y1=(y+y')*( (s+2)*(s + 1/179*(67*a - 317))^2 )/
//    ( s^3 + 1/179*(134*a - 97)*s^2 + 1/32041*(5494*a - 71997)*s + 1/32041*(-42478*a + 58852) ),
// we find that K1(t,x,y)^<i*> = K1(s,X,y1) and 
//
// y1^2= (s-2)*H3(X) where H3 is the quartic in K1(s)[X] with coefficients
cs3 := 
[
    (-69643152*a + 155726921)*s^8 + (191265401*a - 427682729)*s^7 + 1/4*(592730317*a - 1325388447)*s^6 + 
        1/8*(-6123072939*a + 13691623263)*s^5 + 1/16*(-926904572*a + 2072651317)*s^4 + 1/2*(3543532756*a - 
        7923588921)*s^3 + 1/2*(-351541864*a + 786069199)*s^2 + (-3806809762*a + 8512288536)*s + 3687677148*a - 
        8245897115,
    (-2096932*a + 4689008)*s^6 + 1/2*(8789895*a - 19655187)*s^5 + 1/2*(7349795*a - 16436177)*s^4 + 
        1/2*(-22591507*a + 50517998)*s^3 + (3742435*a - 8367122)*s^2 + (444342*a - 994644)*s + 6693100*a - 
        14966608,
    (-23664*a + 52974)*s^4 + 1/2*(62037*a - 138785)*s^3 + 1/2*(90334*a - 202479)*s^2 + (-54598*a + 122176)*s + 
        15500*a - 34470,
    (-116*a + 272)*s^2 + (66*a - 148)*s + 164*a - 392,
    1
];
// Replacing s-2 by s (so s now = t+(1/t)-2) and multiplying both sides of the equation by s
// gives the s*y1^2=F(x1) quartic equation for the generic fibre of X/<i> mentioned near the 
// end of Section 4 of the paper.

///////// Shioda-Inose form (see also computations for Section 5 and the fuller description in Section 5) //////

/*
transform s=t1^2+2 and t1 = s1-(1/s1) the  can transform above equation
to:
      y^2=x^3-1/3*(132*a + 293)*x+((3*a+7)/2)*(s1^2-((1299+559*a)/27)+s1^-2);
---> F^(2) of Kum(E1xE2) where j(Ei)=ji and
j1,j2 are roots of
  x^2-js*x+jp = 0 where
js = 1435200*a + 3208384  jp = 3967162368*a + 8868902912
--->
j1 = 1604192 + 717600*a + b*(1134016 + 506688a)
j2 = 1604192 + 717600*a - b*(1134016 + 506688a)
 where b = Sqrt(2)
// Curves with jinvs j1 and j2 are related by a cyclic 3-isogeny
*/
